INCLUDE(CMakeForceCompiler)

IF(NOT CMAKE_TOOLCHAIN_FILE)
	MESSAGE(FATAL_ERROR "CMAKE_TOOLCHAIN_FILE not defined")
ENDIF()

SET(CAT_DIR "gcc")  # only for pc tools
SET(ZLIB_FOUND 1)

IF(${CMAKE_BUILD_TYPE} MATCHES "Release")
	SET(CAFE_BUILD_TYPE "NDEBUG")
ELSE()
	SET(CAFE_BUILD_TYPE "DEBUG")
ENDIF()

GET_FILENAME_COMPONENT(
	tmp
	"${NLIB_RELEASE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}"
	ABSOLUTE)
nlib_cygwin_getwinpath(${tmp} NLIB_DIR)
MESSAGE(STATUS "NLIB_DIR=${NLIB_DIR}")
MESSAGE(STATUS "CAFE_BUILD_TYPE=${CAFE_BUILD_TYPE}")

# set CAFE_SDK_LIBS if you have to add other libs
LINK_DIRECTORIES("${CYG_CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE}")

# CMAKE_C_FLAGS_INIT and CMAKE_CXX_FLAGS_INIT in toolchain-cafe.cmake do not work on cmake 3.3.1(cygwin) ?
SET(CMAKE_C_FLAGS "--no_commons -c99 -kanji=utf8 -only_explicit_reg_use --no_exceptions --link_once_templates -cpu=espresso -sda=none -Ogeneral -G --quit_after_warnings --Werror --no_implicit_include -dbg_source_root ${WIN_CAFE_ROOT}/system -MD")
SET(CMAKE_CXX_FLAGS "--no_commons -c99 -kanji=utf8 -only_explicit_reg_use --no_exceptions --g++ --link_once_templates -cpu=espresso -sda=none -Ogeneral -G --quit_after_warnings --Werror --no_implicit_include -dbg_source_root ${WIN_CAFE_ROOT}/system -MD")
SET(CMAKE_CXX_FLAGS_DEBUG "${CMAKE_CXX_FLAGS_DEBUG} -D_DEBUG")
SET(CMAKE_CXX_FLAGS_RELEASE "${CMAKE_CXX_FLAGS_RELEASE} -DNDEBUG")
SET(CMAKE_C_FLAGS_DEBUG "${CMAKE_C_FLAGS_DEBUG} -D_DEBUG")
SET(CMAKE_C_FLAGS_RELEASE "${CMAKE_C_FLAGS_RELEASE} -DNDEBUG")

ADD_DEFINITIONS(
	-DNLIB_CAFE_PPC
	-DNDEV=1
	-DCAFE=2
	-DGHS_DISABLE_NON_CRITICAL_WARNINGS
	-DPLATFORM=CAFE
	-DEPPC
	-DCOS_SECURITY_ENABLED=1
	-DIOS_DEBUG_STARTUP_STATE=FALSE
)

MESSAGE(STATUS "include directories: ${CYG_CAFE_ROOT}/system/include/")
INCLUDE_DIRECTORIES(AFTER ${CYG_CAFE_ROOT}/system/include/)

MACRO(library_default_build_rule target)
	ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	SET_PROPERTY(TARGET ${target} APPEND PROPERTY INTERFACE_INCLUDE_DIRECTORIES "$<BUILD_INTERFACE:${NLIB_PKGROOT_DIR}/include>")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	INSTALL(TARGETS ${target} DESTINATION system/lib/ghs/cafe/${CAFE_BUILD_TYPE})
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDMACRO()

# sdkpad -> -lsdkpad -> ${sdklibdir}/pad.a --> linker
MACRO(target_link_libraries_default target)
	TARGET_LINK_LIBRARIES(${target} 
		sdkpad
		sdkgfd sdkmtx sdkdemo sdkgx2ut sdknn_util sdknn_crypto sdknsyshid sdknlibcurl
		sdknsysnet sdknsysccr sdknsysuvd sdkgx2 sdkavm sdktcl sdktve sdkdc sdksnd_core
		sdksnd_user sdkuvc sdkuvd sdkcamera sdkh264 sdkdmae sdknn_acp sdknn_save sdkvpad
		sdkvpadbase sdkproc_ui sdknn_ac sdkpadscore sdkcoredyn sdkzlib125)
ENDMACRO()


