
IF(DEFINED NLIB_ONCE_COMMON_CMAKE)
  RETURN()
ENDIF()
SET(NLIB_ONCE_COMMON_CMAKE True)

IF(${CMAKE_BUILD_TYPE} MATCHES "^$")
	SET(CMAKE_BUILD_TYPE "Release")
ENDIF()

cmake_policy(SET CMP0022 NEW)
cmake_policy(SET CMP0038 NEW)
cmake_policy(SET CMP0042 NEW)
cmake_policy(SET CMP0054 NEW)

IF(NOT NLIB_PKGROOT_DIR)
	SET(NLIB_PKGROOT_DIR ${NLIB_RELEASE_ROOT})
ENDIF()

SET(CMAKE_EXPORT_COMPILE_COMMANDS ON)

# environment variable defined by cibuild like jenkins
IF($ENV{NLIB_CIBUILD})
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_CIBUILD")
ENDIF()

IF(CYGWIN OR CAFE OR CTR)
	FIND_PROGRAM(CYGPATH cygpath)
	IF(NOT CYGPATH)
		MESSAGE(FATAL_ERROR "cannot find cygpath")
	ENDIF()
	FUNCTION(nlib_cygwin_getcygpath _from _to)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -u "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		GET_FILENAME_COMPONENT(_toans ${_toans} ABSOLUTE)
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()

	FUNCTION(nlib_cygwin_getwinpath _from _to)
		GET_FILENAME_COMPONENT(_from ${_from} ABSOLUTE)
		EXECUTE_PROCESS(COMMAND
			${CYGPATH} -m "${_from}"
			RESULT_VARIABLE cygpath_result
			ERROR_VARIABLE cygpath_err
			OUTPUT_VARIABLE _toraw
		)
		IF(cygpath_result OR cygpath_err)
			MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
		ENDIF()
		STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
		SET(${_to} ${_toans} PARENT_SCOPE)
	ENDFUNCTION()
ENDIF()

FUNCTION(join VALUES GLUE OUTPUT)
  STRING(REPLACE ";" "${GLUE}" _TMP_STR "${VALUES}")
  SET(${OUTPUT} "${_TMP_STR}" PARENT_SCOPE)
ENDFUNCTION()

IF(CAFE)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.cafe.cmake")
ELSEIF(CTR)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.ctr.cmake")
ELSEIF(WIN32 OR CYGWIN OR UNIX)
	SET(NLIB_PCDEV 1)
	INCLUDE("${CMAKE_CURRENT_LIST_DIR}/common.pc.cmake")
ELSE()
	MESSAGE(FATAL_ERROR "!!!!! Sorry, Platform not supported !!!!!")
ENDIF()

IF(NLIB_APPLE_DIR_WORKAROUND)
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
ELSEIF(NLIB_PCDEV)
	SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/${CAT_DIR}/${CMAKE_BUILD_TYPE})
ELSE()
	IF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc120amd64/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc120amd64/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/vc120/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/vc120/Release)
	ELSEIF(EXISTS ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/gcc/Release)
	ELSE()
		SET(NLIB_TOOL_DIR ${NLIB_RELEASE_ROOT}/bin/cmake/clang/Release)
	ENDIF()
ENDIF()

MESSAGE(STATUS "CONFIG : ${CONFIG}")
MESSAGE(STATUS "CMAKE_BUILD_TYPE : ${CMAKE_BUILD_TYPE}")
MESSAGE(STATUS "CMAKE_CXX_COMPILER_ID : ${CMAKE_CXX_COMPILER_ID}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS : ${CMAKE_CXX_FLAGS}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_DEBUG : ${CMAKE_CXX_FLAGS_DEBUG}")
MESSAGE(STATUS "CMAKE_CXX_FLAGS_RELEASE : ${CMAKE_CXX_FLAGS_RELEASE}")
MESSAGE(STATUS "CMAKE_STANDARD_LIBRARIES : ${CMAKE_STANDARD_LIBRARIES}")
MESSAGE(STATUS "CMAKE_EXE_LINKER_FLAGS : ${CMAKE_EXE_LINKER_FLAGS}")
MESSAGE(STATUS "CMAKE_DEBUG_POSTFIX : ${CMAKE_DEBUG_POSTFIX}")
MESSAGE(STATUS "CMAKE_RELEASE_POSTFIX : ${CMAKE_RELEASE_POSTFIX}")

# you can check command lines if you uncomment this
# set( CMAKE_VERBOSE_MAKEFILE on )
