
#
# GCC or clang
#
IF(CMAKE_COMPILER_IS_GNUCXX)
	IF (NOT CAT_DIR)
		SET(CAT_DIR "gcc")
	ENDIF()
	EXECUTE_PROCESS(COMMAND ${CMAKE_CXX_COMPILER} -dumpversion OUTPUT_VARIABLE GCC_VERSION)
	MESSAGE(STATUS "Found g++ ${GCC_VERSION}")
	IF(GCC_VERSION VERSION_LESS 4.5)
		MESSAGE(FATAL_ERROR "!!!!! Sorry, not supported. Please use g++ 4.5 or later. !!!!!")
	ENDIF()
ELSE()
	IF (NOT CAT_DIR)
		SET(CAT_DIR "clang")
	ENDIF()
ENDIF()

#
# Compiler options
#   -ffast-math cannot be specified for SQLite
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Debug>:DEBUG;_DEBUG>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "$<$<CONFIG:Release>:NDEBUG>")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-g;-Wall;-Wextra;-fstack-protector")
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<CONFIG:Release>:-O3>")

message(STATUS "CMAKE_SYSTEM_PROCESSOR ${CMAKE_SYSTEM_PROCESSOR}")

IF(NOT NLIB_ARM)
    SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-msse4.2")
    IF(CMAKE_SIZEOF_VOID_P EQUAL 8)
    ELSE()
        SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-mfpmath=sse")
    ENDIF()
ENDIF()

SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -fstack-protector")
SET(CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -fstack-protector")
SET(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -fstack-protector")
IF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
	SET(CMAKE_EXE_LINKER_FLAGS "${CMAKE_EXE_LINKER_FLAGS} -Wl,-rpath=/usr/local/lib/gcc5")
	SET(CMAKE_MODULE_LINKER_FLAGS "${CMAKE_MODULE_LINKER_FLAGS} -Wl,-rpath=/usr/local/lib/gcc5")
	SET(CMAKE_SHARED_LINKER_FLAGS "${CMAKE_SHARED_LINKER_FLAGS} -Wl,-rpath=/usr/local/lib/gcc5")
ENDIF()

# gcc specific compiler options
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<CXX_COMPILER_ID:GNU>:>")
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<C_COMPILER_ID:GNU>:>")

# clang compiler options
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<OR:$<CXX_COMPILER_ID:Clang>,$<CXX_COMPILER_ID:AppleClang>>:>")
# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<OR:$<C_COMPILER_ID:Clang>,$<C_COMPILER_ID:AppleClang>>:>")

SET(CMAKE_CXX_STANDARD 14)

if(CMAKE_COMPILER_IS_GNUCXX)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<NOT:$<VERSION_LESS:$<CXX_COMPILER_VERSION>,4.7>>:-Wstack-usage=65536>")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<VERSION_LESS:$<CXX_COMPILER_VERSION>,4.7>:-std=gnu++0x>")
	# EXECUTE_PROCESS(COMMAND ${CMAKE_CXX_COMPILER} -dumpversion OUTPUT_VARIABLE GCC_VERSION)
	# IF(NOT (GCC_VERSION VERSION_LESS 4.9))
	#	SET(CMAKE_REQUIRED_FLAGS "-std=gnu++14")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=gnu++14")
    # ELSEIF(NOT (GCC_VERSION VERSION_LESS 4.7))
	#	SET(CMAKE_REQUIRED_FLAGS "-std=gnu++11")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=gnu++11")
	#	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Wstack-usage=65536")
	# ELSE()
	#	SET(CMAKE_REQUIRED_FLAGS "-std=gnu++0x")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=gnu++0x")
	# ENDIF()
ELSEIF(CMAKE_CXX_COMPILER_ID STREQUAL Clang OR CMAKE_CXX_COMPILER_ID STREQUAL AppleClang)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-Wthread-safety-analysis")
	# SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "$<$<NOT:$<VERSION_LESS:$<CXX_COMPILER_VERSION>,3.5>>:-Wno-pointer-bool-conversion>")
	# EXECUTE_PROCESS( COMMAND ${CMAKE_CXX_COMPILER} --version OUTPUT_VARIABLE clang_full_version_string )
	# string (REGEX REPLACE ".*clang version ([0-9]+\\.[0-9]+).*" "\\1" CLANG_VERSION_STRING ${clang_full_version_string})
	# IF(NOT (CLANG_VERSION_STRING VERSION_LESS 3.5))
	#	SET(CMAKE_REQUIRED_FLAGS "-std=c++14")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++14")
	# ELSEIF(NOT (CLANG_VERSION_STRING VERSION_LESS 3.1))
	#	SET(CMAKE_REQUIRED_FLAGS "-std=c++1y")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++1y")
	# ELSE()
	#	SET(CMAKE_REQUIRED_FLAGS "-std=c++0x")
	#	SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -std=c++0x")
	# ENDIF()
ELSE()
	message(FATAL "compiler not supproted")
ENDIF()


IF(CYGWIN)
	OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." OFF)
	SET(CMAKE_DEBUG_POSTFIX "_d")
ELSEIF(UNIX)
	SET(CMAKE_DEBUG_POSTFIX "_d")
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "-fPIC")
	IF(${CMAKE_SYSTEM_NAME} MATCHES "Linux")
		OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
		SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "_GNU_SOURCE")
	ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
		OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
		IF(CMAKE_CXX_COMPILER_ID STREQUAL Clang)
			SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -stdlib=libc++")
		ENDIF()
	ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
		SET(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS} -stdlib=libc++")
		IF(NOT NLIB_APPLE_DIR_WORKAROUND)
			OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
		ELSE()
			OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." OFF)
		ENDIF()
	ELSE()
		MESSAGE(FATAL_ERROR "Unknown environment for nlib")
	ENDIF()
ENDIF()

MACRO(library_default_build_rule target)
	IF(UNIX AND NOT CYGWIN)
		IF(NOT NLIB_APPLE_DIR_WORKAROUND)
			ADD_LIBRARY(${target} SHARED ${LIB_SRC} ${LIB_HEADER})
			IF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
				MATH(EXPR BN0 "${NLIB_BUILD_NUMBER} / 1000")
				MATH(EXPR BN1 "${NLIB_BUILD_NUMBER} % 1000")
				SET_PROPERTY(TARGET ${target} PROPERTY VERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}.${BN0}.${BN1}")
			ELSE()
				SET_PROPERTY(TARGET ${target} PROPERTY VERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}.${NLIB_BUILD_NUMBER}")
			ENDIF()
			SET_PROPERTY(TARGET ${target} PROPERTY SOVERSION "${NLIB_PRODUCT_NUMBER}.${NLIB_PRODUCT_VERSION}")
			SET_PROPERTY(TARGET ${target} PROPERTY CXX_VISIBILITY_PRESET hidden)
			SET_PROPERTY(TARGET ${target} PROPERTY C_VISIBILITY_PRESET hidden)
			SET_PROPERTY(TARGET ${target} PROPERTY VISIBILITY_INLINES_HIDDEN ON)
		ELSE()
			ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
		ENDIF()
	ELSE()
		ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	ENDIF()
	SET_PROPERTY(TARGET ${target} APPEND PROPERTY INTERFACE_INCLUDE_DIRECTORIES "$<BUILD_INTERFACE:${NLIB_PKGROOT_DIR}/include>")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	target_link_libraries_default(${target})
	IF (NLIB_APPLE_DIR_WORKAROUND)
		INSTALL(DIRECTORY ${CMAKE_CURRENT_BINARY_DIR}/${CMAKE_BUILD_TYPE}${CMAKE_XCODE_EFFECTIVE_PLATFORMS}/
				DESTINATION lib/cmake/${CAT_DIR})
	ELSE()
		INSTALL(TARGETS ${target} DESTINATION lib/cmake/${CAT_DIR})
	ENDIF()
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDMACRO()

MACRO(target_link_libraries_default target)
#	IF(${CMAKE_SYSTEM_NAME} MATCHES "FreeBSD")
#		TARGET_LINK_LIBRARIES(${target} pthread rt execinfo)
#	ELSEIF(${CMAKE_SYSTEM_NAME} MATCHES "Darwin")
#		TARGET_LINK_LIBRARIES(${target} pthread)
#	ELSE()
#		TARGET_LINK_LIBRARIES(${target} pthread rt)
#	ENDIF()
ENDMACRO()

GET_FILENAME_COMPONENT(
	NLIB_DIR
	"${NLIB_RELEASE_ROOT}/lib/cmake/${CAT_DIR}"
	ABSOLUTE)

