
#
# Check VC version and 32/64 bit
#

IF(${NLIB_DLL})
	OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." ON)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NLIB_WINDLL")
	IF(${NLIB_IDE})
		SET(CAT_DIR "idedll")
	ELSEIF(MSVC80)
		SET(CAT_DIR "vc80dll")
	ELSEIF(MSVC90)
		SET(CAT_DIR "vc90dll")
	ELSEIF(MSVC10)
		SET(CAT_DIR "vc100dll")
	ELSEIF(MSVC11)
		SET(CAT_DIR "vc110dll")
	ELSEIF(MSVC12)
		SET(CAT_DIR "vc120dll")
	ELSEIF(MSVC14)
		SET(CAT_DIR "vc140dll")
	ENDIF()
ELSE()
	OPTION(BUILD_SHARED_LIBS "Build shared libraries (DLLs)." OFF)
	IF(${NLIB_IDE})
		SET(CAT_DIR "ide")
	ELSEIF(MSVC80)
		SET(CAT_DIR "vc80")
	ELSEIF(MSVC90)
		SET(CAT_DIR "vc90")
	ELSEIF(MSVC10)
		SET(CAT_DIR "vc100")
	ELSEIF(MSVC11)
		SET(CAT_DIR "vc110")
	ELSEIF(MSVC12)
		SET(CAT_DIR "vc120")
	ELSEIF(MSVC14)
		SET(CAT_DIR "vc140")
	ENDIF()
ENDIF()

IF(MSVC11)
	# VS2012 doesn't support correctly the tuples yet
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "_VARIADIC_MAX=10")
ELSEIF(MSVC60 OR MSVC70 OR MSVC71)
	MESSAGE(FATAL_ERROR "!!!!! Sorry, not supported. Please use Visual Studio 2008 or later. !!!!!")
ELSEIF(NOT CAT_DIR)
	MESSAGE(FATAL_ERROR "!!!!! Sorry, Visual C++ not found. !!!!!")
ENDIF()

IF(CMAKE_SIZEOF_VOID_P EQUAL 8)
	SET(CAT_DIR "${CAT_DIR}amd64")
	SET(ENV{PROGRAMFILES} "C:/Program Files") # workaround
	SET(BOOST_ROOT "C:/Program Files/boost") # workaround
	SET(BOOSTROOT "D:/Program Files/boost") # workaround
ELSE()
	SET(ENV{PROGRAMFILES} "C:/Program Files (x86)") # workaround
	SET(BOOST_ROOT "C:/Program Files (x86)/boost") # workaround
	SET(BOOSTROOT "D:/Program Files (x86)/boost") # workaround
ENDIF()

SET(CMAKE_DEBUG_POSTFIX "-${CAT_DIR}_d")
SET(CMAKE_RELEASE_POSTFIX "-${CAT_DIR}")

#
# Compiler options
#
# http://www.cmake.org/Wiki/CMake_FAQ#Dynamic_Replace
FOREACH(flag_var
         CMAKE_CXX_FLAGS CMAKE_CXX_FLAGS_DEBUG CMAKE_CXX_FLAGS_RELEASE
         CMAKE_CXX_FLAGS_MINSIZEREL CMAKE_CXX_FLAGS_RELWITHDEBINFO
         CMAKE_C_FLAGS CMAKE_C_FLAGS_DEBUG CMAKE_C_FLAGS_RELEASE
         CMAKE_C_FLAGS_MINSIZEREL CMAKE_C_FLAGS_RELWITHDEBINFO)
	IF(MSVC80 OR MSVC90)
		# /MD does not work on some programs
		STRING(REPLACE "/MD" "/MT" ${flag_var} "${${flag_var}}")
	ENDIF()
	STRING(REPLACE "/Zi" "/Z7" ${flag_var} "${${flag_var}}")
	STRING(REPLACE "/W3" "/W4" ${flag_var} "${${flag_var}}")
ENDFOREACH()
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_DEFINITIONS "NOMINMAX;WIN32_LEAN_AND_MEAN;_SCL_SECURE_NO_WARNINGS;_USE_MATH_DEFINES")

IF(MSVC12)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "/Zc:rvalueCast")
ENDIF(MSVC12)
IF(NOT CMAKE_SIZEOF_VOID_P EQUAL 8)
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "/arch:SSE2")
ENDIF()
IF(MSVC80 OR MSVC90 OR MSVC10)
ELSE()
	SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "/volatile:iso")
ENDIF()
SET_PROPERTY(DIRECTORY APPEND PROPERTY COMPILE_OPTIONS "/bigobj;/Oi")

# To remove _MBCS
ADD_DEFINITIONS(-D_SBCS)

MACRO(library_default_build_rule target)
	IF(${NLIB_DLL})
		IF(EXISTS ${NLIB_PKGROOT_DIR}/../_common/cmake/version.rc.in)
			configure_file(
				${NLIB_PKGROOT_DIR}/../_common/cmake/version.rc.in
				${CMAKE_CURRENT_BINARY_DIR}/version.rc
				@ONLY)
			ADD_LIBRARY(${target} SHARED ${LIB_SRC} ${LIB_HEADER} ${CMAKE_CURRENT_BINARY_DIR}/version.rc)
			IF(ZLIB_FOUND)
				TARGET_LINK_LIBRARIES(${target} ${ZLIB_LIBRARIES})
			ENDIF()
		ELSE()
			ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
		ENDIF()
	ELSE()
		ADD_LIBRARY(${target} STATIC ${LIB_SRC} ${LIB_HEADER})
	ENDIF()
	SET_PROPERTY(TARGET ${target} APPEND PROPERTY INTERFACE_INCLUDE_DIRECTORIES "$<BUILD_INTERFACE:${NLIB_PKGROOT_DIR}/include>")
	TARGET_INCLUDE_DIRECTORIES(${target} BEFORE PRIVATE "${NLIB_PKGROOT_DIR}/include" "${NLIB_PKGROOT_DIR}/sources")
	SET(win_no_def NOGDICAPMASKS NOVIRTUALKEYCODES NOWINMESSAGES NOWINSTYLES
		 NOSYSMETRICS NOMENUS NOICONS NOKEYSTATES NOSYSCOMMANDS NORASTEROPS NOSHOWWINDOW
		 OEMRESOURCE NOATOM NOCLIPBOARD NOCTRLMGR NODRAWTEXT NOGDI NOMB NOMETAFILE NOSCROLL
		 NOSERVICE NOSOUND NOTEXTMETRIC NOWH NOWINOFFSETS NOCOMM NOKANJI NOHELP
		 NODEFFERWINDOWPOS NOMCX)
	SET_PROPERTY(TARGET ${target} APPEND PROPERTY COMPILE_DEFINITIONS "${win_no_def}")
	IF(NOT ${CMAKE_GENERATOR} MATCHES "NMake Makefiles.*")
		SET_PROPERTY(TARGET ${target} APPEND PROPERTY COMPILE_OPTIONS "/MP")
	ENDIF()
	INSTALL(TARGETS ${target} DESTINATION lib/cmake/msvc)
	INSTALL(DIRECTORY ${NLIB_PKGROOT_DIR}/include DESTINATION .)
ENDMACRO()

MACRO(target_link_libraries_default target)
	IF(NOT ${CMAKE_GENERATOR} MATCHES "NMake Makefiles.*")
		SET_PROPERTY(TARGET ${target} APPEND PROPERTY COMPILE_OPTIONS "/MP")
	ENDIF()
ENDMACRO()

GET_FILENAME_COMPONENT(
	NLIB_DIR
	"${NLIB_RELEASE_ROOT}/lib/cmake/msvc"
	ABSOLUTE)


