INCLUDE(CMakeForceCompiler)

SET(CMAKE_SYSTEM_NAME Generic)
SET(CMAKE_SYSTEM_VERSION 1)
SET(CAFE 1)
SET(CMAKE_HOST_SYSTEM_NAME Cygwin)
SET(CMAKE_HOST_SYSTEM_VERSION 1)

IF(${CMAKE_BUILD_TYPE} MATCHES "Release")
	SET(CAFE_BUILD_TYPE "NDEBUG")
ELSE()
	SET(CAFE_BUILD_TYPE "DEBUG")
ENDIF()

FIND_PROGRAM(CYGPATH cygpath)
IF(NOT CYGPATH)
	MESSAGE(FATAL_ERROR "cannot find cygpath")
ENDIF()
FUNCTION(nlib_cygwin_getcygpath_ _from _to)
	EXECUTE_PROCESS(COMMAND
		${CYGPATH} -u "${_from}"
		RESULT_VARIABLE cygpath_result
		ERROR_VARIABLE cygpath_err
		OUTPUT_VARIABLE _toraw
	)
	IF(cygpath_result OR cygpath_err)
		MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
	ENDIF()
	STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
	GET_FILENAME_COMPONENT(_toans ${_toans} ABSOLUTE)
	SET(${_to} ${_toans} CACHE FILEPATH "" FORCE)
ENDFUNCTION()

FUNCTION(nlib_cygwin_getwinpath_ _from _to)
	GET_FILENAME_COMPONENT(_from ${_from} ABSOLUTE)
	EXECUTE_PROCESS(COMMAND
		${CYGPATH} -m "${_from}"
		RESULT_VARIABLE cygpath_result
		ERROR_VARIABLE cygpath_err
		OUTPUT_VARIABLE _toraw
	)
	IF(cygpath_result OR cygpath_err)
		MESSAGE(FATAL_ERROR "${CYGPATH} failed:: ${cygpath_err}")
	ENDIF()
	STRING(REGEX REPLACE "[\r\n]" "" _toans "${_toraw}")
	SET(${_to} ${_toans} CACHE FILEPATH "" FORCE)
ENDFUNCTION()

FUNCTION(join VALUES GLUE OUTPUT)
  STRING(REPLACE ";" "${GLUE}" _TMP_STR "${VALUES}")
  SET(${OUTPUT} "${_TMP_STR}" PARENT_SCOPE)
ENDFUNCTION()

nlib_cygwin_getcygpath_($ENV{GHS_ROOT} CYG_GHS_ROOT)
nlib_cygwin_getwinpath_(${CYG_GHS_ROOT} WIN_GHS_ROOT)
nlib_cygwin_getcygpath_($ENV{CAFE_ROOT} CYG_CAFE_ROOT)
nlib_cygwin_getwinpath_(${CYG_CAFE_ROOT} WIN_CAFE_ROOT)
MESSAGE(STATUS "WIN_CAFE_ROOT=${WIN_CAFE_ROOT}")
MESSAGE(STATUS "CYG_CAFE_ROOT=${CYG_CAFE_ROOT}")
MESSAGE(STATUS "WIN_GHS_ROOT=${WIN_GHS_ROOT}")
MESSAGE(STATUS "CYG_GHS_ROOT=${CYG_GHS_ROOT}")

STRING(REPLACE "/" "\\\\/" WIN_CAFE_ROOT2 "${WIN_CAFE_ROOT}")

CMAKE_FORCE_C_COMPILER(${CYG_GHS_ROOT}/ccppc.exe GHS)
CMAKE_FORCE_CXX_COMPILER(${CYG_GHS_ROOT}/cxppc.exe GHS)
SET(CMAKE_AR "${CYG_GHS_ROOT}/ccppc.exe" CACHE FILEPATH "Archiver" FORCE)
SET(CMAKE_NM "${CYG_GHS_ROOT}/gnm.exe" CACHE FILEPATH "NM" FORCE)
SET(CMAKE_EXECUTABLE_SUFFIX ".elf" CACHE FILEPATH "Executable suffix")

# CMAKE_C_FLAGS_INIT and CMAKE_CXX_FLAGS_INIT do not work on cmake 3.3.1(cygwin) ?
#SET(CMAKE_C_FLAGS_INIT "--no_commons -c99 -kanji=utf8 -only_explicit_reg_use --no_exceptions --link_once_templates -cpu=espresso -sda=none -Ogeneral -G --quit_after_warnings --Werror --no_implicit_include -dbg_source_root ${WIN_CAFE_ROOT}/system -MD")
#SET(CMAKE_CXX_FLAGS_INIT "--no_commons -c99 -kanji=utf8 -only_explicit_reg_use --no_exceptions --g++ --link_once_templates -cpu=espresso -sda=none -Ogeneral -G --quit_after_warnings --Werror --no_implicit_include -dbg_source_root ${WIN_CAFE_ROOT}/system -MD")
SET(CMAKE_C_LINK_FLAGS "-e _start -relprog_cafe -lnk=-nosegments_always_executable -nostartfile -Mn -Mu -delete ${WIN_CAFE_ROOT}/system/include/cafe/eppc.Cafe.ld")
SET(CMAKE_CXX_LINK_FLAGS "-e _start -relprog_cafe -lnk=-nosegments_always_executable -nostartfile -Mn -Mu -delete ${WIN_CAFE_ROOT}/system/include/cafe/eppc.Cafe.ld")

#  set(CMAKE_C_COMPILE_OBJECT
#    "<CMAKE_C_COMPILER> <DEFINES> <FLAGS> -o <OBJECT> -c <SOURCE>")
#  set(CMAKE_CXX_COMPILE_OBJECT
#    "<CMAKE_CXX_COMPILER>  <DEFINES> <FLAGS> -o <OBJECT> -c <SOURCE>")
SET(tmp
	"echo <DEFINES> <FLAGS> -o <OBJECT> -c <SOURCE>"
	"sed -e \"s/-I\\/cygdrive\\/\\(.\\)\\//-I\\1:\\//g\""
	"sed -e \"s/-c \\/cygdrive\\/\\(.\\)\\//-c \\1:\\//g\""
	"xargs <CMAKE_C_COMPILER>")
join("${tmp}" " | " tmp)
SET(CMAKE_C_COMPILE_OBJECT ${tmp})

SET(tmp
	"echo <DEFINES> <FLAGS> -o <OBJECT> -c <SOURCE>"
	"sed -e \"s/-I\\/cygdrive\\/\\(.\\)\\//-I\\1:\\//g\""
	"sed -e \"s/-c \\/cygdrive\\/\\(.\\)\\//-c \\1:\\//g\""
	"xargs <CMAKE_CXX_COMPILER>")
join("${tmp}" " | " tmp)
SET(CMAKE_CXX_COMPILE_OBJECT ${tmp})

SET(CMAKE_CXX_ARCHIVE_CREATE "<CMAKE_AR> -archive -o <TARGET> <LINK_FLAGS> <OBJECTS>")
SET(CMAKE_C_ARCHIVE_CREATE "<CMAKE_AR> -archive -o <TARGET> <LINK_FLAGS> <OBJECTS>")

SET(filter
	"awk -v RS=' ' '$0!~/-L[^ ]/{print}/-L[^ ]/{cmd=\\\"cygpath -m \\\"substr($0, 3)\;cmd|getline\;print \\\"-L\\\" $0}'|awk -v ORS=' ' '{print}'")

#<CMAKE_CXX_COMPILER>  <FLAGS> <CMAKE_CXX_LINK_FLAGS> <LINK_FLAGS> <OBJECTS>  -o <TARGET> <LINK_LIBRARIES>")
#  set(CMAKE_C_LINK_EXECUTABLE
#   "<CMAKE_C_COMPILER> <FLAGS> <CMAKE_C_LINK_FLAGS> <LINK_FLAGS> <OBJECTS>  -o <TARGET> <LINK_LIBRARIES>")
SET(CMAKE_CXX_LINK_EXECUTABLE
	"sh -c \"echo ${CYG_CAFE_ROOT}/system/bin/tool/preprpl64 -e _start <LINK_LIBRARIES> -o <OBJECT_DIR>/rpl_export.o <OBJECTS> | sed -r -e 's/-lsdk\\([^ ]*\\)/\\\\1.a/g' | sed -r -e 's/-l\\([^ ]*\\)/lib\\\\1.a/g' | ${filter} > <OBJECT_DIR>/linktmp.sh\""
	"sh <OBJECT_DIR>/linktmp.sh"
	"sh -c \"echo <CMAKE_CXX_COMPILER> -o <TARGET_BASE>.elf <FLAGS> <CMAKE_CXX_LINK_FLAGS> -map=<TARGET_BASE>.map <OBJECT_DIR>/rpl_export.o <OBJECTS> <LINK_LIBRARIES> | sed -r -e 's/-lsdk\\([^ ]*\\)/${WIN_CAFE_ROOT2}\\\\/system\\\\/lib\\\\/ghs\\\\/cafe\\\\/${CAFE_BUILD_TYPE}\\\\/\\\\1.a/g' | ${filter} > <OBJECT_DIR>/linktmp2.sh \""
	"sh <OBJECT_DIR>/linktmp2.sh"
	"${CYG_CAFE_ROOT}/system/bin/tool/makerpl64 <TARGET_BASE>.elf -f -t BUILD_TYPE=${CAFE_BUILD_TYPE} -s -l <OBJECT_DIR>/<TARGET_BASE>.a")

SET(CMAKE_C_LINK_EXECUTABLE
	"sh -c \"echo ${CYG_CAFE_ROOT}/system/bin/tool/preprpl64 -e _start -L${WIN_CAFE_ROOT}/system/lib/ghs/cafe/${CAFE_BUILD_TYPE} <LINK_LIBRARIES> -o <OBJECT_DIR>/rpl_export.o <OBJECTS> | sed -r -e 's/-lsdk\\([^ ]*\\)/\\\\1.a/g' | sed -r -e 's/-l\\([^ ]*\\)/lib\\\\1.a/g' | ${filter} > <OBJECT_DIR>/linktmp.sh\""
	"sh <OBJECT_DIR>/linktmp.sh"
	"sh -c \"echo <CMAKE_C_COMPILER> -o <TARGET_BASE>.elf <FLAGS> <CMAKE_C_LINK_FLAGS> -map=<TARGET_BASE>.map <OBJECT_DIR>/rpl_export.o <OBJECTS> <LINK_LIBRARIES> | sed -r -e 's/-lsdk\\([^ ]*\\)/${WIN_CAFE_ROOT2}\\\\/system\\\\/lib\\\\/ghs\\\\/cafe\\\\/${CAFE_BUILD_TYPE}\\\\/\\\\1.a/g' | ${filter} > <OBJECT_DIR>/linktmp2.sh \""
	"sh <OBJECT_DIR>/linktmp2.sh"
	"${CYG_CAFE_ROOT}/system/bin/tool/makerpl64 <TARGET_BASE>.elf -f -t BUILD_TYPE=${CAFE_BUILD_TYPE} -s -l <OBJECT_DIR>/<TARGET_BASE>.a")

# search for programs in the build host directories
SET(CMAKE_FIND_ROOT_PATH_MODE_PROGRAM NEVER)
# for libraries and headers in the target directories
SET(CMAKE_FIND_ROOT_PATH_MODE_LIBRARY ONLY)
SET(CMAKE_FIND_ROOT_PATH_MODE_INCLUDE ONLY)

