﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_BASE64OUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_BASE64OUTPUTSTREAM_H_

#include "nn/nlib/OutputConverterStream.h"

NLIB_NAMESPACE_BEGIN

class OutputStream;

namespace detail {

class NLIB_VIS_PUBLIC Base64OutputTransform NLIB_FINAL : public OutputTransform {
 public:
    enum NewLineMode { NEWLINEMODE_NONE = 0, NEWLINEMODE_CRLF, NEWLINEMODE_LF };

    Base64OutputTransform() NLIB_NOEXCEPT;
    virtual errno_t Transform(OutputStream* os, const void* p, size_t nBytes,
                              bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;

    void SetNewLineMode(NewLineMode mode) NLIB_NOEXCEPT { m_NewLine = mode; }

 private:
    int m_C;
    int m_CLen;
    NewLineMode m_NewLine;
    int m_Column;
    unsigned char m_Buf[192];
};

}  // namespace detail

#ifndef NLIB_DOXYGEN
typedef OutputConverterStreamTempl<detail::Base64OutputTransform> Base64OutputStream;
#else
class NLIB_VIS_PUBLIC Base64OutputStream NLIB_FINAL
    : public OutputConverterStreamTempl<detail::Base64OutputTransform> {
 public:
    // NOTE: define NewLineMode (not typedef) again for doxygen.
    enum NewLineMode { NEWLINEMODE_NONE = 0, NEWLINEMODE_CRLF, NEWLINEMODE_LF };

 public:
    void SetNewLineMode(NewLineMode mode) NLIB_NOEXCEPT {
        NLIB_STATIC_ASSERT(static_cast<int>(NEWLINEMODE_NONE) ==
                           static_cast<int>(TransformType::NEWLINEMODE_NONE));
        NLIB_STATIC_ASSERT(static_cast<int>(NEWLINEMODE_CRLF) ==
                           static_cast<int>(TransformType::NEWLINEMODE_CRLF));
        NLIB_STATIC_ASSERT(static_cast<int>(NEWLINEMODE_LF) ==
                           static_cast<int>(TransformType::NEWLINEMODE_LF));
        m_Transform.SetNewLineMode(static_cast<TransformType::NewLineMode>(mode));
    }
};
#endif

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_BASE64OUTPUTSTREAM_H_
