﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_CSTRING_H_
#define INCLUDE_NN_NLIB_CSTRING_H_

#include <stdarg.h>
#include <string.h>
#ifndef NLIB_ISDIGIT_USE_FALLBACK
#include <ctype.h>
#endif

#include "nn/nlib/Config.h"

// functions below may be deprecated ....
NLIB_NAMESPACE_BEGIN

inline size_t StrLen(const char* str) NLIB_NOEXCEPT { return nlib_strlen(str); }
inline size_t StrLen(const nlib_utf16_t* str) NLIB_NOEXCEPT { return nlib_utf16len(str); }
inline size_t StrLen(const nlib_utf32_t* str) NLIB_NOEXCEPT { return nlib_utf32len(str); }
inline size_t StrLen(const wchar_t* str) NLIB_NOEXCEPT { return nlib_wcslen(str); }

inline errno_t CodePointCount(const char* str, size_t* count) NLIB_NOEXCEPT {
    return nlib_strcplen(count, str);
}
inline errno_t CodePointCount(const nlib_utf16_t* str, size_t* count) NLIB_NOEXCEPT {
    return nlib_utf16cplen(count, str);
}
inline errno_t CodePointCount(const nlib_utf32_t* str, size_t* count) NLIB_NOEXCEPT {
    return nlib_utf32cplen(count, str);
}
inline errno_t CodePointCount(const wchar_t* str, size_t* count) NLIB_NOEXCEPT {
    return nlib_wcscplen(count, str);
}

NLIB_VIS_PUBLIC int StrCmpFallback(const wchar_t* s1, const wchar_t* s2) NLIB_NOEXCEPT;
NLIB_VIS_PUBLIC int StrCmpFallback(const nlib_utf16_t* s1, const nlib_utf16_t* s2) NLIB_NOEXCEPT;
NLIB_VIS_PUBLIC int StrCmpFallback(const nlib_utf32_t* s1, const nlib_utf32_t* s2) NLIB_NOEXCEPT;

inline int StrCmp(const char* s1, const char* s2) NLIB_NOEXCEPT { return strcmp(s1, s2); }
inline int StrCmp(const wchar_t* s1, const wchar_t* s2) NLIB_NOEXCEPT {
#ifndef NLIB_WCSCMP_USE_FALLBACK
    return wcscmp(s1, s2);
#else
    return StrCmpFallback(s1, s2);
#endif
}
inline int StrCmp(const nlib_utf16_t* s1, const nlib_utf16_t* s2) NLIB_NOEXCEPT {
    return StrCmpFallback(s1, s2);
}
inline int StrCmp(const nlib_utf32_t* s1, const nlib_utf32_t* s2) NLIB_NOEXCEPT {
    return StrCmpFallback(s1, s2);
}

NLIB_VIS_PUBLIC int StrNcmpFallback(const char* s1, const char* s2, size_t n) NLIB_NOEXCEPT;
NLIB_VIS_PUBLIC int StrNcmpFallback(const wchar_t* s1, const wchar_t* s2, size_t n) NLIB_NOEXCEPT;
NLIB_VIS_PUBLIC int StrNcmpFallback(const nlib_utf16_t* s1,
                                    const nlib_utf16_t* s2, size_t n) NLIB_NOEXCEPT;
NLIB_VIS_PUBLIC int StrNcmpFallback(const nlib_utf32_t* s1,
                                    const nlib_utf32_t* s2, size_t n) NLIB_NOEXCEPT;

inline int StrNcmp(const char* s1, const char* s2, size_t n) NLIB_NOEXCEPT {
#ifndef NLIB_STRNCMP_USE_FALLBACK
    return strncmp(s1, s2, n);
#else
    return StrNcmpFallback(s1, s2, n);
#endif
}
inline int StrNcmp(const wchar_t* s1, const wchar_t* s2, size_t n) NLIB_NOEXCEPT {
#ifndef NLIB_WCSNCMP_USE_FALLBACK
    return wcsncmp(s1, s2, n);
#else
    return StrNcmpFallback(s1, s2, n);
#endif
}
inline int StrNcmp(const nlib_utf16_t* s1, const nlib_utf16_t* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmpFallback(s1, s2, n);
}
inline int StrNcmp(const nlib_utf32_t* s1, const nlib_utf32_t* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmpFallback(s1, s2, n);
}

// StrEqual to be deprecated
inline bool StrEqual(const char* s1, const char* s2) NLIB_NOEXCEPT {
    return s1[0] == s2[0] && StrCmp(s1, s2) == 0;
}
inline bool StrEqual(const nlib_utf16_t* s1, const nlib_utf16_t* s2) NLIB_NOEXCEPT {
    return s1[0] == s2[0] && StrCmp(s1, s2) == 0;
}
inline bool StrEqual(const nlib_utf32_t* s1, const nlib_utf32_t* s2) NLIB_NOEXCEPT {
    return s1[0] == s2[0] && StrCmp(s1, s2) == 0;
}
inline bool StrEqual(const wchar_t* s1, const wchar_t* s2) NLIB_NOEXCEPT {
    return s1[0] == s2[0] && StrCmp(s1, s2) == 0;
}

// StrNequal to be deprecated
inline bool StrNequal(const char* s1, const char* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmp(s1, s2, n) == 0;
}
inline bool StrNequal(const nlib_utf16_t* s1, const nlib_utf16_t* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmp(s1, s2, n) == 0;
}
inline bool StrNequal(const nlib_utf32_t* s1, const nlib_utf32_t* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmp(s1, s2, n) == 0;
}
inline bool StrNequal(const wchar_t* s1, const wchar_t* s2, size_t n) NLIB_NOEXCEPT {
    return StrNcmp(s1, s2, n) == 0;
}

inline errno_t StrCpy(char* dest, size_t destSize, const char* src) NLIB_NOEXCEPT {
    return nlib_strcpy(dest, destSize, src);
}
inline errno_t StrCpy(nlib_utf16_t* dest, size_t destSize, const nlib_utf16_t* src) NLIB_NOEXCEPT {
    return nlib_utf16cpy(dest, destSize, src);
}
inline errno_t StrCpy(nlib_utf32_t* dest, size_t destSize, const nlib_utf32_t* src) NLIB_NOEXCEPT {
    return nlib_utf32cpy(dest, destSize, src);
}
inline errno_t StrCpy(wchar_t* dest, size_t destSize, const wchar_t* src) NLIB_NOEXCEPT {
    return nlib_wcscpy(dest, destSize, src);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrCpy(T (&dest)[N], const T* src) NLIB_NOEXCEPT {
    return StrCpy(&dest[0], N, src);
}

inline errno_t StrNcpy(char* dest, size_t destSize, const char* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_strncpy(dest, destSize, src, maxNumCopyWithoutNull);
}
inline errno_t StrNcpy(nlib_utf16_t* dest, size_t destSize, const nlib_utf16_t* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_utf16ncpy(dest, destSize, src, maxNumCopyWithoutNull);
}
inline errno_t StrNcpy(nlib_utf32_t* dest, size_t destSize, const nlib_utf32_t* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_utf32ncpy(dest, destSize, src, maxNumCopyWithoutNull);
}
inline errno_t StrNcpy(wchar_t* dest, size_t destSize, const wchar_t* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_wcsncpy(dest, destSize, src, maxNumCopyWithoutNull);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrNcpy(T(&dest)[N], const T* src,
                                                   size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return StrNcpy(&dest[0], N, src, maxNumCopyWithoutNull);
}

inline errno_t MemCpy(void* dest, size_t destSize, const void* src, size_t srcSize) NLIB_NOEXCEPT {
    return nlib_memcpy(dest, destSize, src, srcSize);
}

inline errno_t MemMove(void* dest, size_t destSize, const void* src, size_t srcSize) NLIB_NOEXCEPT {
    return nlib_memmove(dest, destSize, src, srcSize);
}

inline errno_t StrCat(char* dest, size_t destSize, const char* src) NLIB_NOEXCEPT {
    return nlib_strcat(dest, destSize, src);
}

inline errno_t StrCat(wchar_t* dest, size_t destSize, const wchar_t* src) NLIB_NOEXCEPT {
    return nlib_wcscat(dest, destSize, src);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrCat(T(&dest)[N], const T* src) NLIB_NOEXCEPT {
    return StrCat(&dest[0], N, src);
}

inline errno_t StrNcat(char* dest, size_t destSize, const char* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_strncat(dest, destSize, src, maxNumCopyWithoutNull);
}

inline errno_t StrNcat(wchar_t* dest, size_t destSize, const wchar_t* src,
                       size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return nlib_wcsncat(dest, destSize, src, maxNumCopyWithoutNull);
}

template<class T, size_t N>
NLIB_ALWAYS_INLINE errno_t StrNcat(T (&dest)[N], const T* src,
                                                   size_t maxNumCopyWithoutNull) NLIB_NOEXCEPT {
    return StrNcat(&dest[0], N, src, maxNumCopyWithoutNull);
}

/*
    STR37-C
    isalnum, isalpha, isascii, isblank,
    iscntrl, isdigit, isgraph, islower,
    isprint, ispunct, isspace, isupper,
    isxdigit, toascii, toupper, tolower
*/

#ifndef NLIB_ISDIGIT_USE_FALLBACK
inline int IsAlnum(int c) NLIB_NOEXCEPT { return isalnum(static_cast<unsigned char>(c)); }
inline int IsAlpha(int c) NLIB_NOEXCEPT { return isalpha(static_cast<unsigned char>(c)); }
// isascii
// isblank
inline int IsCntrl(int c) NLIB_NOEXCEPT { return iscntrl(static_cast<unsigned char>(c)); }
inline int IsDigit(int c) NLIB_NOEXCEPT { return isdigit(static_cast<unsigned char>(c)); }
inline int IsGraph(int c) NLIB_NOEXCEPT { return isgraph(static_cast<unsigned char>(c)); }
inline int IsLower(int c) NLIB_NOEXCEPT { return islower(static_cast<unsigned char>(c)); }

inline int IsPrint(int c) NLIB_NOEXCEPT { return isprint(static_cast<unsigned char>(c)); }
inline int IsPunct(int c) NLIB_NOEXCEPT { return ispunct(static_cast<unsigned char>(c)); }
inline int IsSpace(int c) NLIB_NOEXCEPT { return isspace(static_cast<unsigned char>(c)); }
inline int IsUpper(int c) NLIB_NOEXCEPT { return isupper(static_cast<unsigned char>(c)); }

inline int IsXdigit(int c) NLIB_NOEXCEPT { return isxdigit(static_cast<unsigned char>(c)); }
// toascii
inline int ToUpper(int c) NLIB_NOEXCEPT { return toupper(static_cast<unsigned char>(c)); }
inline int ToLower(int c) NLIB_NOEXCEPT { return tolower(static_cast<unsigned char>(c)); }

#else
inline int IsAlnum(int ch) NLIB_NOEXCEPT {
    return ('0' <= ch && ch <= '9') || ('A' <= ch && ch <= 'Z') || ('a' <= ch && ch <= 'z');
}
inline int IsAlpha(int ch) NLIB_NOEXCEPT {
    return ('A' <= ch && ch <= 'Z') || ('a' <= ch && ch <= 'z');
}
// isascii
// isblank
inline int IsCntrl(int ch) NLIB_NOEXCEPT { return (ch >= 0 && ch <= 0x1F) || ch == 0x7F; }
inline int IsDigit(int ch) NLIB_NOEXCEPT { return ('0' <= ch && ch <= '9'); }
inline int IsGrpah(int ch) NLIB_NOEXCEPT { return ch >= 0x21 && ch <= 0x7E; }
inline int IsLower(int ch) NLIB_NOEXCEPT { return (ch >= 'a' && ch <= 'z'); }

inline int IsPrint(int ch) NLIB_NOEXCEPT { return ch >= 0x20 && ch <= 0x7E; }
inline int IsPunct(int ch) NLIB_NOEXCEPT { return (ch >= 0x00 && ch <= 0x20) || ch == 0x7F; }
inline int IsSpace(int c) NLIB_NOEXCEPT { return ((c) == ' ' || (c) == '\t' || (c) == '\n'); }
inline int IsUpper(int ch) NLIB_NOEXCEPT { return (ch >= 'A' && ch <= 'Z'); }

inline int IsXdigit(int ch) NLIB_NOEXCEPT {
    return static_cast<unsigned int>(ch - '0') < 10u ||
           static_cast<unsigned int>((ch | 0x20) - 'a') < 6u;
}
// toascii
inline int ToLower(int ch) NLIB_NOEXCEPT {
    return (ch >= 'A' && ch <= 'Z') ? ch + ('a' - 'A') : ch;
}
inline int ToUpper(int ch) NLIB_NOEXCEPT {
    return (ch >= 'a' && ch <= 'z') ? ch - ('a' - 'A') : ch;
}
#endif

// NOTE: It is possible that NLIB_VSNPRINTF is VsnPrintfFallback.
NLIB_VIS_PUBLIC int VsnPrintfFallback(char* buf, size_t size, const char* fmt,
                                      va_list args) NLIB_NOEXCEPT;

inline int VsnPrintf(char* buf, size_t size, const char* fmt, va_list args) NLIB_NOEXCEPT {
    size_t count;
    errno_t e = nlib_vsnprintf(&count, buf, size, fmt, args);
#ifndef NLIB_NO_ERRNO
    if (e != 0) errno = e;
#endif
    return (e == 0) ? static_cast<int>(count) : -1;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintf(char (&buf)[N], const char* fmt, va_list args) NLIB_NOEXCEPT {
    return VsnPrintf(buf, N, fmt, args);
}

inline int SnPrintf(char* buf, size_t size, const char* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintf(char (&buf)[N], const char* fmt, ...) NLIB_NOEXCEPT {
    // gcc cannot inline vaargs functions
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, N, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintfFallback(char (&buf)[N], const char* fmt,
                                         va_list args) NLIB_NOEXCEPT {
    return VsnPrintfFallback(buf, N, fmt, args);
}

inline int SnPrintfFallback(char* buf, size_t size, const char* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintfFallback(char (&buf)[N],
                                            const char* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, N, fmt, args);
    va_end(args);
    return rval;
}

NLIB_VIS_PUBLIC int VsnPrintfFallback(wchar_t* buf, size_t size, const wchar_t* fmt,
                                      va_list args) NLIB_NOEXCEPT;

inline int VsnPrintf(wchar_t* buf, size_t size, const wchar_t* fmt, va_list args) NLIB_NOEXCEPT {
    size_t count;
    errno_t e = nlib_vsnwprintf(&count, buf, size, fmt, args);
#ifndef NLIB_NO_ERRNO
    if (e != 0) errno = e;
#endif
    return (e == 0) ? static_cast<int>(count) : -1;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintf(wchar_t(&buf)[N], const wchar_t* fmt, va_list args) NLIB_NOEXCEPT {
    return VsnPrintf(buf, N, fmt, args);
}

inline int SnPrintf(wchar_t* buf, size_t size, const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintf(wchar_t(&buf)[N], const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintf(buf, N, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
NLIB_ALWAYS_INLINE int VsnPrintfFallback(wchar_t(&buf)[N], const wchar_t* fmt,
                                         va_list args) NLIB_NOEXCEPT {
    return VsnPrintfFallback(buf, N, fmt, args);
}

inline int SnPrintfFallback(wchar_t* buf, size_t size, const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, size, fmt, args);
    va_end(args);
    return rval;
}

template <size_t N>
inline NLIB_VIS_HIDDEN int SnPrintfFallback(wchar_t(&buf)[N],
                                            const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    int rval = VsnPrintfFallback(buf, N, fmt, args);
    va_end(args);
    return rval;
}

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_CSTRING_H_
