﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_
#define INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN

class DynamicAlignedStorage {
 public:
    DynamicAlignedStorage() NLIB_NOEXCEPT : storage_(NULL) {}
    NLIB_MOVE_MEMBER_HELPER_1(DynamicAlignedStorage, storage_)
    errno_t Init(size_t size, size_t align) NLIB_NOEXCEPT {
        if (align == 0) return EINVAL;
        if ((align & (align - 1)) != 0) return EINVAL;
#ifdef NN_PLATFORM_CTR
        void* p = nlib_malloc(size + align);
#elif defined(_MSC_VER)
        void* p = _aligned_malloc(size, align);
#else
        void* p = nlib_memalign(size, align);
#endif
        if (!p) return ENOMEM;
        storage_ = p;
#ifdef NN_PLATFORM_CTR
        align_ = align;
#endif
        return 0;
    }
    void swap(DynamicAlignedStorage& rhs) NLIB_NOEXCEPT {
        void* tmp = rhs.storage_;
        rhs.storage_ = storage_;
        storage_ = tmp;
    }
    ~DynamicAlignedStorage() NLIB_NOEXCEPT {
#ifdef _MSC_VER
        _aligned_free(storage_);
#else
        nlib_free(storage_);
#endif
    }
    void* Get() NLIB_NOEXCEPT {
#ifndef NN_PLATFORM_CTR
        return storage_;
#else
        uintptr_t pos = (reinterpret_cast<uintptr_t>(storage_) + (align_ - 1)) & ~(align_ - 1);
        return reinterpret_cast<void*>(pos);
#endif
    }

 private:
    void* storage_;
#ifdef NN_PLATFORM_CTR
    size_t align_;
#endif
    NLIB_DISALLOW_COPY_AND_ASSIGN(DynamicAlignedStorage);
};

NLIB_ALWAYS_INLINE void swap(DynamicAlignedStorage& lhs,
                             DynamicAlignedStorage& rhs) NLIB_NOEXCEPT {
    lhs.swap(rhs);
}

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_DYNAMICALIGNEDSTORAGE_H_
