﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

struct FileOutputStreamSettings {
    void* buf;
    size_t buffer_size;

 public:
    NLIB_CEXPR FileOutputStreamSettings() NLIB_NOEXCEPT : buf(NULL), buffer_size(4096) {}
    NLIB_CEXPR FileOutputStreamSettings(void* buf_, size_t buffer_size_) NLIB_NOEXCEPT
        : buf(buf_),
          buffer_size(buffer_size_) {}
};

class NLIB_VIS_PUBLIC FileOutputStream NLIB_FINAL : public OutputStream{
 public:
    FileOutputStream() NLIB_NOEXCEPT : m_Fd(NLIB_FD_INVALID),
                                       m_Buf(NULL),
                                       m_BufSize(0),
                                       m_IsInternalBuffer(false) {}
    virtual ~FileOutputStream() NLIB_NOEXCEPT NLIB_FINAL NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR FileOutputStreamSettings settings;
        return this->Init(settings);
    }
    NLIB_CHECK_RESULT errno_t Init(const FileOutputStreamSettings& settings) NLIB_NOEXCEPT;
    NLIB_CHECK_RESULT errno_t Open(const char* filename) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t Open(const wchar_t* filename) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_CHECK_RESULT errno_t FdOpen(nlib_fd fd) NLIB_NOEXCEPT;
    nlib_fd GetFd() const NLIB_NOEXCEPT { return m_Fd; }

 private:
    virtual bool PushBuffer_(const void* p, size_t nBytes,
                             bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool WriteGather_(const nlib_fd_iovec* iov, int iovcnt) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_fd m_Fd;
    unsigned char* m_Buf;
    size_t m_BufSize;
    bool m_IsInternalBuffer;
    NLIB_DISALLOW_COPY_AND_ASSIGN(FileOutputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_FILEOUTPUTSTREAM_H_
