﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HTTPSTYLEURI_H_
#define INCLUDE_NN_NLIB_HTTPSTYLEURI_H_

#include <utility>

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/Uri.h"

NLIB_NAMESPACE_BEGIN

class HttpStyleUri NLIB_FINAL {
 public:
    typedef std::pair<const char*, const char*> KeyValue;
    // cppcheck-suppress uninitMemberVar
    HttpStyleUri() NLIB_NOEXCEPT { this->Reset(); }
    NLIB_VIS_PUBLIC ~HttpStyleUri() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool FromUri(const Uri& uri) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool ToUri(Uri* uri) const NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Parse(const char* str) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool ComposeString(char* buf, size_t size) const NLIB_NOEXCEPT;
    template <size_t N>
    bool ComposeString(char (&buf)[N]) const NLIB_NOEXCEPT {
        return this->ComposeString(buf, N);
    }
    NLIB_VIS_PUBLIC bool SetScheme(const char* scheme) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool SetUserInfo(const char* userinfo) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool SetHost(const char* host) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool SetPort(int port) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool SetPath(const char* path) NLIB_NOEXCEPT;
    bool SetQuery(const char* query) NLIB_NOEXCEPT { return m_Query.Set(query); }
    NLIB_VIS_PUBLIC bool SetFragment(const char* fragment) NLIB_NOEXCEPT;
    bool AddQuery(const char* key, const char* value) NLIB_NOEXCEPT {
        return m_Query.Add(key, value);
    }
    const char* GetScheme() const NLIB_NOEXCEPT {
        return m_IsSchemeHttps ? "https" : "http";
    }
    const char* GetUserInfo() const NLIB_NOEXCEPT { return m_UserInfo.get(); }
    const char* GetHost() const NLIB_NOEXCEPT { return m_Host.get(); }
    int GetPort() const NLIB_NOEXCEPT { return m_Port; }
    size_t GetNumSegment() const NLIB_NOEXCEPT {
        return m_Segments.GetNumSegment();
    }
    const char* GetSegment(size_t i) const NLIB_NOEXCEPT {
        return m_Segments.GetSegment(i);
    }
    bool IsPathDirectory() const NLIB_NOEXCEPT {
        return m_Segments.IsDirectory();
    }
    const char* GetQueryValue(const char* key) const NLIB_NOEXCEPT {
        return m_Query.GetValue(key);
    }
    size_t GetNumQuery() const NLIB_NOEXCEPT { return m_Query.GetNumQuery(); }
    const KeyValue* GetQueryKeyValue(size_t i) const NLIB_NOEXCEPT {
        return m_Query.GetKeyValue(i);
    }
    const char* GetFragment() const NLIB_NOEXCEPT { return m_Fragment.get(); }
    NLIB_VIS_PUBLIC bool AddBaseUri(const Uri& relative, const HttpStyleUri& base) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC void Reset() NLIB_NOEXCEPT;

 private:
    bool ComposeQuery(char* first, char* last) const NLIB_NOEXCEPT {
        return m_Query.Compose(&first, last);
    }

 private:
    bool m_IsSchemeHttps;
    int m_Port;
    UniquePtr<char[]> m_UserInfo;
    UniquePtr<char[]> m_Host;
    UniquePtr<char[]> m_Fragment;
    detail::Segments m_Segments;
    detail::Query m_Query;
};

NLIB_NAMESPACE_END
#endif  // INCLUDE_NN_NLIB_HTTPSTYLEURI_H_
