﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_

#include "nn/nlib/InputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC MemoryInputStream NLIB_FINAL : public InputStream {
 public:
    MemoryInputStream() NLIB_NOEXCEPT : m_Buf(NULL), m_Size(0) {}
    virtual ~MemoryInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    template <class T, size_t N>
    explicit MemoryInputStream(const T(&buf)[N]) NLIB_NOEXCEPT : m_Buf(buf),
                                                                 m_Size(N * sizeof(T)) {}
    MemoryInputStream(const void* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;
    template <class T, size_t N>
    void Init(const T(&buf)[N]) NLIB_NOEXCEPT {
        this->Init(&buf[0], N * sizeof(T));
    }
    void Init(const void* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;

 private:
    virtual size_t FillBuffer_(void* p, size_t nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE {
        m_Buf = NULL;
        m_Size = 0;
        return true;
    }

 private:
    const void* m_Buf;
    size_t m_Size;
    NLIB_DISALLOW_COPY_AND_ASSIGN(MemoryInputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_MEMORYINPUTSTREAM_H_
