﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC MemoryOutputStream NLIB_FINAL : public OutputStream {
 public:
    MemoryOutputStream() NLIB_NOEXCEPT : m_MemBuf(NULL), m_MemPos(0), m_MemSize(0) {}
    MemoryOutputStream(void* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;
    template <class T, size_t N>
    explicit MemoryOutputStream (T(&buf)[N]) NLIB_NOEXCEPT
        : m_MemBuf(reinterpret_cast<unsigned char*>(&buf[0])),
          m_MemPos(0),
          m_MemSize(N * sizeof(T)) {}
    virtual ~MemoryOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    const void* data() const NLIB_NOEXCEPT { return m_MemBuf; }
    void Init(void* buf, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;
    template <class T, size_t N>
    void Init(T (&buf)[N]) NLIB_NOEXCEPT {
        this->Init(&buf[0], N * sizeof(T));
    }

 private:
    virtual bool PushBuffer_(const void* p, size_t nBytes,
                             bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE {
        m_MemBuf = NULL;
        m_MemPos = 0;
        m_MemSize = 0;
        return true;
    }
    virtual void* GetWorkBuffer_(size_t* nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    unsigned char* m_MemBuf;
    size_t m_MemPos;
    size_t m_MemSize;
    NLIB_DISALLOW_COPY_AND_ASSIGN(MemoryOutputStream);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_MEMORYOUTPUTSTREAM_H_
