﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_
#define INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_

#include "nn/nlib/OutputStream.h"

NLIB_NAMESPACE_BEGIN

class NLIB_VIS_PUBLIC OutputTransform {
    // define a derived class and use it from OutputConverterStream
 public:
    virtual ~OutputTransform() {}
    virtual errno_t Transform(OutputStream* os, const void* p, size_t nBytes, bool doFlush) = 0;

    // called only once at first, and pass the workbuffer which OutputConverterStream uses.
    // The derived class has to manage the memory.
    virtual void* GetWorkBuffer(size_t* n) = 0;

    // called when OutputConverterStream::SetStream() is called.
    virtual errno_t OnSetStream(OutputStream* os) = 0;

    // called when the stream is closing.
    virtual errno_t OnClose(OutputStream* os) = 0;
};

class NLIB_VIS_PUBLIC OutputConverterStream : public OutputStream {
 public:
    OutputConverterStream() NLIB_NOEXCEPT;

    // Note that SetTransform returns ENOMEM when cvtr->GetWorkBuffer() fails
    errno_t SetTransform(OutputTransform* cvtr) NLIB_NOEXCEPT;
    errno_t SetStream(OutputStream* ostr) NLIB_NOEXCEPT;
    OutputStream* GetStream() const NLIB_NOEXCEPT { return m_Ostr; }

    // NOTE:
    // The destructor does not close its m_Ostr, nor call OutputTransform::OnClose().
    // it is because OutputConverterStream does not own m_Ostr nor m_Cvtr.
    virtual ~OutputConverterStream() NLIB_NOEXCEPT NLIB_OVERRIDE {}

 private:
    virtual bool PushBuffer_(const void* p, size_t nBytes,
                             bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;
    virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;
    virtual void* GetWorkBuffer_(size_t* nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE NLIB_FINAL;

 private:
    OutputStream* m_Ostr;
    OutputTransform* m_Cvtr;
    size_t m_BufSize;
    unsigned char* m_Buf;

    NLIB_DISALLOW_COPY_AND_ASSIGN(OutputConverterStream);
};

template <class TRANSFORM>
class OutputConverterStreamTempl : public OutputConverterStream {
 public:
    typedef TRANSFORM TransformType;

    OutputConverterStreamTempl() NLIB_NOEXCEPT {
        this->SetTransform(&m_Transform);
    }
    virtual ~OutputConverterStreamTempl() NLIB_NOEXCEPT NLIB_OVERRIDE {
        // note that OutputConverterStream::Close_() is final
        bool result = this->Close();
        NLIB_UNUSED(result);
    }

 private:
    void SetTransform(OutputTransform* cvtr) NLIB_NOEXCEPT {
        // to hide SetTransform()
        errno_t e = OutputConverterStream::SetTransform(cvtr);
        if (e != 0) this->SetError(e);
    }

 protected:
    TRANSFORM m_Transform;

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(OutputConverterStreamTempl);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_OUTPUTCONVERTERSTREAM_H_
