﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
#define INCLUDE_NN_NLIB_PLATFORM_H_

#ifdef __ILP64__
# error Sorry, ILP64 not supported
#endif

// Note that the C preprocessor # operator must be able to use.
#define NLIB_VERSION_YEAR 2015
#define NLIB_VERSION_YEAR_SHORT 15
#define NLIB_VERSION_DATE 0908
#define NLIB_VERSION 20150908

#ifdef __cplusplus
# ifndef __STDC_LIMIT_MACROS
#  ifdef _MSC_VER
#   pragma message(__FILE__ ": __STDC_LIMIT_MACROS not defined, compile may fail")
#  elif !defined(NN_PLATFORM_CTR) && !defined(CAFE)
#   warning __STDC_LIMIT_MACROS not defined, compile may fail
#  endif
#  define __STDC_LIMIT_MACROS
# endif
# ifndef __STDC_CONSTANT_MACROS
#  ifdef _MSC_VER
#   pragma message(__FILE__ ": __STDC_CONSTANT_MACROS not defined, compile may fail")
#  elif !defined(NN_PLATFORM_CTR) && !defined(CAFE)
#   warning __STDC_CONSTANT_MACROS not defined, compile may fail
#  endif
#  define __STDC_CONSTANT_MACROS
# endif
# ifndef __STDC_FORMAT_MACROS
#  ifdef _MSC_VER
#   pragma message(__FILE__ ": __STDC_FORMAT_MACROS not defined, compile may fail")
#  elif !defined(NN_PLATFORM_CTR) && !defined(CAFE)
#   warning __STDC_FORMAT_MACROS not defined, compile may fail
#  endif
#  define __STDC_FORMAT_MACROS
# endif
#endif

#include <stddef.h>  // for size_t
#include <stdio.h>   // for SEEK_CUR, SEEK_END, SEEK_SET
#include <stdarg.h>  // for va_list
#include <string.h>  // for memcpy, memmove, memset

#if defined(NLIB_RENAME_CAPI) && !defined(_MSC_VER)
#ifndef NLIB_CAPI
# define NLIB_CAPI(x) mypfx##x
#endif

#define nlib_error_string NLIB_CAPI(nlib_error_string)
#define nlib_get_native_last_error NLIB_CAPI(nlib_get_native_last_error)
#define nlib_getversion NLIB_CAPI(nlib_getversion)
#define nlib_epochtime NLIB_CAPI(nlib_epochtime)
#define nlib_ticktime NLIB_CAPI(nlib_ticktime)
#define nlib_sleep NLIB_CAPI(nlib_sleep)
#define nlib_gen_random NLIB_CAPI(nlib_gen_random)
#define nlib_mempagesize NLIB_CAPI(nlib_mempagesize)
#define nlib_virtual_alloc NLIB_CAPI(nlib_virtual_alloc)
#define nlib_virtual_free NLIB_CAPI(nlib_virtual_free)
#define nlib_physical_alloc NLIB_CAPI(nlib_physical_alloc)
#define nlib_physical_free NLIB_CAPI(nlib_physical_free)
#define nlib_mlock NLIB_CAPI(nlib_mlock)
#define nlib_munlock NLIB_CAPI(nlib_munlock)
#define nlib_tls_alloc NLIB_CAPI(nlib_tls_alloc)
#define nlib_tls_free NLIB_CAPI(nlib_tls_free)
#define nlib_tls_setvalue NLIB_CAPI(nlib_tls_setvalue)
#define nlib_tls_getvalue NLIB_CAPI(nlib_tls_getvalue)
#define nlib_mutex_init NLIB_CAPI(nlib_mutex_init)
#define nlib_mutex_recursive_init NLIB_CAPI(nlib_mutex_recursive_init)
#define nlib_mutex_recursive_timed_init NLIB_CAPI(nlib_mutex_recursive_timed_init)
#define nlib_mutex_lock NLIB_CAPI(nlib_mutex_lock)
#define nlib_mutex_trylock NLIB_CAPI(nlib_mutex_trylock)
#define nlib_mutex_trylock_for NLIB_CAPI(nlib_mutex_trylock_for)
#define nlib_mutex_unlock NLIB_CAPI(nlib_mutex_unlock)
#define nlib_mutex_destroy NLIB_CAPI(nlib_mutex_destroy)
#define nlib_semaphore_init NLIB_CAPI(nlib_semaphore_init)
#define nlib_semaphore_wait NLIB_CAPI(nlib_semaphore_wait)
#define nlib_semaphore_trywait NLIB_CAPI(nlib_semaphore_trywait)
#define nlib_semaphore_trywait_for NLIB_CAPI(nlib_semaphore_trywait_for)
#define nlib_semaphore_post NLIB_CAPI(nlib_semaphore_post)
#define nlib_semaphore_post_ex NLIB_CAPI(nlib_semaphore_post_ex)
#define nlib_semaphore_destroy NLIB_CAPI(nlib_semaphore_destroy)
#define nlib_cond_init NLIB_CAPI(nlib_cond_init)
#define nlib_cond_signal NLIB_CAPI(nlib_cond_signal)
#define nlib_cond_broadcast NLIB_CAPI(nlib_cond_broadcast)
#define nlib_cond_wait NLIB_CAPI(nlib_cond_wait)
#define nlib_cond_wait_for NLIB_CAPI(nlib_cond_wait_for)
#define nlib_cond_wait_until NLIB_CAPI(nlib_cond_wait_until)
#define nlib_cond_destroy NLIB_CAPI(nlib_cond_destroy)
#define nlib_rwlock_init NLIB_CAPI(nlib_rwlock_init)
#define nlib_rwlock_destroy NLIB_CAPI(nlib_rwlock_destroy)
#define nlib_rwlock_rdlock NLIB_CAPI(nlib_rwlock_rdlock)
#define nlib_rwlock_tryrdlock NLIB_CAPI(nlib_rwlock_tryrdlock)
#define nlib_rwlock_tryrdlock_for NLIB_CAPI(nlib_rwlock_tryrdlock_for)
#define nlib_rwlock_tryrdlock_until NLIB_CAPI(nlib_rwlock_tryrdlock_until)
#define nlib_rwlock_rdunlock NLIB_CAPI(nlib_rwlock_rdunlock)
#define nlib_rwlock_wrlock NLIB_CAPI(nlib_rwlock_wrlock)
#define nlib_rwlock_trywrlock NLIB_CAPI(nlib_rwlock_trywrlock)
#define nlib_rwlock_trywrlock_for NLIB_CAPI(nlib_rwlock_trywrlock_for)
#define nlib_rwlock_trywrlock_until NLIB_CAPI(nlib_rwlock_trywrlock_until)
#define nlib_rwlock_wrunlock NLIB_CAPI(nlib_rwlock_wrunlock)
#define nlib_condrwlock_init NLIB_CAPI(nlib_condrwlock_init)
#define nlib_condrwlock_destroy NLIB_CAPI(nlib_condrwlock_destroy)
#define nlib_condrwlock_signal NLIB_CAPI(nlib_condrwlock_signal)
#define nlib_condrwlock_broadcast NLIB_CAPI(nlib_condrwlock_broadcast)
#define nlib_condrwlock_wait NLIB_CAPI(nlib_condrwlock_wait)
#define nlib_condrwlock_wait_for NLIB_CAPI(nlib_condrwlock_wait_for)
#define nlib_condrwlock_wait_until NLIB_CAPI(nlib_condrwlock_wait_until)
#define nlib_barrier_init NLIB_CAPI(nlib_barrier_init)
#define nlib_barrier_destroy NLIB_CAPI(nlib_barrier_destroy)
#define nlib_barrier_wait NLIB_CAPI(nlib_barrier_wait)
#define nlib_once NLIB_CAPI(nlib_once)
#define nlib_tryonce NLIB_CAPI(nlib_tryonce)
#define nlib_mq_open NLIB_CAPI(nlib_mq_open)
#define nlib_mq_getattr NLIB_CAPI(nlib_mq_getattr)
#define nlib_mq_close NLIB_CAPI(nlib_mq_close)
#define nlib_mq_readonly NLIB_CAPI(nlib_mq_readonly)
#define nlib_mq_send NLIB_CAPI(nlib_mq_send)
#define nlib_mq_send_until NLIB_CAPI(nlib_mq_send_until)
#define nlib_mq_receive NLIB_CAPI(nlib_mq_receive)
#define nlib_mq_receive_until NLIB_CAPI(nlib_mq_receive_until)
#define nlib_mq_drop NLIB_CAPI(nlib_mq_drop)
#define nlib_yield NLIB_CAPI(nlib_yield)
#define nlib_thread_create NLIB_CAPI(nlib_thread_create)
#define nlib_thread_join NLIB_CAPI(nlib_thread_join)
#define nlib_thread_detach NLIB_CAPI(nlib_thread_detach)
#define nlib_thread_self NLIB_CAPI(nlib_thread_self)
#define nlib_thread_getconcurrency NLIB_CAPI(nlib_thread_getconcurrency)
#define nlib_thread_getid NLIB_CAPI(nlib_thread_getid)
#define nlib_thread_equal NLIB_CAPI(nlib_thread_equal)
#define nlib_thread_getcpu NLIB_CAPI(nlib_thread_getcpu)
#define nlib_thread_setaffinity NLIB_CAPI(nlib_thread_setaffinity)
#define nlib_thread_setname NLIB_CAPI(nlib_thread_setname)
#define nlib_thread_attr_init NLIB_CAPI(nlib_thread_attr_init)
#define nlib_thread_attr_setint NLIB_CAPI(nlib_thread_attr_setint)
#define nlib_thread_attr_getint NLIB_CAPI(nlib_thread_attr_getint)
#define nlib_thread_attr_setptr NLIB_CAPI(nlib_thread_attr_setptr)
#define nlib_thread_attr_getptr NLIB_CAPI(nlib_thread_attr_getptr)
#define nlib_thread_attr_setstack NLIB_CAPI(nlib_thread_attr_setstack)
#define nlib_thread_attr_getstack NLIB_CAPI(nlib_thread_attr_getstack)
#define nlib_thread_attr_destroy NLIB_CAPI(nlib_thread_attr_destroy)
#define nlib_thread_getpriority NLIB_CAPI(nlib_thread_getpriority)
#define nlib_thread_setpriority NLIB_CAPI(nlib_thread_setpriority)
#define nlib_thread_priority_min NLIB_CAPI(nlib_thread_priority_min)
#define nlib_thread_priority_max NLIB_CAPI(nlib_thread_priority_max)
#define nlib_thread_priority_default NLIB_CAPI(nlib_thread_priority_default)
#define nlib_thread_exit NLIB_CAPI(nlib_thread_exit)
#define nlib_thread_cleanup_push_ NLIB_CAPI(nlib_thread_cleanup_push_)
#define nlib_thread_cleanup_pop_ NLIB_CAPI(nlib_thread_cleanup_pop_)
#define nlib_write_stdout NLIB_CAPI(nlib_write_stdout)
#define nlib_write_stderr NLIB_CAPI(nlib_write_stderr)
#define nlib_debug_break NLIB_CAPI(nlib_debug_break)
#define nlib_debug_backtrace NLIB_CAPI(nlib_debug_backtrace)
#define nlib_debug_backtrace_gettext NLIB_CAPI(nlib_debug_backtrace_gettext)
#define nlib_getenv NLIB_CAPI(nlib_getenv)
#define nlib_log_print NLIB_CAPI(nlib_log_print)
#define nlib_log_vprint NLIB_CAPI(nlib_log_vprint)
#define nlib_log_attr_setint NLIB_CAPI(nlib_log_attr_setint)
#define nlib_fd_open NLIB_CAPI(nlib_fd_open)
#define nlib_fd_close NLIB_CAPI(nlib_fd_close)
#define nlib_fd_read NLIB_CAPI(nlib_fd_read)
#define nlib_fd_write NLIB_CAPI(nlib_fd_write)
#define nlib_fd_seek NLIB_CAPI(nlib_fd_seek)
#define nlib_fd_pread NLIB_CAPI(nlib_fd_pread)
#define nlib_fd_pwrite NLIB_CAPI(nlib_fd_pwrite)
#define nlib_fd_truncate NLIB_CAPI(nlib_fd_truncate)
#define nlib_fd_getsize NLIB_CAPI(nlib_fd_getsize)
#define nlib_fd_flush NLIB_CAPI(nlib_fd_flush)
#define nlib_fd_sync NLIB_CAPI(nlib_fd_sync)
#define nlib_fd_native_handle NLIB_CAPI(nlib_fd_native_handle)
#define nlib_fd_readv NLIB_CAPI(nlib_fd_readv)
#define nlib_fd_writev NLIB_CAPI(nlib_fd_writev)
#define nlib_fd_preadv NLIB_CAPI(nlib_fd_preadv)
#define nlib_fd_pwritev NLIB_CAPI(nlib_fd_pwritev)
#define nlib_unlink NLIB_CAPI(nlib_unlink)
#define nlib_mkdir NLIB_CAPI(nlib_mkdir)
#define nlib_rmdir NLIB_CAPI(nlib_rmdir)
#define nlib_rename NLIB_CAPI(nlib_rename)
#define nlib_dir_open NLIB_CAPI(nlib_dir_open)
#define nlib_dir_close NLIB_CAPI(nlib_dir_close)
#define nlib_dir_read NLIB_CAPI(nlib_dir_read)
#define nlib_is_dir NLIB_CAPI(nlib_is_dir)
#define nlib_exist_path NLIB_CAPI(nlib_exist_path)
#define nlib_disk_freespace NLIB_CAPI(nlib_disk_freespace)
#define nlib_spinlock_lock_ NLIB_CAPI(nlib_spinlock_lock_)
#define nlib_spinlock_unlock_ NLIB_CAPI(nlib_spinlock_unlock_)
#define nlib_vsnprintf NLIB_CAPI(nlib_vsnprintf)
#define nlib_snprintf NLIB_CAPI(nlib_snprintf)
#define nlib_vdprintf NLIB_CAPI(nlib_vdprintf)
#define nlib_dprintf NLIB_CAPI(nlib_dprintf)
#define nlib_printf NLIB_CAPI(nlib_printf)
#define nlib_vsnwprintf NLIB_CAPI(nlib_vsnwprintf)
#define nlib_snwprintf NLIB_CAPI(nlib_snwprintf)
#define nlib_vdwprintf NLIB_CAPI(nlib_vdwprintf)
#define nlib_dwprintf NLIB_CAPI(nlib_dwprintf)
#define nlib_wprintf NLIB_CAPI(nlib_wprintf)
#define nlib_vsnprintf_fallback NLIB_CAPI(nlib_vsnprintf_fallback)
#define nlib_snprintf_fallback NLIB_CAPI(nlib_snprintf_fallback)
#define nlib_vsnwprintf_fallback NLIB_CAPI(nlib_vsnwprintf_fallback)
#define nlib_snwprintf_fallback NLIB_CAPI(nlib_snwprintf_fallback)
#define nlib_memcmp NLIB_CAPI(nlib_memcmp)
#define nlib_memchr NLIB_CAPI(nlib_memchr)
#define nlib_memrchr NLIB_CAPI(nlib_memrchr)
#define nlib_memchr_not NLIB_CAPI(nlib_memchr_not)
#define nlib_memchr_range_not NLIB_CAPI(nlib_memchr_range_not)
#define nlib_memchr_lt NLIB_CAPI(nlib_memchr_lt)
#define nlib_memchr_gt NLIB_CAPI(nlib_memchr_gt)
#define nlib_memchr_mb NLIB_CAPI(nlib_memchr_mb)
#define nlib_memchr_set NLIB_CAPI(nlib_memchr_set)
#define nlib_memchr_set_not NLIB_CAPI(nlib_memchr_set_not)
#define nlib_skipws NLIB_CAPI(nlib_skipws)
#define nlib_strlen NLIB_CAPI(nlib_strlen)
#define nlib_strnlen NLIB_CAPI(nlib_strnlen)
#define nlib_strcpy NLIB_CAPI(nlib_strcpy)
#define nlib_strncpy NLIB_CAPI(nlib_strncpy)
#define nlib_strchr NLIB_CAPI(nlib_strchr)
#define nlib_strrchr NLIB_CAPI(nlib_strrchr)
#define nlib_strchr_mb NLIB_CAPI(nlib_strchr_mb)
#define nlib_wcslen NLIB_CAPI(nlib_wcslen)
#define nlib_wcsnlen NLIB_CAPI(nlib_wcsnlen)
#define nlib_wcscpy NLIB_CAPI(nlib_wcscpy)
#define nlib_wcsncpy NLIB_CAPI(nlib_wcsncpy)
#define nlib_strcat NLIB_CAPI(nlib_strcat)
#define nlib_strncat NLIB_CAPI(nlib_strncat)
#define nlib_wcscat NLIB_CAPI(nlib_wcscat)
#define nlib_wcsncat NLIB_CAPI(nlib_wcsncat)
#define nlib_wide_to_utf8 NLIB_CAPI(nlib_wide_to_utf8)
#define nlib_utf8_to_wide NLIB_CAPI(nlib_utf8_to_wide)
#define nlib_strcplen NLIB_CAPI(nlib_strcplen)
#define nlib_strcplen_ex NLIB_CAPI(nlib_strcplen_ex)
#define nlib_wcscplen NLIB_CAPI(nlib_wcscplen)
#define nlib_swapendian_16 NLIB_CAPI(nlib_swapendian_16)
#define nlib_swapendian_32 NLIB_CAPI(nlib_swapendian_32)
#define nlib_swapendian_64 NLIB_CAPI(nlib_swapendian_64)
#define nlib_malloc NLIB_CAPI(nlib_malloc)
#define nlib_free NLIB_CAPI(nlib_free)
#define nlib_calloc NLIB_CAPI(nlib_calloc)
#define nlib_realloc NLIB_CAPI(nlib_realloc)
#define nlib_free_size NLIB_CAPI(nlib_free_size)
#define nlib_memalign NLIB_CAPI(nlib_memalign)
#define nlib_popcnt NLIB_CAPI(nlib_popcnt)
#define nlib_popcnt64 NLIB_CAPI(nlib_popcnt64)
#define nlib_popcnt16 NLIB_CAPI(nlib_popcnt16)
#endif

#if defined(_MSC_VER)
# include "nn/nlib/Platform_win32.h"
#elif defined(__linux__)   || \
      defined(__FreeBSD__) || \
      defined(__CYGWIN__)  || \
     (defined(__APPLE__) && defined(__MACH__))
# ifndef NLIB_UNIX
#  define NLIB_UNIX
# endif
# include "nn/nlib/Platform_unix.h"
#elif defined(NN_PLATFORM_CTR)
# include "nn/nlib/Platform_ctr.h"
#elif defined(CAFE)
# ifndef NLIB_CAFE_PPC
#  define NLIB_CAFE_PPC
# endif
# include "nn/nlib/Platform_cafe.h"
#endif

#if defined(__ARM_NEON__) || defined(__aarch64__)
# ifndef NLIB_NEON
#  define NLIB_NEON
# endif
#endif

#ifdef __SSE4_1__
# ifndef NLIB_SSE41
#  define NLIB_SSE41
# endif
#endif

#if defined(NLIB_SSE41) || defined(NLIB_NEON)
# define NLIB_SIMD
# ifdef NLIB_NEON
#  include <arm_neon.h>  // NOLINT
# endif
# ifdef NLIB_SSE41
// Use _mm_popcnt_u32() only if the executing processor supports SSE4.2
#  include <nmmintrin.h>  // NOLINT
# endif
#endif

// https://www.jpcert.or.jp/sc-rules/c-int01-c.html
// https://www.securecoding.cert.org/confluence/display/seccode/INT01-C.+Use+rsize_t+or+size_t+for+all+integer+values+representing+the+size+of+an+object
#ifndef RSIZE_MAX
# ifndef NLIB_64BIT
#  define RSIZE_MAX 0x7FFFFFFFL
# else
#  define RSIZE_MAX 0x7FFFFFFFFFFFFFFFLL
# endif
#endif

#ifndef NLIB_WARN
# define NLIB_WARN(exp) ("WARNING: " exp)
// #pragma message NLIB_WARN("your message")
#endif

// NOTE:
// You can use those macros on the environment which printf does not support '%z'
// for size_t.
// "%"PRIuS, sizet_val
// See: https://google-styleguide.googlecode.com/svn/trunk/cppguide.html#64-bit_Portability
// In Japanese: http://www.textdrop.net/google-styleguide-ja/cppguide.xml
#ifndef __PRIS_PREFIX
# define __PRIS_PREFIX "z"
#endif

#ifndef PRIdS
# define PRIdS __PRIS_PREFIX "d"
#endif

#ifndef PRIxS
# define PRIxS __PRIS_PREFIX "x"
#endif

#ifndef PRIuS
# define PRIuS __PRIS_PREFIX "u"
#endif

#ifndef PRIXS
# define PRIXS __PRIS_PREFIX "X"
#endif

#ifndef PRIoS
# define PRIoS __PRIS_PREFIX "o"
#endif

#ifndef NLIB_LOCKABLE
# define NLIB_LOCKABLE
#endif

#ifndef NLIB_LOCK_FUNC
# define NLIB_LOCK_FUNC(...)
#endif

#ifndef NLIB_SHARED_LOCK_FUNC
# define NLIB_SHARED_LOCK_FUNC(...)
#endif

#ifndef NLIB_SHARED_TRYLOCK_FUNC
# define NLIB_SHARED_TRYLOCK_FUNC(...)
#endif

#ifndef NLIB_UNLOCK_FUNC
# define NLIB_UNLOCK_FUNC(...)
#endif

#ifndef NLIB_SHARED_UNLOCK_FUNC
# define NLIB_SHARED_UNLOCK_FUNC(...)
#endif

#ifndef NLIB_TRYLOCK_FUNC
# define NLIB_TRYLOCK_FUNC(...)
#endif

#ifndef NLIB_GUARDED_BY
# define NLIB_GUARDED_BY(x)
#endif

#ifndef NLIB_PT_GUARDED_BY
# define NLIB_PT_GUARDED_BY(x)
#endif

#ifndef NLIB_LOCK_REQUIRED
# define NLIB_LOCK_REQUIRED(...)
#endif

#ifndef NLIB_LOCK_EXCLUDED
# define NLIB_LOCK_EXCLUDED(...)
#endif

#ifndef NLIB_SHARED_LOCK_REQUIRED
# define NLIB_SHARED_LOCK_REQUIRED(...)
#endif

#ifndef NLIB_SCOPED_LOCKABLE
# define NLIB_SCOPED_LOCKABLE
#endif

#ifndef NLIB_NO_THREAD_SAFETY_ANALYSIS
# define NLIB_NO_THREAD_SAFETY_ANALYSIS
#endif

#ifndef NLIB_ASSUME
# define NLIB_ASSUME(cond) switch (0) case 0: default: if (cond) ; else __builtin_unreachable()  /* NOLINT */
#endif

#ifdef NLIB_NONNULL_ENABLED
# define NLIB_EINVAL_IFNULL(p)
#else
# define NLIB_EINVAL_IFNULL(p) if (!p) return EINVAL
#endif

//
// Error Type
//
typedef int errno_t;  // TR 24731-1

//
// stdint
//
#ifdef NLIB_HAS_STDHEADER_STDINT
# include <stdint.h>  // NOLINT
#endif

#ifdef NLIB_HAS_STDHEADER_INTTYPES
# include <inttypes.h>  // NOLINT
#endif

#if defined(__LP64__) && __LP64__ == 1
// if long, unsinged long are 64 bits long
#define NLIB_LP64
typedef int64_t nlib_long_compatible_t;
typedef uint64_t nlib_ulong_compatible_t;
#else
typedef int32_t nlib_long_compatible_t;
typedef uint32_t nlib_ulong_compatible_t;
#endif

#ifndef NLIB_VIS_PUBLIC_ALT
#define NLIB_VIS_PUBLIC_ALT NLIB_VIS_PUBLIC
#endif

#ifdef __cplusplus
extern "C" {
#endif

NLIB_VIS_PUBLIC const char* nlib_error_string(errno_t e);

//
// Native error
//
NLIB_VIS_PUBLIC unsigned int nlib_get_native_last_error(void);

//
// Version
//

// returns NLIB_VERSION
NLIB_VIS_PUBLIC int nlib_getversion(void);

//
// Atomic
//
#if defined(__INTELLISENSE__)
#define NLIB_ATOMIC_RELAXED (0)
#define NLIB_ATOMIC_ACQUIRE (1)
#define NLIB_ATOMIC_RELEASE (2)
#define NLIB_ATOMIC_ACQ_REL (3)
#define NLIB_ATOMIC_SEQ_CST (7)

int32_t nlib_atomic_load32(const int32_t* ptr, int memorder);
void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder);
// *target = value, and returns the old value of *target
int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                   int32_t desired, int weak,
                                   int success_memorder, int failure_memorder);
// *ptr += val; return *ptr;
int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr -= val; return *ptr;
int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr &= val; return *ptr;
int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr ^= val; return *ptr;
int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder);
// *ptr |= val; return *ptr;
int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr += val; return tmp;
int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr -= val; return tmp;
int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr &= val; return tmp;
int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr ^= val; return tmp;
int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder);
// tmp = *ptr; *ptr |= val; return tmp;
int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder);

int64_t nlib_atomic_load64(const int64_t* ptr, int memorder);
void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder);
// *target = value, and returns the old value of *target
int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                   int64_t desired, int weak,
                                   int success_memorder, int failure_memorder);
// *ptr += val; return *ptr;
int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr -= val; return *ptr;
int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr &= val; return *ptr;
int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr ^= val; return *ptr;
int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder);
// *ptr |= val; return *ptr;
int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr += val; return tmp;
int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr -= val; return tmp;
int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr &= val; return tmp;
int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr ^= val; return tmp;
int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder);
// tmp = *ptr; *ptr |= val; return tmp;
int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder);

void* nlib_atomic_loadptr(void* const* ptr, int memorder);
void nlib_atomic_storeptr(void** ptr, void* val, int memorder);
// *ptr = desired and return non-zero if successful
int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                    void* desired, int weak,
                                    int success_memorder, int failure_memorder);

void nlib_atomic_thread_fence(int memorder);
#endif

//
// Time, Duration
//

// 100ns => 1, 1970/01/01 == 0
typedef int64_t nlib_time;
// 100ns => 1, 1ms => 10000
typedef int64_t nlib_duration;

// 100ns => 1, 1970/01/01 == 0
NLIB_VIS_PUBLIC errno_t nlib_epochtime(nlib_time* t) NLIB_NONNULL;
// 100ns => 1, boot time == 0, msec = *t / 10000
NLIB_VIS_PUBLIC errno_t nlib_ticktime(nlib_duration* t) NLIB_NONNULL;
// 100ns => 1, sleep 1 msec = nlib_sleep(10000)
NLIB_VIS_PUBLIC errno_t nlib_sleep(nlib_duration t);

//
// Random
//

// Store 'size' bytes of random values on 'buf'
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_gen_random(void* buf, size_t size) NLIB_NONNULL;

//
// Virtual Memory, Physical Memory
//
NLIB_VIS_PUBLIC errno_t nlib_mempagesize(size_t* size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_virtual_alloc(void** ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_virtual_free(void* ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_physical_alloc(void* ptr, size_t size, int prot) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_physical_free(void* ptr, size_t size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mlock(void* addr, size_t len) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_munlock(void* addr, size_t len) NLIB_NONNULL;

#define NLIB_PHYSICAL_ALLOC_PROT_NONE 0
#define NLIB_PHYSICAL_ALLOC_PROT_READ 1
#define NLIB_PHYSICAL_ALLOC_PROT_WRITE 2
#define NLIB_PHYSICAL_ALLOC_PROT_EXEC 4

//
// TLS
//
#ifdef NLIB_UNIX
typedef pthread_key_t nlib_tls;
#else
typedef unsigned int nlib_tls;
#endif
typedef void (*nlib_tls_destructor)(void* tls_value);
// code snippets:
//   # map tls_key on thread local storage
//     nlib_tls tls_key;
//     e = nlib_tls_alloc(&tls_key, NULL);  # no dtor invoked if destr is NULL
//     if (e != 0) { error ... }
//   # unmap tls_key
//     nlib_tls_free(tls_key);
//   # access from a thread
//     void* thread_local_value;
//     nlib_tls_getvalue(tls_key, &thread_local_value);
//     # use and update thread_local_value
//     nlib_tls_setvalue(tls_key, thread_local_value);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_tls_alloc(nlib_tls* tls,
                                                         nlib_tls_destructor destr) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_tls_free(nlib_tls tls);
NLIB_VIS_PUBLIC errno_t nlib_tls_setvalue(nlib_tls tls, const void* value);
NLIB_VIS_PUBLIC errno_t nlib_tls_getvalue(nlib_tls tls, void** value) NLIB_NONNULL;

//
// Mutex
//

// you can use NLIB_MUTEX_INITIALIZER static initializer
NLIB_VIS_PUBLIC errno_t nlib_mutex_init(nlib_mutex* mutex) NLIB_NONNULL;
// you can use NLIB_RECURSIVE_MUTEX_INITIALIZER static initializer
NLIB_VIS_PUBLIC errno_t nlib_mutex_recursive_init(nlib_mutex* mutex) NLIB_NONNULL;
// you can use NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER static initializer
NLIB_VIS_PUBLIC errno_t nlib_mutex_recursive_timed_init(nlib_mutex* mutex) NLIB_NONNULL;

// code snippets:
//   nlib_mutex m;
//   if (nlib_mutex_init(&m) != 0) { ... } # always returns 0 on almost all platforms?
//   nlib_mutex_lock(&m);
//      ....
//   nlib_mutex_unlock(&m);
//   nlib_mutex_destroy(&m);
NLIB_VIS_PUBLIC errno_t nlib_mutex_lock(nlib_mutex* mutex) NLIB_NONNULL;
// returns EBUSY if a lock cannot be acquired
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mutex_trylock(nlib_mutex* mutex) NLIB_NONNULL;
// returns ETIMEDOUT if timeout
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mutex_trylock_for(nlib_mutex* mutex,
                                                                 nlib_duration delta) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mutex_unlock(nlib_mutex* mutex) NLIB_NONNULL;
// don't forget to write this, some platforms require this called.
NLIB_VIS_PUBLIC errno_t nlib_mutex_destroy(nlib_mutex* mutex) NLIB_NONNULL;

//
// Semaphore
//

NLIB_VIS_PUBLIC errno_t nlib_semaphore_init(nlib_semaphore* sem, int initial_count) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_wait(nlib_semaphore* sem) NLIB_NONNULL;
// returns EAGAIN if semaphore cannot be acquired
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_semaphore_trywait(nlib_semaphore* sem) NLIB_NONNULL;
// returns ETIMEDOUT if timeout
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_semaphore_trywait_for(
    nlib_semaphore* sem, nlib_duration duration) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_post(nlib_semaphore* __restrict sem,
                                            int* __restrict previous_count) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_post_ex(nlib_semaphore* __restrict sem, int release_count,
                                               int* __restrict previous_count) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_semaphore_destroy(nlib_semaphore* sem) NLIB_NONNULL;

//
// Condition Variable
//

// you can use NLIB_COND_INITIALIZER for static initializer
NLIB_VIS_PUBLIC errno_t nlib_cond_init(nlib_cond* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_cond_signal(nlib_cond* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_cond_broadcast(nlib_cond* cond) NLIB_NONNULL;
// code snippets:
//   Initialization:
//     bool flag = false;
//     nlib_mutex m;
//     nlib_cond cond;
//     nlib_mutex_init(&m);
//     nlib_cond_init(&cond);
//   Thread1:
//     nlib_mutex_lock(&m);
//     while (!flag)
//       e = nlib_cond_wait(&cond, &m);  # m to be unlocked in nlib_cond_wait
//       # note that nlib_cond_wait may return without signal notified
//       if (e != 0) { error .... }
//     # do job and reset flag
//     flag = false;
//     nlib_mutex_unlock(&m);
//   Thread2:
//     nlib_mutex_lock(&m);
//     flag = true;
//     nlib_cond_broadcast(&cond);
//     nlib_mutex_unlock(&m);
NLIB_VIS_PUBLIC errno_t nlib_cond_wait(nlib_cond* __restrict cond,
                                       nlib_mutex* __restrict mutex) NLIB_NONNULL;
// returns ETIMEDOUT if timeout, and see baloon for nlib_cond_wait()
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_cond_wait_for(nlib_cond* __restrict cond,
                                                             nlib_mutex* __restrict mutex,
                                                             nlib_duration duration) NLIB_NONNULL;
// returns ETIMEDOUT if timeout, and see baloon for nlib_cond_wait()
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_cond_wait_until(nlib_cond* __restrict cond,
                                                               nlib_mutex* __restrict mutex,
                                                               nlib_time abstime) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_cond_destroy(nlib_cond* cond) NLIB_NONNULL;

//
// Read/Write lock
//
#ifndef NLIB_RWLOCK_HAS_NATIVE
struct nlib_rwlock_ {
    int32_t _0[3];
    nlib_mutex _1[2];
    nlib_cond _2;
};
typedef struct nlib_rwlock_ nlib_rwlock;

#define NLIB_RWLOCK_INITIALIZER {                                                       \
    { 0, 0, 0 },                                                                        \
    { NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER, NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER }, \
      NLIB_COND_INITIALIZER }
#endif

NLIB_VIS_PUBLIC errno_t nlib_rwlock_init(nlib_rwlock* rwlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_destroy(nlib_rwlock* rwlock) NLIB_NONNULL;

NLIB_VIS_PUBLIC errno_t nlib_rwlock_rdlock(nlib_rwlock* rwlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_tryrdlock(nlib_rwlock* rwlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_tryrdlock_for(nlib_rwlock* rwlock,
                                                  nlib_duration duration) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_tryrdlock_until(nlib_rwlock* rwlock,
                                                    nlib_time abstime) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_rdunlock(nlib_rwlock* rwlock) NLIB_NONNULL;

NLIB_VIS_PUBLIC errno_t nlib_rwlock_wrlock(nlib_rwlock* rwlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_trywrlock(nlib_rwlock* rwlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_trywrlock_for(nlib_rwlock* rwlock,
                                                  nlib_duration duration) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_trywrlock_until(nlib_rwlock* rwlock,
                                                    nlib_time abstime) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_rwlock_wrunlock(nlib_rwlock* rwlock) NLIB_NONNULL;

#if defined(_MSC_VER) && defined(NLIB_RWLOCK_HAS_NATIVE)
typedef struct nlib_condrwlock_ {
    CONDITION_VARIABLE cond;
} nlib_condrwlock;
#define NLIB_CONDRWLOCK_INITIALIZER { CONDITION_VARIABLE_INIT }
#else
typedef struct nlib_condrwlock_ {
    nlib_cond cond;
    nlib_mutex mutex;
} nlib_condrwlock;
#define NLIB_CONDRWLOCK_INITIALIZER { NLIB_COND_INITIALIZER, NLIB_MUTEX_INITIALIZER }
#endif

NLIB_VIS_PUBLIC errno_t nlib_condrwlock_init(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_destroy(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_signal(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_broadcast(nlib_condrwlock* cond) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait(nlib_condrwlock* __restrict cond,
                                             nlib_rwlock* __restrict rwlock,
                                             int rdlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait_for(nlib_condrwlock* __restrict cond,
                                                 nlib_rwlock* __restrict rwlock,
                                                 nlib_duration duration,
                                                 int rdlock) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_condrwlock_wait_until(nlib_condrwlock* __restrict cond,
                                                   nlib_rwlock* __restrict rwlock,
                                                   nlib_time abstime,
                                                   int rdlock) NLIB_NONNULL;

//
// Barrier
//
#ifndef NLIB_BARRIER_HAS_NATIVE
struct nlib_barrier_ {
    nlib_mutex _0;
    nlib_cond _1;
    unsigned int _2[3];
};
typedef struct nlib_barrier_ nlib_barrier;
#endif

NLIB_VIS_PUBLIC errno_t nlib_barrier_init(nlib_barrier* barrier, unsigned int count) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_barrier_destroy(nlib_barrier* barrier) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_barrier_wait(nlib_barrier* barrier) NLIB_NONNULL;

//
// Once
//
struct nlib_onceflag_ {
    int status;
};
typedef struct nlib_onceflag_ nlib_onceflag;
#define NLIB_ONCE_INIT { 0 }
typedef void (*nlib_oncefunc)(void);

// code snippets
//   void OnceFunc() { .... }
//   nlib_onceflag flag = NLIB_ONCE_INIT;  // should be static initialized
//   nlib_once(&flag, OnceFunc);  // OnceFunc executes only once
NLIB_VIS_PUBLIC errno_t nlib_once(nlib_onceflag* flag, nlib_oncefunc func) NLIB_NONNULL;
// returns EBUSY immedidately if an other thread is executing 'func'
NLIB_VIS_PUBLIC errno_t nlib_tryonce(nlib_onceflag* flag, nlib_oncefunc func) NLIB_NONNULL;

//
// Message Queue
//
#ifdef NLIB_DOXYGEN
typedef int32_t nlib_mq;
#else
typedef struct nlib_mq_ {
    int32_t raw_handle;  // 0 for invalid handle
} nlib_mq;
#endif
typedef void* nlib_mq_msg;

#define NLIB_MQ_BLOCK 0
#define NLIB_MQ_NONBLOCK 1
#define NLIB_MQ_LOCKFREE 2

typedef void (*nlib_mq_msg_destructor)(nlib_mq_msg);
#ifdef NLIB_DOXYGEN
struct nlib_mq_attr {
    int32_t flag;
    int32_t max_msg;
    int32_t cur_msg;
    nlib_mq_msg_destructor destructor;
};
#else
typedef struct nlib_mq_attr_ {
    int32_t flag;  // NLIB_MQ_BLOCK / NLIB_MQ_NONBLOCK
    int32_t max_msg;  // 0 for 128
    int32_t cur_msg;  // blocking mode only
    nlib_mq_msg_destructor destructor;
} nlib_mq_attr;
#endif

NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_open(nlib_mq* mq,
                                                       const nlib_mq_attr* attr) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_getattr(nlib_mq mq,
                                                          nlib_mq_attr* attr) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_mq_close(nlib_mq mq);
NLIB_VIS_PUBLIC errno_t nlib_mq_readonly(nlib_mq mq);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_send(nlib_mq mq, nlib_mq_msg msg, int prio);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_send_until(nlib_mq mq, nlib_mq_msg msg,
                                                             int prio, nlib_time abstime);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_receive(nlib_mq mq, nlib_mq_msg* msg,
                                                          int* prio) NLIB_NONNULL_2;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_receive_until(nlib_mq mq,
                                                                nlib_mq_msg* msg,
                                                                int* prio,
                                                                nlib_time abstime) NLIB_NONNULL_2;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mq_drop(nlib_mq mq, nlib_mq_msg* msg,
                                                       int* prio) NLIB_NONNULL_2;

//
// Thread
//
NLIB_VIS_PUBLIC errno_t nlib_yield(void);
#define NLIB_THREAD_INVALID (nlib_thread)(0)  // NOLINT

#ifndef NLIB_SPINLOCK_HAS_NATIVE
typedef int32_t nlib_spinlock;
#endif

#ifndef NLIB_THREAD_ATTR_HAS_NATIVE
struct nlib_thread_attr_ {
    nlib_spinlock spin;
    int detach_state;
    int priority;
    uint32_t affinity;
    void* stack_addr;
    size_t stack_size;
};
typedef struct nlib_thread_attr_ nlib_thread_attr;
#else
struct nlib_thread_attr_ {
    pthread_attr_t attr;
    uint32_t affinity;
};
typedef struct nlib_thread_attr_ nlib_thread_attr;
#endif
typedef void (*nlib_thread_func)(void* arg);
// -1 for invalid
typedef int nlib_thread_id;

// code snippets:
//   nlib_thread th;
//   if ((e = nlib_thread_create(&th, NULL, myfunc, myarg)) != 0) { error ... }
//   nlib_thread_join(th); # or nlib_thread_detach(th);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_thread_create(
    nlib_thread* __restrict thread,
    const nlib_thread_attr* __restrict attr,
    nlib_thread_func func,
    void* __restrict arg) NLIB_NONNULL_1 NLIB_NONNULL_3;
NLIB_VIS_PUBLIC errno_t nlib_thread_join(nlib_thread thread);
NLIB_VIS_PUBLIC errno_t nlib_thread_detach(nlib_thread thread);
NLIB_VIS_PUBLIC errno_t nlib_thread_self(nlib_thread* thread) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_getconcurrency(unsigned int* num_cpu) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_getid(nlib_thread_id* id) NLIB_NONNULL;
NLIB_VIS_PUBLIC int nlib_thread_equal(nlib_thread th1, nlib_thread th2);
NLIB_VIS_PUBLIC errno_t nlib_thread_getcpu(int* result) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_setaffinity(nlib_thread thread, uint32_t affinity);
// the argument must be string literal
//   nlib_thread_setname("MY THREAD");
NLIB_VIS_PUBLIC errno_t nlib_thread_setname(const char* string_literal) NLIB_NONNULL;

// NOTE:
// win32 does not have GetThreadAffinityMask()
// errno_t nlib_thread_get_affinify(nlib_thread thread, uint32_t* affinity);

NLIB_VIS_PUBLIC errno_t nlib_thread_attr_init(nlib_thread_attr* attr) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_setint(nlib_thread_attr* attr, int key,
                                                int value) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_getint(const nlib_thread_attr* __restrict attr, int key,
                                                int* __restrict value) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_setptr(nlib_thread_attr* __restrict attr, int key,
                                                void* __restrict value) NLIB_NONNULL_1;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_getptr(const nlib_thread_attr* __restrict attr, int key,
                                                void** __restrict value) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_setstack(nlib_thread_attr* __restrict attr,
                                                  void* __restrict stack_addr,
                                                  size_t stack_size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_getstack(const nlib_thread_attr* __restrict attr,
                                                  void** __restrict stack_addr,
                                                  size_t* __restrict stack_size) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_attr_destroy(nlib_thread_attr* attr) NLIB_NONNULL;

#define NLIB_THREAD_ATTR_KEY_DETACHSTATE (1)
#define NLIB_THREAD_ATTR_KEY_STACKSIZE   (2)
#define NLIB_THREAD_ATTR_KEY_PRIORITY    (4)
#define NLIB_THREAD_ATTR_KEY_AFFINITY    (5)

NLIB_VIS_PUBLIC errno_t nlib_thread_getpriority(nlib_thread thread, int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_setpriority(nlib_thread thread, int priority);
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_min(int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_max(int* priority) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_thread_priority_default(int* priority) NLIB_NONNULL;

#ifndef NN_PLATFORM_CTR
// See also nlib_thread_exit_cpp();
NLIB_VIS_PUBLIC void nlib_thread_exit(void) NLIB_NORETURN;
#endif

#ifdef NLIB_DOXYGEN
void nlib_thread_cleanup_push(void (*fn)(void*), void* arg);
void nlib_thread_cleanup_pop(int exec);
#elif defined(pthread_cleanup_push)
# define nlib_thread_cleanup_push(fn, arg) pthread_cleanup_push(fn, arg)
# define nlib_thread_cleanup_pop(exec) pthread_cleanup_pop(exec)
#elif !defined(NN_PLATFORM_CTR)
struct nlib_thread_cleanup_handler_ {
    void (*func)(void*);
    void* arg;
    struct nlib_thread_cleanup_handler_* next;
};
#define nlib_thread_cleanup_push(fn, arg) switch (0) case 0: default: { \
    struct nlib_thread_cleanup_handler_ _thread_cleanup_handler = { fn, arg, NULL }; \
    nlib_thread_cleanup_push_(&_thread_cleanup_handler)
#define nlib_thread_cleanup_pop(exec) nlib_thread_cleanup_pop_(exec); }

NLIB_VIS_PUBLIC void nlib_thread_cleanup_push_(struct nlib_thread_cleanup_handler_* handler);
NLIB_VIS_PUBLIC void nlib_thread_cleanup_pop_(int exec);
#endif

//
// Console/Debug
//

// note that buf is not null terminated
NLIB_VIS_PUBLIC errno_t nlib_write_stdout(size_t* __restrict result, const void* __restrict buf,
                                          size_t count) NLIB_NONNULL;
// note that buf is not null terminated
NLIB_VIS_PUBLIC errno_t nlib_write_stderr(size_t* __restrict result, const void* __restrict buf,
                                          size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC void nlib_debug_break(void);
NLIB_VIS_PUBLIC errno_t nlib_debug_backtrace(size_t* __restrict result, void** __restrict buffer,
                                             size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_debug_backtrace_gettext(char* __restrict str, size_t strbufsize,
                                                     void* const* __restrict buf,
                                                     size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_getenv(size_t* __restrict result, char* __restrict buf, size_t bufsize,
                                    const char* __restrict varname) NLIB_NONNULL_1 NLIB_NONNULL_4;

typedef enum nlib_log_priority {
    NLIB_LOG_UNKNOWN = 0,
    NLIB_LOG_DEAFULT,
    NLIB_LOG_VERBOSE,
    NLIB_LOG_DEBUG,
    NLIB_LOG_INFO,
    NLIB_LOG_WARN,
    NLIB_LOG_ERROR,
    NLIB_LOG_FATAL,
    NLIB_LOG_SILENT,
    NLIB_LOG_LEVEL_EQUAL_OR_ABOVE = 0x10,
    NLIB_LOG_LEVEL_EQUAL_OR_BELOW = 0x20,
    NLIB_LOG_LEVEL_ALL = 0x30
} nlib_log_priority;

typedef enum nlib_log_key {
    NLIB_LOG_ATTR_UNKNOWN = 0,
    NLIB_LOG_ATTR_STDOUT,
    NLIB_LOG_ATTR_STDERR,
    NLIB_LOG_ATTR_MSVC_TRACE,
    NLIB_LOG_ATTR_SYSLOG,
    NLIB_LOG_ATTR_NLIB_FD,
    NLIB_LOG_ATTR_MAX
} nlib_log_key;

#ifndef NLIB_ATTRIBUTE_PRINTF
# define NLIB_ATTRIBUTE_PRINTF(x, y) __attribute__((format(printf, x, y)))
#endif

NLIB_VIS_PUBLIC int nlib_log_print(int prio, _Printf_format_string_ const char* __restrict tag,
                                   const char* __restrict fmt, ...)
                                   NLIB_ATTRIBUTE_PRINTF(3, 4) NLIB_NONNULL;
NLIB_VIS_PUBLIC int nlib_log_vprint(int prio, _Printf_format_string_ const char* __restrict tag,
                                    const char* __restrict fmt, va_list ap) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_log_attr_setint(int prio, int key, int value);

//
// File Access
//

#ifdef O_RDONLY
# define NLIB_FD_O_RDONLY O_RDONLY
#else
# define NLIB_FD_O_RDONLY (0x0000)
#endif

#ifdef O_WRONLY
# define NLIB_FD_O_WRONLY O_WRONLY
#else
# define NLIB_FD_O_WRONLY (0x0001)
#endif

#ifdef O_RDWR
# define NLIB_FD_O_RDWR   O_RDWR
#else
# define NLIB_FD_O_RDWR   (0x0002)
#endif

#ifdef O_CREAT
# define NLIB_FD_O_CREAT  O_CREAT
#else
# define NLIB_FD_O_CREAT  (0x0100)
#endif

#ifdef O_TRUNC
# define NLIB_FD_O_TRUNC  O_TRUNC
#else
# define NLIB_FD_O_TRUNC  (0x0800)
#endif

#ifdef O_APPEND
# define NLIB_FD_O_APPEND O_APPEND
#else
# define NLIB_FD_O_APPEND (0x0008)
#endif

#ifdef SEEK_SET
# define NLIB_SEEK_SET SEEK_SET
#else
# define NLIB_SEEK_SET 0
#endif

#ifdef SEEK_CUR
# define NLIB_SEEK_CUR SEEK_CUR
#else
# define NLIB_SEEK_CUR 1
#endif

// NOTE:
// SEEK_END not supported(because of FIO19-C)

typedef int64_t nlib_offset;
typedef int nlib_fd;
#define NLIB_FD_INVALID (-1)

NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_open(
    nlib_fd* fd, const char* native_path, unsigned int flags, const void* extra)
    NLIB_NONNULL_1 NLIB_NONNULL_2;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_close(nlib_fd fd);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_read(
    size_t* __restrict result, nlib_fd fd, void* __restrict buf, size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_write(
    size_t* __restrict result, nlib_fd fd, const void* __restrict buf, size_t count) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_seek(
    nlib_offset* result, nlib_fd fd, nlib_offset offset, int whence) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_pread(size_t* __restrict result, nlib_fd fd,
                                                        void* __restrict buf, size_t count,
                                                        nlib_offset offset) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_pwrite(size_t* __restrict result, nlib_fd fd,
                                                         const void* __restrict buf,
                                                         size_t count, nlib_offset offset)
                                                         NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_truncate(nlib_fd fd, nlib_offset length);

NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_getsize(nlib_offset* size,
                                                          nlib_fd fd) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_flush(nlib_fd fd);
NLIB_VIS_PUBLIC errno_t nlib_fd_sync(nlib_fd fd);
NLIB_VIS_PUBLIC errno_t nlib_fd_native_handle(void** native_handle, nlib_fd fd) NLIB_NONNULL;
// errno_t nlib_fd_fcntl_getflag(unsigned int* flags, nlib_fd fd);
// errno_t nlib_fd_fcntl_setflag(nlib_fd fd, unsigned int flags);

// Scatter/Gather buffer
struct nlib_fd_iovec_ {
    void* iov_base;
    size_t iov_len;
};
typedef struct nlib_fd_iovec_ nlib_fd_iovec;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_readv(size_t* __restrict result, nlib_fd fd,
                                                        const nlib_fd_iovec* __restrict iov,
                                                        int iovcnt) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_writev(size_t* __restrict result, nlib_fd fd,
                                                         const nlib_fd_iovec* __restrict iov,
                                                         int iovcnt) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_preadv(size_t* __restrict result, nlib_fd fd,
                                                         const nlib_fd_iovec* __restrict iov,
                                                         int iovcnt,
                                                         nlib_offset offset) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_fd_pwritev(size_t* __restrict result, nlib_fd fd,
                                                          const nlib_fd_iovec* __restrict iov,
                                                          int iovcnt,
                                                          nlib_offset offset) NLIB_NONNULL;
// errno_t nlib_fd_stat(stat* stat, nlib_fd fd);

NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_unlink(const char* native_path) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_mkdir(const char* native_path,
                                                     unsigned int flags) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_rmdir(const char* native_path) NLIB_NONNULL;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_rename(const char* __restrict old_path,
                                                      const char* __restrict new_path)
                                                      NLIB_NONNULL;

struct nlib_dir_ {
    void* ptr;
};
typedef struct nlib_dir_ nlib_dir;
typedef struct nlib_dirent_ {
    uint32_t flags;  // 0: file, 1: directory
    char name[768];
} nlib_dirent;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_dir_open(nlib_dir* __restrict dir,
                                                        const char* __restrict native_path)
                                                        NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_dir_close(nlib_dir dir);
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t nlib_dir_read(nlib_dirent* ent,
                                                        nlib_dir dir) NLIB_NONNULL;

// CTR does not have getcwd.....
// errno_t nlib_getcwd(char* buf, size_t bufsize);
// errno_t nlib_chdir(const char* path);

NLIB_VIS_PUBLIC errno_t nlib_is_dir(int* __restrict result,
                                    const char* __restrict native_path) NLIB_NONNULL;
// *result != 0 if exists
NLIB_VIS_PUBLIC errno_t nlib_exist_path(int* __restrict result,
                                        const char* __restrict native_path) NLIB_NONNULL;
NLIB_VIS_PUBLIC errno_t nlib_disk_freespace(const char* __restrict native_path,
                                            uint64_t* __restrict free_bytes_available,
                                            uint64_t* __restrict total_bytes,
                                            uint64_t* __restrict total_free_bytes) NLIB_NONNULL_1;

//
// Socket(Win32/Linux/Cygwin only)
//
#if defined(_MSC_VER) || defined(NLIB_UNIX)
#define NLIB_SOCKET_ENABLED

#ifdef AF_INET
# define NLIB_AF_INET AF_INET
#endif

#ifdef AF_INET6
# define NLIB_AF_INET6 AF_INET6
#endif

#ifdef SOCK_STREAM
# define NLIB_SOCK_STREAM SOCK_STREAM
#endif

#ifdef SOCK_DGRAM
# define NLIB_SOCK_DGRAM SOCK_DGRAM
#endif

#ifdef SOCK_RAW
# define NLIB_SOCK_RAW SOCK_RAW
#endif

#ifdef SOCK_RDM
# define NLIB_SOCK_RDM SOCK_RDM
#endif

// sizeof(SOCKET) is 8 on Win64, but it's safe because the upper 32bit of SOCKET is always 0.
// https://stackoverflow.com/questions/1953639/is-it-safe-to-cast-socket-to-int-under-win64
typedef int nlib_sock;
#define NLIB_SOCKET_INVALID (nlib_sock)(-1)  // NOLINT

NLIB_VIS_PUBLIC errno_t nlib_socket(nlib_sock* sockfd, int af, int type, int protocol);

typedef struct sockaddr nlib_sockaddr;
typedef struct sockaddr_in nlib_sockaddr_in;
typedef struct sockaddr_in6 nlib_sockaddr_in6;
NLIB_VIS_PUBLIC errno_t nlib_bind(nlib_sock sockfd, const nlib_sockaddr* addr, int namelen);
NLIB_VIS_PUBLIC errno_t nlib_listen(nlib_sock sockfd, int backlog);
NLIB_VIS_PUBLIC errno_t nlib_accept(nlib_sock* __restrict s, nlib_sock sockfd,
                                    nlib_sockaddr* __restrict addr, int* __restrict addrlen);
NLIB_VIS_PUBLIC errno_t nlib_connect(nlib_sock sockfd, const nlib_sockaddr* addr, int addrlen);

#ifdef MSG_DONTROUTE
# define NLIB_MSG_DONTROUTE MSG_DONTROUTE
#endif

#ifdef MSG_OOB
# define NLIB_MSG_OOB MSG_OOB
#endif

NLIB_VIS_PUBLIC errno_t nlib_send(size_t* __restrict size, nlib_sock sockfd,
                                  const void* __restrict buf,
                                  size_t len, int flags);
NLIB_VIS_PUBLIC errno_t nlib_sendto(size_t* __restrict size, nlib_sock sockfd,
                                    const void* __restrict buf, size_t len, int flags,
                                    const nlib_sockaddr* __restrict dest_addr, int addrlen);

#ifdef MSG_PEEK
# define NLIB_MSG_PEEK MSG_PEEK
#endif

#ifdef MSG_WAITALL
# define NLIB_MSG_WAITALL MSG_WAITALL
#endif

NLIB_VIS_PUBLIC errno_t nlib_recv(size_t* __restrict size, nlib_sock sockfd, void* __restrict buf,
                                  size_t len, int flags);
NLIB_VIS_PUBLIC errno_t nlib_recvfrom(size_t* __restrict size, nlib_sock sockfd,
                                      void* __restrict buf, size_t len, int flags,
                                      nlib_sockaddr* __restrict dest_addr, int* __restrict addrlen);
NLIB_VIS_PUBLIC errno_t nlib_closesocket(nlib_sock sockfd);

#ifdef _MSC_VER
# define SHUT_RD SD_RECEIVE
# define SHUT_WR SD_SEND
# define SHUT_RDWR SD_BOTH
#endif

#ifdef SHUT_RD
# define NLIB_SHUT_RD SHUT_RD
#endif

#ifdef SHUT_WR
# define NLIB_SHUT_WR SHUT_WR
#endif

#ifdef SHUT_RDWR
# define NLIB_SHUT_RDWR SHUT_RDWR
#endif
NLIB_VIS_PUBLIC errno_t nlib_shutdownsocket(nlib_sock sockfd, int how);

NLIB_VIS_PUBLIC uint32_t nlib_htonl(uint32_t hostlong);
NLIB_VIS_PUBLIC uint16_t nlib_htons(uint16_t hostshort);
NLIB_VIS_PUBLIC uint32_t nlib_ntohl(uint32_t netlong);
NLIB_VIS_PUBLIC uint16_t nlib_ntohs(uint16_t netshort);

#ifdef INADDR_ANY
# define NLIB_INADDR_ANY INADDR_ANY
#endif

#ifdef IN6ADDR_ANY_INIT
# define NLIB_IN6ADDR_ANY_INIT IN6ADDR_ANY_INIT
#endif

#ifdef INADDR_LOOPBACK
# define NLIB_INADDR_LOOPBACK INADDR_LOOPBACK
#endif

#ifdef IN6ADDR_LOOPBACK_INIT
# define NLIB_IN6ADDR_LOOPBACK_INIT IN6ADDR_LOOPBACK_INIT
#endif

typedef struct in_addr nlib_in_addr;
typedef struct in6_addr nlib_in6_addr;
NLIB_VIS_PUBLIC errno_t nlib_inet_pton(int af, const char* __restrict src, void* __restrict dst);
NLIB_VIS_PUBLIC errno_t nlib_inet_ntop(int af, const void* __restrict src, char* __restrict dst,
                                       size_t dst_size);

/*
# define EAI_ADDRFAMILY 1
# define EAI_AGAIN 2
# define EAI_BADFLAGS 3
# define EAI_FAIL 4
# define EAI_FAMILY 5
# define EAI_MEMORY 6
# define EAI_NODATA 7
# define EAI_NONAME 8
# define EAI_SERVICE 9
# define EAI_SOCKTYPE 10
# define EAI_SYSTEM 11
# define EAI_BADHINTS 12
# define EAI_PROTOCOL 13
# define EAI_OVERFLOW 14
*/

// for EAI_ADDRFAMILY, EAI_AGAIN, ... etc.
typedef int eai_error_t;
typedef struct addrinfo nlib_addrinfo;
NLIB_VIS_PUBLIC eai_error_t nlib_getaddrinfo(const char* __restrict node,
                                             const char* __restrict service,
                                             const nlib_addrinfo* __restrict hints,
                                             nlib_addrinfo** __restrict res);
NLIB_VIS_PUBLIC void nlib_freeaddrinfo(nlib_addrinfo* res);

#ifdef SOL_SOCKET
# define NLIB_SOL_SOCKET SOL_SOCKET
#endif

#ifdef IPPROTO_TCP
# define NLIB_IPPROTO_TCP IPPROTO_TCP
#endif

#ifdef SO_KEEPALIVE
# define NLIB_SO_KEEPALIVE SO_KEEPALIVE
#endif

#ifdef SO_LINGER
# define NLIB_SO_LINGER SO_LINGER
#endif

#ifdef SO_OOBINLINE
# define NLIB_SO_OOBINLINE SO_OOBINLINE
#endif

#ifdef SO_RCVBUF
# define NLIB_SO_RCVBUF SO_RCVBUF
#endif

#ifdef SO_REUSEADDR
# define NLIB_SO_REUSEADDR SO_REUSEADDR
#endif

#ifdef SO_RCVTIMEO
# define NLIB_SO_RCVTIMEO SO_RCVTIMEO
#endif

#ifdef SO_SNDBUF
# define NLIB_SO_SNDBUF SO_SNDBUF
#endif

#ifdef SO_SNDTIMEO
# define NLIB_SO_SNDTIMEO SO_SNDTIMEO
#endif

#ifdef TCP_NODELAY
# define NLIB_TCP_NODELAY TCP_NODELAY
#endif

NLIB_VIS_PUBLIC errno_t nlib_getsockopt(nlib_sock sockfd, int level, int optname,
                                        void* __restrict optval, int* __restrict optlen);
NLIB_VIS_PUBLIC errno_t nlib_setsockopt(nlib_sock sockfd, int level, int optname,
                                        const void* optval, int optlen);

typedef fd_set nlib_fd_set;
NLIB_VIS_PUBLIC errno_t nlib_select(int nfds, nlib_fd_set* __restrict readfds,
                                    nlib_fd_set* __restrict writefds,
                                    nlib_fd_set* __restrict exceptfds, nlib_duration duration);

#define NLIB_FD_CLR FD_CLR
#define NLIB_FD_ISSET FD_ISSET
#define NLIB_FD_SET FD_SET
#define NLIB_FD_ZERO FD_ZERO

#endif

//
// errno workaround
//

// PLEASE DO NOT DEFINE POSIX.1-2008 errno macros in your code
// http://pubs.opengroup.org/onlinepubs/9699919799/
#ifdef _MSC_VER
#  ifdef EDQUOT
#   warning  Do not define EDQUOT. nlib may not work correctly.
#  endif
#endif

/*
#ifdef NN_PLATFORM_CTR
// socket_User.autogen.h defines POSIX.1-2008 errno macros as enum values.
# ifdef NN_SOCKET_SOCKET_USER_AUTOGEN_H_
#  warning DO NOT INCLUDE socket_User.autogen.h in CTR socket library, Compile may fail.
#  define NLIB_SKIP_ERRNO_DEFINE
#  ifdef __errno_h
#   warning DO NOT INCLUDE errno.h when you include socket_User.autogen.h, you may not be able to handle errors of CTR socket library correctly.  // NOLINT
#  endif
# endif
#endif
*/

#ifndef NLIB_SKIP_ERRNO_DEFINE
#include <errno.h>  // NOLINT, for POSIX error values
#define NLIB_E_BASE 2000

// if boost/cerrno.hpp has definitions on errno values,
// nlib defines the same values which boost/cerrno.hpp defines.

// [E2BIG]
//     Argument list too long.
#ifndef E2BIG
# define E2BIG 9946
#endif

// [EACCES]
//    Permission denied.
#ifndef EACCES
# define EACCES 9973
#endif

// [EADDRINUSE]
//    Address in use.
#ifndef EADDRINUSE
# define EADDRINUSE 9902
#endif

// [EADDRNOTAVAIL]
//    Address not available.
#ifndef EADDRNOTAVAIL
# define EADDRNOTAVAIL 9903
#endif

// [EAFNOSUPPORT]
//    Address family not supported.
#ifndef EAFNOSUPPORT
# define EAFNOSUPPORT 9901
#endif

// [EAGAIN]
//    Resource unavailable, try again (may be the same value as [EWOULDBLOCK]).
#ifndef EAGAIN
# define EAGAIN 9976
#endif

// [EALREADY]
//    Connection already in progress.
#ifndef EALREADY
# define EALREADY 9907
#endif

// [EBADF]
//    Bad file descriptor.
#ifndef EBADF
# define EBADF 9949
#endif

// [EBADMSG]
//    Bad message.
#ifndef EBADMSG
# define EBADMSG 9905
#endif

// [EBUSY]
//    Device or resource busy.
#ifndef EBUSY
# define EBUSY 9952
#endif

// [ECANCELED]
// Operation canceled.
#ifndef ECANCELED
# define ECANCELED 9927
#endif

// [ECHILD]
//    No child processes.
#ifndef ECHILD
# define ECHILD 9963
#endif

// [ECONNABORTED]
//    Connection aborted.
#ifndef ECONNABORTED
# define ECONNABORTED 9906
#endif

// [ECONNREFUSED]
//    Connection refused.
#ifndef ECONNREFUSED
# define ECONNREFUSED 9908
#endif

// [ECONNRESET]
//    Connection reset.
#ifndef ECONNRESET
# define ECONNRESET 9909
#endif

// [EDEADLK]
//    Resource deadlock would occur.
#ifndef EDEADLK
# define EDEADLK 9975
#endif

// [EDESTADDRREQ]
//    Destination address required.
#ifndef EDESTADDRREQ
# define EDESTADDRREQ 9910
#endif

// [EDOM]
//    Mathematics argument out of domain of function.
#ifndef EDOM
# define EDOM 9947
#endif

// [EDQUOT]
//    Reserved.
#ifndef EDQUOT
# define EDQUOT (NLIB_E_BASE + 19)
#endif

// [EEXIST]
//    File exists.
#ifndef EEXIST
# define EEXIST 9955
#endif

// [EFAULT]
//    Bad address.
#ifndef EFAULT
# define EFAULT 9948
#endif

// [EFBIG]
//    File too large.
#ifndef EFBIG
# define EFBIG 9956
#endif

// [EHOSTUNREACH]
//    Host is unreachable.
#ifndef EHOSTUNREACH
# define EHOSTUNREACH 9911
#endif

// [EIDRM]
//    Identifier removed.
#ifndef EIDRM
# define EIDRM 9912
#endif

// [EILSEQ]
//    Illegal byte sequence.
#ifndef EILSEQ
# define EILSEQ 9945
#endif

// [EINPROGRESS]
//    Operation in progress.
#ifndef EINPROGRESS
# define EINPROGRESS 9928
#endif

// [EINTR]
//    Interrupted function.
#ifndef EINTR
# define EINTR 9959
#endif

// [EINVAL]
//    Invalid argument.
#ifndef EINVAL
# define EINVAL 9943
#endif

// [EIO]
//    I/O error.
#ifndef EIO
# define EIO 9961
#endif

// [EISCONN]
//    Socket is connected.
#ifndef EISCONN
# define EISCONN 9904
#endif

// [EISDIR]
//    Is a directory.
#ifndef EISDIR
# define EISDIR 9962
#endif

// [ELOOP]
//    Too many levels of symbolic links.
#ifndef ELOOP
# define ELOOP 9939
#endif

// [EMFILE]
//    File descriptor value too large.
#ifndef EMFILE
# define EMFILE 9978
#endif

// [EMLINK]
//    Too many links.
#ifndef EMLINK
# define EMLINK 9979
#endif

// [EMSGSIZE]
//    Message too large.
#ifndef EMSGSIZE
# define EMSGSIZE 9913
#endif

// [EMULTIHOP]
//    Reserved.
#ifndef EMULTIHOP
# define EMULTIHOP (NLIB_E_BASE + 36)
#endif

// [ENAMETOOLONG]
//    Filename too long.
#ifndef ENAMETOOLONG
# define ENAMETOOLONG 9957
#endif

// [ENETDOWN]
//    Network is down.
#ifndef ENETDOWN
# define ENETDOWN 9914
#endif

// [ENETRESET]
//    Connection aborted by network.
#ifndef ENETRESET
# define ENETRESET 9915
#endif

// [ENETUNREACH]
//    Network unreachable.
#ifndef ENETUNREACH
# define ENETUNREACH 9916
#endif

// [ENFILE]
//    Too many files open in system.
#ifndef ENFILE
# define ENFILE 9977
#endif

// [ENOBUFS]
//    No buffer space available.
#ifndef ENOBUFS
# define ENOBUFS 9917
#endif

// [ENODATA]
//    [OB XSR] [Option Start] No message is available on the STREAM head read queue. [Option End]
#ifndef ENODATA
# define ENODATA 9919
#endif

// [ENODEV]
//    No such device.
#ifndef ENODEV
# define ENODEV 9967
#endif

// [ENOENT]
//    No such file or directory.
#ifndef ENOENT
# define ENOENT 9968
#endif

// [ENOEXEC]
//    Executable file format error.
#ifndef ENOEXEC
# define ENOEXEC 9954
#endif

// [ENOLCK]
//    No locks available.
#ifndef ENOLCK
# define ENOLCK 9964
#endif

// [ENOLINK]
//    Reserved.
#ifndef ENOLINK
# define ENOLINK 9918
#endif

// [ENOMEM]
//    Not enough space.
#ifndef ENOMEM
# define ENOMEM 9971
#endif

// [ENOMSG]
//    No message of the desired type.
#ifndef ENOMSG
# define ENOMSG 9920
#endif

// [ENOPROTOOPT]
//    Protocol not available.
#ifndef ENOPROTOOPT
# define ENOPROTOOPT 9921
#endif

// [ENOSPC]
//    No space left on device.
#ifndef ENOSPC
# define ENOSPC 9965
#endif

// [ENOSR]
//    [OB XSR] [Option Start] No STREAM resources. [Option End]
#ifndef ENOSR
# define ENOSR 9922
#endif

// [ENOSTR]
//    [OB XSR] [Option Start] Not a STREAM. [Option End]
#ifndef ENOSTR
# define ENOSTR 9924
#endif

// [ENOSYS]
//    Function not supported.
#ifndef ENOSYS
# define ENOSYS 9942
#endif

// [ENOTCONN]
//    The socket is not connected.
#ifndef ENOTCONN
# define ENOTCONN 9925
#endif

// [ENOTDIR]
//    Not a directory or a symbolic link to a directory.
#ifndef ENOTDIR
# define ENOTDIR 9970
#endif

// [ENOTEMPTY]
//    Directory not empty.
#ifndef ENOTEMPTY
# define ENOTEMPTY 9953
#endif

// [ENOTRECOVERABLE]
//    State not recoverable.
#ifndef ENOTRECOVERABLE
# define ENOTRECOVERABLE 9934
#endif

// [ENOTSOCK]
//    Not a socket.
#ifndef ENOTSOCK
# define ENOTSOCK 9923
#endif

// [ENOTSUP]
//    Not supported (may be the same value as [EOPNOTSUPP]).
#ifndef ENOTSUP
# define ENOTSUP 9926
#endif

// [ENOTTY]
//    Inappropriate I/O control operation.
#ifndef ENOTTY
# define ENOTTY 9958
#endif

// [ENXIO]
//    No such device or address.
#ifndef ENXIO
# define ENXIO 9966
#endif

// [EOPNOTSUPP]
//    Operation not supported on socket (may be the same value as [ENOTSUP]).
#ifndef EOPNOTSUPP
# define EOPNOTSUPP 9929
#endif

// [EOVERFLOW]
//    Value too large to be stored in data type.
#ifndef EOVERFLOW
# define EOVERFLOW 9940
#endif

// [EOWNERDEAD]
//    Previous owner died.
#ifndef EOWNERDEAD
# define EOWNERDEAD  9931
#endif

// [EPERM]
//    Operation not permitted.
#ifndef EPERM
# define EPERM 9972
#endif

// [EPIPE]
//    Broken pipe.
#ifndef EPIPE
# define EPIPE 9950
#endif

// [EPROTO]
//    Protocol error.
#ifndef EPROTO
# define EPROTO 9932
#endif

// [EPROTONOSUPPORT]
//    Protocol not supported.
#ifndef EPROTONOSUPPORT
# define EPROTONOSUPPORT 9933
#endif

// [EPROTOTYPE]
//    Protocol wrong type for socket.
#ifndef EPROTOTYPE
# define EPROTOTYPE 9941
#endif

// [ERANGE]
//    Result too large.
#ifndef ERANGE
# define ERANGE 9944
#endif

// [EROFS]
//    Read-only file system.
#ifndef EROFS
# define EROFS 9974
#endif

// [ESPIPE]
//    Invalid seek.
#ifndef ESPIPE
# define ESPIPE 9960
#endif

// [ESRCH]
//    No such process.
#ifndef ESRCH
# define ESRCH 9969
#endif

// [ESTALE]
//    Reserved.
#ifndef ESTALE
# define ESTALE (NLIB_E_BASE + 75)
#endif

// [ETIME]
//    [OB XSR] [Option Start] Stream ioctl() timeout. [Option End]
#ifndef ETIME
# define ETIME 9935
#endif

// [ETIMEDOUT]
//    Connection timed out.
#ifndef ETIMEDOUT
# define ETIMEDOUT 9938
#endif

// [ETXTBSY]
//    Text file busy.
#ifndef ETXTBSY
# define ETXTBSY 9936
#endif

// [EWOULDBLOCK]
//    Operation would block (may be the same value as [EAGAIN]).
#ifndef EWOULDBLOCK
# define EWOULDBLOCK 9930
#endif

// [EXDEV]
//    Cross-device link.
#ifndef EXDEV
# define EXDEV 9951
#endif
#endif  // NLIB_SKIP_ERRNO_DEFINE

//
// nlib_spinlock
//
#ifndef NLIB_SPINLOCK_HAS_NATIVE
#define NLIB_SPINLOCK_INITIALIZER (0)
NLIB_VIS_PUBLIC void nlib_spinlock_lock_(nlib_spinlock* lock) NLIB_NONNULL;
#ifndef NN_PLATFORM_CTR
NLIB_VIS_PUBLIC void nlib_spinlock_unlock_(nlib_spinlock* lock) NLIB_NONNULL;
#endif

static NLIB_ALWAYS_INLINE void nlib_spinlock_init(nlib_spinlock* lock) {
    *lock = 0;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_lock(nlib_spinlock* lock) {
    int32_t expected = 0;
    if (!nlib_atomic_compare_exchange32(lock, &expected, 1, 1,
                                        NLIB_ATOMIC_ACQUIRE, NLIB_ATOMIC_RELAXED)) {
        nlib_spinlock_lock_(lock);
    }
}
static NLIB_ALWAYS_INLINE errno_t nlib_spinlock_trylock(nlib_spinlock* lock) {
    int32_t expected = 0;
    if (nlib_atomic_compare_exchange32(lock, &expected, 1, 0,
        NLIB_ATOMIC_ACQUIRE, NLIB_ATOMIC_RELAXED))
        return 0;
    else
        return EBUSY;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_unlock(nlib_spinlock* lock) {
#ifdef NN_PLATFORM_CTR
    nlib_atomic_store32(lock, 0, NLIB_ATOMIC_RELEASE);
#else
    int32_t expected = 1;
    if (!nlib_atomic_compare_exchange32(lock, &expected, 0, 0,
        NLIB_ATOMIC_RELEASE, NLIB_ATOMIC_RELAXED)) {
        NLIB_ASSUME(expected == 2);
        nlib_spinlock_unlock_(lock);
    }
#endif
}
#endif

//
// Utilities(Safer style functions)
//
NLIB_VIS_PUBLIC errno_t nlib_vsnprintf(size_t* __restrict count, char* __restrict buf, size_t size,
    _Printf_format_string_ const char* __restrict fmt, va_list args) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_snprintf(size_t* __restrict count, char* __restrict buf, size_t size,
    _Printf_format_string_ const char* __restrict fmt,
    ...) NLIB_ATTRIBUTE_PRINTF(4, 5) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_vdprintf(nlib_fd fd, size_t* __restrict count,
    _Printf_format_string_ const char* __restrict fmt, va_list args) NLIB_NONNULL_3;
NLIB_VIS_PUBLIC errno_t nlib_dprintf(nlib_fd fd, size_t* __restrict count,
    _Printf_format_string_ const char* __restrict fmt,
    ...) NLIB_ATTRIBUTE_PRINTF(3, 4) NLIB_NONNULL_3;
// fmt and string must be in UTF-8 even if you use Visual Studio
NLIB_VIS_PUBLIC int nlib_printf(
    _Printf_format_string_ const char* fmt, ...) NLIB_ATTRIBUTE_PRINTF(1, 2) NLIB_NONNULL_1;

NLIB_VIS_PUBLIC errno_t nlib_vsnwprintf(size_t* __restrict count, wchar_t* __restrict buf,
                                        size_t size,
                                        _Printf_format_string_ const wchar_t* __restrict fmt,
                                        va_list args) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t
    nlib_snwprintf(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                   _Printf_format_string_ const wchar_t* __restrict fmt, ...) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_vdwprintf(nlib_fd fd, size_t* __restrict count,
                                       _Printf_format_string_ const wchar_t* __restrict fmt,
                                       va_list args) NLIB_NONNULL_3;
NLIB_VIS_PUBLIC errno_t nlib_dwprintf(nlib_fd fd, size_t* __restrict count,
                                      _Printf_format_string_ const wchar_t* __restrict fmt,
                                      ...) NLIB_NONNULL_3;
NLIB_VIS_PUBLIC int nlib_wprintf(_Printf_format_string_ const wchar_t* fmt, ...) NLIB_NONNULL_1;

NLIB_VIS_PUBLIC errno_t nlib_vsnprintf_fallback(size_t* __restrict count, char* __restrict buf,
                                                size_t size,
                                                _Printf_format_string_ const char* __restrict fmt,
                                                va_list args) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t
    nlib_snprintf_fallback(size_t* __restrict count, char* __restrict buf, size_t size,
                           _Printf_format_string_ const char* __restrict fmt, ...)
        NLIB_ATTRIBUTE_PRINTF(4, 5) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t
    nlib_vsnwprintf_fallback(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                             _Printf_format_string_ const wchar_t* __restrict fmt,
                             va_list args) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t
    nlib_snwprintf_fallback(size_t* __restrict count, wchar_t* __restrict buf, size_t size,
                            _Printf_format_string_ const wchar_t* __restrict fmt,
                            ...) NLIB_NONNULL_4;

// http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1078.pdf
// http://www.open-std.org/jtc1/sc22/wg14/www/docs/n1079.htm
// a bit different from Microsoft's strcpy_s or etc.
// returns ERANGE instead of EINVAL

#ifdef NLIB_DOXYGEN
errno_t nlib_memcpy(void* s1, size_t s1max, const void* s2, size_t n);
errno_t nlib_memmove(void *s1, size_t s1max, const void *s2, size_t n);
errno_t nlib_memset(void* buf, int ch, size_t n);
#endif

static errno_t nlib_memcpy(void* __restrict s1, size_t s1max, const void* __restrict s2,
                           size_t n) NLIB_NONNULL;
static errno_t nlib_memmove(void* s1, size_t s1max, const void* s2, size_t n) NLIB_NONNULL;
static errno_t nlib_memset(void* buf, int ch, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT int nlib_memcmp(const void* buf1, const void* buf2, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr(const void* s, int c, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memrchr(const void* s, int c, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_not(const void* s, int c, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_range_not(const void* __restrict s,
                                                      const char* __restrict range,
                                                      size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_lt(const void* s, int c, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_gt(const void* s, int c, size_t n) NLIB_NONNULL;
// find (c & 0x80) != 0
NLIB_VIS_PUBLIC_ALT const void* nlib_memchr_mb(const void* s, size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC const void* nlib_memchr_set(const void* __restrict s, const char* __restrict set,
                                            size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC const void* nlib_memchr_set_not(const void* __restrict s,
                                                const char* __restrict set, size_t n) NLIB_NONNULL;
// ' ', CR, LF, HT are skipped
NLIB_VIS_PUBLIC_ALT const char* nlib_skipws(size_t* __restrict cnt_lf,
                                            const char** __restrict last_lf,
                                            const char* __restrict s, size_t n) NLIB_NONNULL_2;

NLIB_VIS_PUBLIC_ALT size_t nlib_strlen(const char* s) NLIB_NONNULL;
// see strnlen_s
NLIB_VIS_PUBLIC_ALT size_t nlib_strnlen(const char* s, size_t maxsize) NLIB_NONNULL;
// see strcpy_s
NLIB_VIS_PUBLIC errno_t
    nlib_strcpy(char* __restrict s1, size_t s1max, const char* __restrict s2) NLIB_NONNULL;
// see strncpy_s
NLIB_VIS_PUBLIC errno_t nlib_strncpy(char* __restrict s1, size_t s1max, const char* __restrict s2,
                                     size_t n) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const char* nlib_strchr(const char* s, int c) NLIB_NONNULL;
NLIB_VIS_PUBLIC_ALT const char* nlib_strrchr(const char* s, int c) NLIB_NONNULL;
// find (c & 0x80) != 0, used for skipping ASCII chars
NLIB_VIS_PUBLIC_ALT const char* nlib_strchr_mb(const char* s) NLIB_NONNULL;

NLIB_VIS_PUBLIC size_t nlib_wcslen(const wchar_t* s) NLIB_NONNULL;
NLIB_VIS_PUBLIC size_t nlib_wcsnlen(const wchar_t* s, size_t maxsize) NLIB_NONNULL;  // wcsnlen_s
NLIB_VIS_PUBLIC errno_t nlib_wcscpy(wchar_t* __restrict s1, size_t s1max,
                                    const wchar_t* __restrict s2) NLIB_NONNULL;  // wcscpy_s
NLIB_VIS_PUBLIC errno_t nlib_wcsncpy(wchar_t* __restrict s1, size_t s1max,
                                     const wchar_t* __restrict s2,
                                     size_t n) NLIB_NONNULL;  // wcsncpy_s

NLIB_VIS_PUBLIC errno_t nlib_strcat(char* __restrict s1, size_t s1max,
                                    const char* __restrict s2) NLIB_NONNULL;  // strcat_s
NLIB_VIS_PUBLIC errno_t nlib_strncat(char* __restrict s1, size_t s1max, const char* __restrict s2,
                                     size_t n) NLIB_NONNULL;  // strncat_s

NLIB_VIS_PUBLIC errno_t nlib_wcscat(wchar_t* __restrict s1, size_t s1max,
                                    const wchar_t* __restrict s2) NLIB_NONNULL;  // wcscat_s
NLIB_VIS_PUBLIC errno_t nlib_wcsncat(wchar_t* __restrict s1, size_t s1max,
                                     const wchar_t* __restrict s2,
                                     size_t n) NLIB_NONNULL;  // wcsncat_s

NLIB_VIS_PUBLIC errno_t nlib_wide_to_utf8(size_t* __restrict utf8count, char* __restrict utf8,
                                          size_t buflen,
                                          const wchar_t* __restrict wcstr) NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_utf8_to_wide(size_t* __restrict wccount, wchar_t* __restrict wcstr,
                                          size_t buflen,
                                          const char* __restrict utf8) NLIB_NONNULL_4;

NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t
    nlib_strcplen(size_t* __restrict count, const char* __restrict str) NLIB_NONNULL_2;
// *count = # of codepoints, *len = strlen(str)
NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT errno_t
    nlib_strcplen_ex(size_t* __restrict count, size_t* __restrict len,
                     const char* __restrict str) NLIB_NONNULL_3;
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT errno_t
    nlib_wcscplen(size_t* __restrict count, const wchar_t* __restrict str) NLIB_NONNULL_2;

// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_16(uint16_t* p, size_t count) NLIB_NONNULL;
// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_32(uint32_t* p, size_t count) NLIB_NONNULL;
// for (0..count) { swapendian(p[count]); }
NLIB_VIS_PUBLIC_ALT errno_t nlib_swapendian_64(uint64_t* p, size_t count) NLIB_NONNULL;

//
// malloc functions which nlib uses
// You can redefine them.
// See sample replace_malloc.cpp of nlibnx_heap.a library.
//

// weak function
NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT void* nlib_malloc(size_t size)
NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(1);

// weak function
NLIB_VIS_PUBLIC_ALT void  nlib_free(void* ptr);

// weak function
NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT void* nlib_calloc(size_t nmemb, size_t size)
NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE2(1, 2);

// weak function
NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT void* nlib_realloc(void* ptr, size_t size)
NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(2);

// weak function(calls nlib_free(ptr) by default)
NLIB_VIS_PUBLIC_ALT void nlib_free_size(void* ptr, size_t size);

// weak function, not defined if WIN32 or CTR
NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT void* nlib_memalign(size_t alignment, size_t size)
NLIB_ATTRIBUTE_MALLOC NLIB_ATTRIBUTE_ALLOC_SIZE1(2) NLIB_ATTRIBUTE_ALLOC_ALIGN(1);

#ifndef NLIB_MEMCPY
# define NLIB_MEMCPY(a, b, c) memcpy((a), (b), (c))
#endif

#ifndef NLIB_MEMMOVE
# define NLIB_MEMMOVE(a, b, c) memmove((a), (b), (c))
#endif

#ifndef NLIB_MEMSET
# define NLIB_MEMSET(a, b, c) memset((a), (b), (c))
#endif

#ifndef NLIB_C_INLINE
# define NLIB_C_INLINE __inline
#endif

// memcpy_s
static NLIB_C_INLINE errno_t nlib_memcpy(void* s1, size_t s1max, const void* s2, size_t n) {
#ifndef NLIB_NONNULL_ENABLED
    if (!s1 || !s2) return ERANGE;
#endif
    if (s1max < n) {
        NLIB_MEMSET(s1, 0, s1max);
        return ERANGE;
    }
    NLIB_MEMCPY(s1, s2, n);
    return 0;
}

// memmove_s
static NLIB_C_INLINE errno_t nlib_memmove(void *s1, size_t s1max, const void *s2, size_t n) {
#ifndef NLIB_NONNULL_ENABLED
    if (!s1 || !s2) return ERANGE;
#endif
    if (s1max < n) return ERANGE;
    NLIB_MEMMOVE(s1, s2, n);
    return 0;
}

static NLIB_C_INLINE errno_t nlib_memset(void* buf, int ch, size_t n) {
    NLIB_EINVAL_IFNULL(buf);
    NLIB_MEMSET(buf, ch, n);
    return 0;
}

#undef NLIB_MEMCPY
#undef NLIB_MEMMOVE
#undef NLIB_MEMSET

#ifdef NLIB_DOXYGEN
int nlib_clz64(uint64_t x);
int nlib_ctz64(uint64_t x);
int nlib_clz(uint32_t x);
int nlib_ctz(uint32_t x);
#endif

NLIB_VIS_PUBLIC_ALT int nlib_popcnt(uint32_t x);
NLIB_VIS_PUBLIC_ALT int nlib_popcnt64(uint64_t x);
NLIB_VIS_PUBLIC int nlib_popcnt16(uint16_t x);

#ifdef __cplusplus
}
#endif

//
// C++ functions in global namespace
//
#ifdef __cplusplus

// code is only for C++ because uchar.h is rarely supported
#if defined(__INTELLISENSE__)
# define NLIB_NOEXCEPT
#elif defined(NLIB_CXX11_NOEXCEPT)
# define NLIB_NOEXCEPT noexcept
#elif !defined(NLIB_NOEXCEPT)
# define NLIB_NOEXCEPT throw()
#endif

#ifdef NLIB_CXX11_NEW_CHARACTER_TYPES
typedef char16_t nlib_utf16_t;
typedef char32_t nlib_utf32_t;
#else
typedef uint16_t nlib_utf16_t;
typedef uint32_t nlib_utf32_t;
#endif

// 0 if error
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT int nlib_utf16_to_utf32char(
    nlib_utf32_t* utf32, nlib_utf16_t upper, nlib_utf16_t lower) NLIB_NOEXCEPT NLIB_NONNULL;
// 0 if error
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT int nlib_utf32char_to_utf16(nlib_utf16_t* upper,
                                                              nlib_utf16_t* lower,
                                                              nlib_utf32_t utf32) NLIB_NOEXCEPT;
// 0 if error
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT int nlib_utf8_to_utf32char(nlib_utf32_t* utf32, const char* utf8)
    NLIB_NOEXCEPT NLIB_NONNULL;
// 0 if error
NLIB_VIS_PUBLIC NLIB_CHECK_RESULT int nlib_utf32char_to_utf8(char (&utf8)[4],
                                                             nlib_utf32_t utf32) NLIB_NOEXCEPT;

NLIB_VIS_PUBLIC errno_t nlib_utf16_to_utf8(size_t* utf8count, char* utf8, size_t buflen,
                                           const nlib_utf16_t* utf16) NLIB_NOEXCEPT NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_utf8_to_utf16(size_t* utf16count, nlib_utf16_t* utf16, size_t buflen,
                                           const char* utf8) NLIB_NOEXCEPT NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_utf32_to_utf8(size_t* utf8count, char* utf8, size_t buflen,
                                           const nlib_utf32_t* utf32) NLIB_NOEXCEPT NLIB_NONNULL_4;
NLIB_VIS_PUBLIC errno_t nlib_utf8_to_utf32(size_t* utf32count, nlib_utf32_t* utf32, size_t buflen,
                                           const char* utf8) NLIB_NOEXCEPT NLIB_NONNULL_4;

extern "C" NLIB_VIS_PUBLIC_ALT size_t nlib_utf16len_(const uint16_t* str) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC_ALT size_t
    nlib_utf16nlen_(const uint16_t* str, size_t maxsize) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC errno_t
    nlib_utf16cpy_(uint16_t* s1, size_t s1max, const uint16_t* s2) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC errno_t
    nlib_utf16ncpy_(uint16_t* s1, size_t s1max, const uint16_t* s2, size_t n) NLIB_NONNULL;
NLIB_CHECK_RESULT inline size_t nlib_utf16len(const nlib_utf16_t* str) NLIB_NOEXCEPT {
    return nlib_utf16len_(reinterpret_cast<const uint16_t*>(str));
}
NLIB_CHECK_RESULT inline size_t nlib_utf16nlen(const nlib_utf16_t* str, size_t maxsize)
    NLIB_NOEXCEPT {
    return nlib_utf16nlen_(reinterpret_cast<const uint16_t*>(str), maxsize);
}
inline errno_t nlib_utf16cpy(nlib_utf16_t* s1, size_t s1max,
                             const nlib_utf16_t* s2) NLIB_NOEXCEPT {
    return nlib_utf16cpy_(reinterpret_cast<uint16_t*>(s1), s1max,
                          reinterpret_cast<const uint16_t*>(s2));
}
inline errno_t nlib_utf16ncpy(nlib_utf16_t* s1, size_t s1max,
                              const nlib_utf16_t* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_utf16ncpy_(reinterpret_cast<uint16_t*>(s1), s1max,
                           reinterpret_cast<const uint16_t*>(s2), n);
}

extern "C" NLIB_VIS_PUBLIC_ALT size_t nlib_utf32len_(const uint32_t* str) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC_ALT size_t nlib_utf32nlen_(const uint32_t* str,
                                                      size_t maxsize) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC errno_t
    nlib_utf32cpy_(uint32_t* s1, size_t s1max, const uint32_t* s2) NLIB_NONNULL;
extern "C" NLIB_VIS_PUBLIC errno_t
    nlib_utf32ncpy_(uint32_t* s1, size_t s1max, const uint32_t* s2, size_t n) NLIB_NONNULL;
NLIB_CHECK_RESULT inline size_t nlib_utf32len(const nlib_utf32_t* str) NLIB_NOEXCEPT {
    return nlib_utf32len_(reinterpret_cast<const uint32_t*>(str));
}
NLIB_CHECK_RESULT inline size_t nlib_utf32nlen(const nlib_utf32_t* str, size_t maxsize)
    NLIB_NOEXCEPT {
    return nlib_utf32nlen_(reinterpret_cast<const uint32_t*>(str), maxsize);
}
inline errno_t nlib_utf32cpy(nlib_utf32_t* s1, size_t s1max,
                             const nlib_utf32_t* s2) NLIB_NOEXCEPT {
    return nlib_utf32cpy_(reinterpret_cast<uint32_t*>(s1), s1max,
                          reinterpret_cast<const uint32_t*>(s2));
}
inline errno_t nlib_utf32ncpy(nlib_utf32_t* s1, size_t s1max,
                              const nlib_utf32_t* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_utf32ncpy_(reinterpret_cast<uint32_t*>(s1), s1max,
                           reinterpret_cast<const uint32_t*>(s2), n);
}

extern "C" NLIB_VIS_PUBLIC_ALT NLIB_CHECK_RESULT errno_t
    nlib_utf16cplen_ex_(size_t* count, size_t* len, const uint16_t* str) NLIB_NONNULL_3;

inline errno_t nlib_utf16cplen(
    size_t* count, const nlib_utf16_t* str) NLIB_NOEXCEPT {
    return nlib_utf16cplen_ex_(count, NULL, reinterpret_cast<const uint16_t*>(str));
}
inline errno_t nlib_utf16cplen_ex(
    size_t* count, size_t* len, const nlib_utf16_t* str) NLIB_NOEXCEPT {
    return nlib_utf16cplen_ex_(count, len, reinterpret_cast<const uint16_t*>(str));
}
NLIB_CHECK_RESULT NLIB_VIS_PUBLIC errno_t
    nlib_utf32cplen(size_t* count, const nlib_utf32_t* str) NLIB_NOEXCEPT NLIB_NONNULL_2;

template <size_t N>
NLIB_ALWAYS_INLINE errno_t
    nlib_vsnprintf(size_t* count, char (&buf)[N], _Printf_format_string_ const char* fmt,
                   va_list args) NLIB_NOEXCEPT {
    return nlib_vsnprintf(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snprintf(size_t* count, char (&buf)[N],
    _Printf_format_string_ const char* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnprintf(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t
    nlib_vsnwprintf(size_t* count, wchar_t (&buf)[N], _Printf_format_string_ const wchar_t* fmt,
                    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnwprintf(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snwprintf(size_t* count, wchar_t(&buf)[N],
    _Printf_format_string_ const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnwprintf(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t
    nlib_vsnprintf_fallback(size_t* count, char (&buf)[N], _Printf_format_string_ const char* fmt,
                            va_list args) NLIB_NOEXCEPT {
    return nlib_vsnprintf_fallback(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snprintf_fallback(size_t* count, char (&buf)[N],
    _Printf_format_string_ const char* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnprintf_fallback(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_vsnwprintf_fallback(size_t* count, wchar_t (&buf)[N],
                                                    _Printf_format_string_ const wchar_t* fmt,
                                                    va_list args) NLIB_NOEXCEPT {
    return nlib_vsnwprintf_fallback(count, buf, N, fmt, args);
}

template<size_t N>
inline NLIB_VIS_HIDDEN errno_t nlib_snwprintf_fallback(size_t* count, wchar_t(&buf)[N],
    _Printf_format_string_ const wchar_t* fmt, ...) NLIB_NOEXCEPT {
    va_list args;
    va_start(args, fmt);
    errno_t e = nlib_vsnwprintf_fallback(count, buf, N, fmt, args);
    va_end(args);
    return e;
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strcpy(char (&s1)[N], const char* s2) NLIB_NOEXCEPT {
    return nlib_strcpy(&s1[0], N, s2);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strncpy(char (&s1)[N], const char* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_strncpy(&s1[0], N, s2, n);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_wcscpy(wchar_t (&s1)[N], const wchar_t* s2) NLIB_NOEXCEPT {
    return nlib_wcscpy(&s1[0], N, s2);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t
    nlib_wcsncpy(wchar_t (&s1)[N], const wchar_t* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_wcsncpy(&s1[0], N, s2, n);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strcat(char (&s1)[N], const char* s2) NLIB_NOEXCEPT {
    return nlib_strcat(&s1[0], N, s2);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_strncat(char (&s1)[N], const char* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_strncat(&s1[0], N, s2, n);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t nlib_wcscat(wchar_t (&s1)[N], const wchar_t* s2) NLIB_NOEXCEPT {
    return nlib_wcscat(&s1[0], N, s2);
}

template <size_t N>
NLIB_ALWAYS_INLINE errno_t
    nlib_wcsncat(wchar_t (&s1)[N], const wchar_t* s2, size_t n) NLIB_NOEXCEPT {
    return nlib_wcsncat(&s1[0], N, s2, n);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_wide_to_utf8(
    size_t* result, char (&utf8)[N], const wchar_t* wcstr) NLIB_NOEXCEPT {
    return nlib_wide_to_utf8(result, &utf8[0], N, wcstr);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_utf8_to_wide(
    size_t* result, wchar_t (&wcstr)[N], const char* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_wide(result, &wcstr[0], N, utf8);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_utf16_to_utf8(
    size_t* utf8count, char (&utf8)[N], const nlib_utf16_t* utf16) NLIB_NOEXCEPT {
    return nlib_utf16_to_utf8(utf8count, &utf8[0], N, utf16);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_utf8_to_utf16(
    size_t* utf16count, nlib_utf16_t (&utf16)[N], const char* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_utf16(utf16count, &utf16[0], N, utf8);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_utf32_to_utf8(
    size_t* utf8count, char (&utf8)[N], const nlib_utf32_t* utf32) NLIB_NOEXCEPT {
    return nlib_utf32_to_utf8(utf8count, &utf8[0], N, utf32);
}

template<size_t N>
NLIB_ALWAYS_INLINE NLIB_CHECK_RESULT errno_t nlib_utf8_to_utf32(
    size_t* utf32count, nlib_utf32_t(&utf32)[N], const char* utf8) NLIB_NOEXCEPT {
    return nlib_utf8_to_utf32(utf32count, &utf32[0], N, utf8);
}

#if !defined(NN_PLATFORM_CTR) && !defined(CAFE)
// This may throw C++ exception defined by nlib, See also nlib_thread_exit()
NLIB_VIS_PUBLIC void nlib_thread_exit_cpp() NLIB_NORETURN;
#endif

#endif

#endif  // INCLUDE_NN_NLIB_PLATFORM_H_
