﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_CTR_H_
#define INCLUDE_NN_NLIB_PLATFORM_CTR_H_
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
# error do not include directly
#endif

#ifdef NN_PLATFORM_CTR

#define NLIB_HAS_STDHEADER_STDINT
#define NLIB_HAS_STDHEADER_INTTYPES

#ifndef __USE_C99_MATH
# define __USE_C99_MATH
#endif

#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

#define NLIB_ALWAYS_INLINE __forceinline
#define NLIB_NEVER_INLINE __declspec(noinline)
// Do not write like NLIB_(UN)LIKELY(a || b) or NLIB_(UN)LIKELY(a && b).
// Write like (NLIB_(UN)LIKELY(a) || NLIB_(UN)LIKELY(b)) instead.
// armcc optimizer may generate wrong binary.
#define NLIB_LIKELY(x) __builtin_expect(!!(x), 1)
#define NLIB_UNLIKELY(x) __builtin_expect(!!(x), 0)
#define NLIB_EXPECT(var, exp_value) __builtin_expect((var), (exp_value))
#define NLIB_CHECK_RESULT
#define NLIB_NORETURN __attribute__((noreturn))
#define NLIB_NONNULL __attribute__((nonnull))
#define NLIB_NONNULL_1 __attribute__((nonnull (1)))
#define NLIB_NONNULL_2 __attribute__((nonnull (2)))
#define NLIB_NONNULL_3 __attribute__((nonnull (3)))
#define NLIB_NONNULL_4 __attribute__((nonnull (4)))
#define NLIB_NONNULL_5 __attribute__((nonnull (5)))
#define NLIB_NONNULL_ENABLED
#define NLIB_ATTRIBUTE_MALLOC __attribute__((malloc))
#define NLIB_ATTRIBUTE_ALLOC_SIZE1(n)
#define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1)
#define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
#define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
#define NLIB_DEPRECATED
#define NLIB_DEPRECATED_MSG(msg)
#define NLIB_VIS_HIDDEN
#define NLIB_VIS_PUBLIC
#define NLIB_WEAKSYMBOL __attribute__((weak))
#define _Printf_format_string_
#define NLIB_LITTLE_ENDIAN

void nlib_ctr_barrier(void);
#define NLIB_MEMORY_ORDER_RELEASE nlib_ctr_barrier()
#define NLIB_MEMORY_ORDER_ACQUIRE nlib_ctr_barrier()
#define NLIB_MEMORY_ORDER_ACQ_REL nlib_ctr_barrier()

struct nlib_mutex_ {
    unsigned int _[3];  // ::nn::os::CriticalSection
};
typedef struct nlib_mutex_ nlib_mutex;
#define NLIB_MUTEX_INITIALIZER { 0xFFFFFFFFUL, 0xFFFFFFFFUL, 0xFFFFFFFFUL }
#define NLIB_RECURSIVE_MUTEX_INITIALIZER { 0xFFFFFFFFUL, 0xFFFFFFFFUL, 0xFFFFFFFFUL }
#define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER { 0xFFFFFFFFUL, 0xFFFFFFFFUL, 0xFFFFFFFFUL }

typedef void* nlib_thread;

struct nlib_semaphore_ {
    unsigned int _[1];  //  nn::os::Semaphore
};
typedef struct nlib_semaphore_ nlib_semaphore;

struct nlib_cond_ {
    unsigned int _;  //  nn::os::Event
};
typedef struct nlib_cond_ nlib_cond;
#define NLIB_COND_INITIALIZER {0}

__declspec(noreturn) int MyNoreturn_(void);
#define NLIB_ASSUME(cond) (void)((cond) || MyNoreturn_())

// nlib_clz64(INT64_MIN) -> 0, nlib_clz64(1) -> 63
static NLIB_ALWAYS_INLINE int nlib_clz64(uint64_t x) {
    return x != 0 ? __builtin_clzll(x) : 64;
}

// nlib_ctz64(INT64_MIN) -> 63, nlib_ctz64(1) -> 0
static NLIB_ALWAYS_INLINE int nlib_ctz64(uint64_t x) {
    return 64 - nlib_clz64(~x & (x - 1));
}

// nlib_clz(0x80000000) -> 0, nlib_clz(1) -> 31
static NLIB_ALWAYS_INLINE int nlib_clz(uint32_t x) {
    return x != 0 ? __builtin_clz(x) : 32;
}

// nlib_clz(0x80000000) -> 31, nlib_clz(1) -> 0
static NLIB_ALWAYS_INLINE int nlib_ctz(uint32_t x) {
    return 32 - nlib_clz(~x & (x - 1));
}

#define NLIB_ATOMIC_RELAXED (0)
#define NLIB_ATOMIC_ACQUIRE (1)
#define NLIB_ATOMIC_RELEASE (2)
#define NLIB_ATOMIC_ACQ_REL (3)
#define NLIB_ATOMIC_SEQ_CST (7)

int32_t nlib_atomic_load32(const int32_t* ptr, int memorder);
void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val, int memorder);
int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                   int32_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder);

int64_t nlib_atomic_load64(const int64_t* ptr, int memorder);
void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder);
int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                   int64_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder);

void* nlib_atomic_loadptr(void* const* ptr, int memorder);
void nlib_atomic_storeptr(void** ptr, void* val, int memorder);
int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                    void* desired, int weak,
                                    int success_memorder, int failure_memorder);

static NLIB_ALWAYS_INLINE void nlib_atomic_thread_fence(int memorder) {
    if (memorder != NLIB_ATOMIC_RELAXED)
        nlib_ctr_barrier();
}

#ifdef __cplusplus
}
#endif

#define NLIB_NOEXCEPT

#endif
#endif  // INCLUDE_NN_NLIB_PLATFORM_CTR_H_
