﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
#define INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
#ifndef INCLUDE_NN_NLIB_PLATFORM_H_
# error do not include directly
#endif

#if defined(__linux__)   || \
    defined(__FreeBSD__) || \
    defined(__CYGWIN__)  || \
    (defined(__APPLE__) && defined(__MACH__))

#if defined(__APPLE__) && defined(__MACH__)
#include <libkern/OSAtomic.h>
#include <errno.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

#ifndef NLIB_UNIX
# define NLIB_UNIX
#endif

// For now, only supports gcc or clang
#if !defined(__GNUC__) && !defined(__clang__)
# error
#endif

#define NLIB_HAS_STDHEADER_STDINT
#define NLIB_HAS_STDHEADER_INTTYPES

#include <pthread.h>  // for PTHREAD_MUTEX_INITIALIZER, ....
#include <semaphore.h>  // for sem_t
#include <sys/types.h>  // for pthread_mutex_t, ....
#include <fcntl.h>  // NOLINT
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <stdint.h>

#if defined(__i386__) || defined(__x86_64__)
# include <x86intrin.h>
#endif

#ifndef __CYGWIN__
# define NLIB_VIS_HIDDEN __attribute__((visibility("hidden")))
# define NLIB_VIS_PUBLIC __attribute__((visibility("default")))
# define NLIB_WEAKSYMBOL __attribute__((weak))
#else
# define NLIB_VIS_HIDDEN
# define NLIB_VIS_PUBLIC
# define NLIB_WEAKSYMBOL
#endif

#define NLIB_ALWAYS_INLINE inline __attribute__((always_inline))
#define NLIB_NEVER_INLINE __attribute__((__noinline__))
#define NLIB_LIKELY(x) __builtin_expect(!!(x), 1)
#define NLIB_UNLIKELY(x) __builtin_expect(!!(x), 0)
#define NLIB_EXPECT(var, exp_value) __builtin_expect((var), (exp_value))
#define NLIB_CHECK_RESULT __attribute__((warn_unused_result))
#define NLIB_NORETURN __attribute__((noreturn))
#define NLIB_NONNULL __attribute__((nonnull))
#define NLIB_NONNULL_1 __attribute__((nonnull (1)))
#define NLIB_NONNULL_2 __attribute__((nonnull (2)))
#define NLIB_NONNULL_3 __attribute__((nonnull (3)))
#define NLIB_NONNULL_4 __attribute__((nonnull (4)))
#define NLIB_NONNULL_5 __attribute__((nonnull (5)))
#define NLIB_NONNULL_ENABLED
#define NLIB_ATTRIBUTE_MALLOC __attribute__((malloc))

#ifdef __clang__
# if __has_attribute(alloc_size)
#  define NLIB_ATTRIBUTE_ALLOC_SIZE1(n) __attribute__((alloc_size(n)))
#  define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1) __attribute__((alloc_size(n0, n1)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_SIZE1(n)
#  define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1)
# endif
# if __has_attribute(alloc_align)
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn) __attribute__((alloc_align(algn)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
# endif
# if __has_attribute(assume_aligned)
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n) __attribute__((assume_aligned(n)))
# else
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
# endif
#else
# define NLIB_ATTRIBUTE_ALLOC_SIZE1(n) __attribute__((alloc_size(n)))
# define NLIB_ATTRIBUTE_ALLOC_SIZE2(n0, n1) __attribute__((alloc_size(n0, n1)))
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 9)
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn) __attribute__((alloc_align(algn)))
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n) __attribute__((assume_aligned(n)))
# else
#  define NLIB_ATTRIBUTE_ALLOC_ALIGN(algn)
#  define NLIB_ATTRIBUTE_ASSUME_ALIGNED(n)
# endif
#endif

#define NLIB_DEPRECATED __attribute__((deprecated))
#define NLIB_DEPRECATED_MSG(msg) __attribute__((deprecated(msg)))
#define _Printf_format_string_

#if defined(__LITTLE_ENDIAN__) || defined(__LITTLE_ENDIAN)
# define NLIB_LITTLE_ENDIAN
#elif defined(__BIG_ENDIAN__) || defined(__BIG_ENDIAN)
# undef NLIB_LITTLE_ENDIAN
#else
# if __BYTE_ORDER__ == __ORDER_LITTLE_ENDIAN__
#  define NLIB_LITTLE_ENDIAN
# else
#  undef NLIB_LITTLE_ENDIAN
# endif
#endif
#if defined(__x86_64__) || defined(__aarch64__)
# define NLIB_64BIT
#endif

#if !defined(__clang__) && defined(__GNUC__) && __GNUC__ == 4 && __GNUC_MINOR__ < 7
# if !defined(__i386__) && !defined(__x86_64__)
#  error Sorry
# endif
#define NLIB_MEMORY_ORDER_RELEASE __asm__ __volatile__("sfence": : :"memory")
#define NLIB_MEMORY_ORDER_ACQUIRE __asm__ __volatile__("lfence": : :"memory")
#define NLIB_MEMORY_ORDER_ACQ_REL __asm__ __volatile__("mfence": : :"memory")
#else
#define NLIB_MEMORY_ORDER_RELEASE __atomic_thread_fence(__ATOMIC_RELEASE)
#define NLIB_MEMORY_ORDER_ACQUIRE __atomic_thread_fence(__ATOMIC_ACQUIRE)
#define NLIB_MEMORY_ORDER_ACQ_REL __atomic_thread_fence(__ATOMIC_ACQ_REL)
#endif

typedef pthread_mutex_t nlib_mutex;

#ifdef PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
# ifdef __FreeBSD__
// https://stackoverflow.com/questions/10369606/constexpr-pointer-value
// ((pthread_mutex_t)1) in FreeBSD
#  define NLIB_MUTEX_INITIALIZER (__builtin_constant_p(PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP) ? \
                                  PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP :                       \
                                  PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP)
# else
#  define NLIB_MUTEX_INITIALIZER PTHREAD_ADAPTIVE_MUTEX_INITIALIZER_NP
# endif
#else
# define NLIB_MUTEX_INITIALIZER PTHREAD_MUTEX_INITIALIZER
#endif

#if defined(PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP)
# define NLIB_RECURSIVE_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP
#elif defined(PTHREAD_RECURSIVE_MUTEX_INITIALIZER)
# define NLIB_RECURSIVE_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER PTHREAD_RECURSIVE_MUTEX_INITIALIZER
#elif defined(__FreeBSD__)
// FreeBSD does not support static initializer for recursive mutex
// nlib initializes recursive mutex if the value is 255
# define NLIB_RECURSIVE_MUTEX_INITIALIZER (__builtin_constant_p((pthread_mutex_t)255) ? \
                                           (pthread_mutex_t)255 : (pthread_mutex_t)255)
# define NLIB_RECURSIVE_TIMED_MUTEX_INITIALIZER (__builtin_constant_p((pthread_mutex_t)255) ? \
                                                 (pthread_mutex_t)255 : (pthread_mutex_t)255)
#else
# error Sorry
#endif

#if defined(__APPLE__)
typedef int nlib_semaphore;
#else
typedef sem_t nlib_semaphore;
#endif

typedef pthread_cond_t nlib_cond;
#define NLIB_COND_INITIALIZER PTHREAD_COND_INITIALIZER

typedef pthread_t nlib_thread;

// nlib_clz64(INT64_MIN) -> 0, nlib_clz64(1) -> 63
static inline __attribute__((always_inline)) int nlib_clz64(uint64_t x) {
    return x != 0 ? __builtin_clzll(x) : 64;
}

// nlib_ctz64(INT64_MIN) -> 63, nlib_ctz64(1) -> 0
static inline __attribute__((always_inline)) int nlib_ctz64(uint64_t x) {
    return x != 0 ? __builtin_ctzll(x) : 64;
}

// nlib_clz(0x80000000) -> 0, nlib_clz(1) -> 31
static inline __attribute__((always_inline)) int nlib_clz(uint32_t x) {
    return x != 0 ? __builtin_clz(x) : 32;
}

// nlib_clz(0x80000000) -> 31, nlib_clz(1) -> 0
static inline __attribute__((always_inline)) int nlib_ctz(uint32_t x) {
    return x != 0 ? __builtin_ctz(x) : 32;
}

#if defined(__APPLE__)
#define NLIB_SPINLOCK_HAS_NATIVE
typedef OSSpinLock nlib_spinlock;
#define NLIB_SPINLOCK_INITIALIZER (0)
static NLIB_ALWAYS_INLINE void nlib_spinlock_init(nlib_spinlock* lock) {
    *lock = 0;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_lock(nlib_spinlock* lock) {
    OSSpinLockLock(lock);
}
static NLIB_ALWAYS_INLINE int nlib_spinlock_trylock(nlib_spinlock* lock) {
    return OSSpinLockTry(lock) ? 0 : EBUSY;
}
static NLIB_ALWAYS_INLINE void nlib_spinlock_unlock(nlib_spinlock* lock) {
    OSSpinLockUnlock(lock);
}
#endif

#ifdef __cplusplus
}
#endif

#if defined(__clang__)
# if __has_feature(cxx_unicode_literals)
#  define NLIB_CXX11_NEW_CHARACTER_TYPES
# endif
# if __has_feature(cxx_exceptions)
#  if __has_feature(cxx_noexcept)
#   define NLIB_CXX11_NOEXCEPT
#  endif
# else
#  define NLIB_NOEXCEPT
# endif
# if 0  // __has_attribute(capability)
#  define NLIB_LOCKABLE __attribute__((capability("mutex")))
#  define NLIB_LOCK_FUNC(...) __attribute__((acquire_capability(__VA_ARGS__)))
#  define NLIB_SHARED_LOCK_FUNC(...) __attribute__((acquire_shared_capability(__VA_ARGS__)))
#  define NLIB_UNLOCK_FUNC(...) __attribute__((release_capability(__VA_ARGS__)))
#  define NLIB_SHARED_UNLOCK_FUNC(...) __attribute__((release_shared_capability(__VA_ARGS__)))
#  define NLIB_TRYLOCK_FUNC(...) __attribute__((try_acquire_capability(__VA_ARGS__)))
#  define NLIB_SHARED_TRYLOCK_FUNC(...) __attribute__((try_acquire_shared_capability(__VA_ARGS__)))
#  define NLIB_GUARDED_BY(x) __attribute__((guarded_by(x)))
#  define NLIB_PT_GUARDED_BY(x) __attribute__((pt_guarded_by(x)))
#  define NLIB_LOCK_REQUIRED(...) __attribute__((requires_capability(__VA_ARGS__)))
#  define NLIB_LOCK_EXCLUDED(...) __attribute__((locks_excluded(__VA_ARGS__)))
#  define NLIB_SHARED_LOCK_REQUIRED(...) __attribute__((requires_shared_capability(__VA_ARGS__)))
#  define NLIB_SCOPED_LOCKABLE __attribute__((scoped_lockable))
#  define NLIB_NO_THREAD_SAFETY_ANALYSIS __attribute__((no_thread_safety_analysis))
# else
#  define NLIB_LOCKABLE __attribute__((lockable))
#  define NLIB_LOCK_FUNC(...) __attribute__((exclusive_lock_function(__VA_ARGS__)))
#  define NLIB_SHARED_LOCK_FUNC(...) __attribute__((shared_lock_function(__VA_ARGS__)))
#  define NLIB_UNLOCK_FUNC(...) __attribute__((unlock_function(__VA_ARGS__)))
#  define NLIB_TRYLOCK_FUNC(...) __attribute__((exclusive_trylock_function(__VA_ARGS__)))
#  define NLIB_SHARED_TRYLOCK_FUNC(...) __attribute__((shared_trylock_function(__VA_ARGS__)))
#  define NLIB_GUARDED_BY(x) __attribute__((guarded_by(x)))
#  define NLIB_PT_GUARDED_BY(x) __attribute__((pt_guarded_by(x)))
#  define NLIB_LOCK_REQUIRED(...) __attribute__((exclusive_locks_required(__VA_ARGS__)))
#  define NLIB_LOCK_EXCLUDED(...) __attribute__((locks_excluded(__VA_ARGS__)))
#  define NLIB_SHARED_LOCK_REQUIRED(...) __attribute__((shared_locks_required(__VA_ARGS__)))
#  define NLIB_SCOPED_LOCKABLE __attribute__((scoped_lockable))
#  define NLIB_NO_THREAD_SAFETY_ANALYSIS __attribute__((no_thread_safety_analysis))
# endif
#else
// __GNUC__
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 4)
#  define NLIB_CXX11_NEW_CHARACTER_TYPES
# endif
# if __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6)
#   define NLIB_CXX11_NOEXCEPT
# endif
// # if !defined(__CYGWIN__) && (!defined(__GLIBC__) || __GLIBC__ < 2 || __GLIBC_MINOR__ < 1)
// #  error Sorry, glibc is old.
// # endif
#endif

#ifdef PTHREAD_RWLOCK_INITIALIZER
# define NLIB_RWLOCK_HAS_NATIVE
#endif
#ifdef NLIB_RWLOCK_HAS_NATIVE
typedef pthread_rwlock_t nlib_rwlock;
#define NLIB_RWLOCK_INITIALIZER PTHREAD_RWLOCK_INITIALIZER
#endif

#ifdef PTHREAD_BARRIER_SERIAL_THREAD
# define NLIB_BARRIER_HAS_NATIVE
#endif
#ifdef NLIB_BARRIER_HAS_NATIVE
typedef pthread_barrier_t nlib_barrier;
#endif

#define NLIB_THREAD_ATTR_HAS_NATIVE

#ifndef pthread_cleanup_push
# error pthread_cleanup_push must be a macro
#endif

#ifndef pthread_cleanup_pop
# error pthread_cleanup_pop must be a macro
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if (defined(__clang__) && defined(NLIB_64BIT)) || \
    (defined(__GNUC__) && __GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 7))

#define NLIB_ATOMIC_RELAXED __ATOMIC_RELAXED
#define NLIB_ATOMIC_ACQUIRE __ATOMIC_ACQUIRE
#define NLIB_ATOMIC_RELEASE __ATOMIC_RELEASE
#define NLIB_ATOMIC_ACQ_REL __ATOMIC_ACQ_REL
#define NLIB_ATOMIC_SEQ_CST __ATOMIC_SEQ_CST

#ifdef NLIB_DOXYGEN
int32_t nlib_atomic_load32(const int32_t* ptr, int memorder);
void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val, int memorder);
int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                   int32_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val, int memorder);
int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val, int memorder);

int64_t nlib_atomic_load64(const int64_t* ptr, int memorder);
void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val, int memorder);
int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                   int64_t desired, int weak,
                                   int success_memorder, int failure_memorder);
int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val, int memorder);
int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val, int memorder);

void* nlib_atomic_loadptr(void* const* ptr, int memorder);
void nlib_atomic_storeptr(void** ptr, void* val, int memorder);
int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                    void* desired, int weak,
                                    int success_memorder, int failure_memorder);
void nlib_atomic_thread_fence(int memorder);
#endif

static __inline int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    return __atomic_load_n(ptr, memorder);
}

static __inline void nlib_atomic_store32(int32_t* ptr, int32_t val, int memorder) {
    __atomic_store_n(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val,
                                               int memorder) {
    return __atomic_exchange_n(ptr, val, memorder);
}

static __inline int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                                   int32_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    return __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
}

static __inline int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_add_fetch(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_sub_fetch(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_and_fetch(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_xor_fetch(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val,
                                               int memorder) {
    return __atomic_or_fetch(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_fetch_add(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_fetch_sub(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_fetch_and(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val,
                                                int memorder) {
    return __atomic_fetch_xor(ptr, val, memorder);
}

static __inline int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val,
                                               int memorder) {
    return __atomic_fetch_or(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    return __atomic_load_n(ptr, memorder);
}

static __inline void nlib_atomic_store64(int64_t* ptr, int64_t val, int memorder) {
    __atomic_store_n(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val,
                                               int memorder) {
    return __atomic_exchange_n(ptr, val, memorder);
}

static __inline int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                                   int64_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    return __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
}

static __inline int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_add_fetch(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_sub_fetch(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_and_fetch(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_xor_fetch(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val,
                                               int memorder) {
    return __atomic_or_fetch(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_fetch_add(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_fetch_sub(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_fetch_and(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val,
                                                int memorder) {
    return __atomic_fetch_xor(ptr, val, memorder);
}

static __inline int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val,
                                               int memorder) {
    return __atomic_fetch_or(ptr, val, memorder);
}

static __inline void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    return __atomic_load_n(ptr, memorder);
}

static __inline void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    __atomic_store_n(ptr, val, memorder);
}

static __inline int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                                    void* desired, int weak,
                                                    int success_memorder, int failure_memorder) {
    return __atomic_compare_exchange_n(ptr, expected, desired, weak,
                                       success_memorder, failure_memorder);
}

static __inline void nlib_atomic_thread_fence(int memorder) {
    __atomic_thread_fence(memorder);
}

#else
#define NLIB_ATOMIC_RELAXED 0
#define NLIB_ATOMIC_ACQUIRE 1
#define NLIB_ATOMIC_RELEASE 2
#define NLIB_ATOMIC_ACQ_REL 3
#define NLIB_ATOMIC_SEQ_CST 7

static __inline int32_t nlib_atomic_load32(const int32_t* ptr, int memorder) {
    int32_t rval = *(volatile int32_t*)ptr;
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store32(int32_t* ptr, int32_t val,
                                                   int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    __sync_lock_test_and_set(ptr, val);
}

static __inline int32_t nlib_atomic_exchange32(int32_t* ptr, int32_t val,
                                               int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    return __sync_lock_test_and_set(ptr, val);
}

static __inline int nlib_atomic_compare_exchange32(int32_t* ptr, int32_t* expected,
                                                   int32_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int32_t old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline int32_t nlib_atomic_add_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_add_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_sub_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_sub_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_and_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_and_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_xor_fetch32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_xor_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_or_fetch32(int32_t* ptr, int32_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_or_and_fetch(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_add32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_add(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_sub32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_sub(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_and32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_and(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_xor32(int32_t* ptr, int32_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_xor(ptr, val);
}

static __inline int32_t nlib_atomic_fetch_or32(int32_t* ptr, int32_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_fetch_and_or(ptr, val);
}

static __inline int64_t nlib_atomic_load64(const int64_t* ptr, int memorder) {
    int64_t rval = *(volatile int64_t*)ptr;
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_store64(int64_t* ptr, int64_t val,
                                                   int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    __sync_lock_test_and_set(ptr, val);
}

static __inline int64_t nlib_atomic_exchange64(int64_t* ptr, int64_t val,
                                               int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    return __sync_lock_test_and_set(ptr, val);
}

static __inline int nlib_atomic_compare_exchange64(int64_t* ptr, int64_t* expected,
                                                   int64_t desired, int weak,
                                                   int success_memorder, int failure_memorder) {
    int64_t old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline int64_t nlib_atomic_add_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_add_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_sub_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_sub_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_and_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_and_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_xor_fetch64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_xor_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_or_fetch64(int64_t* ptr, int64_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_or_and_fetch(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_add64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_add(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_sub64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_sub(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_and64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_and(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_xor64(int64_t* ptr, int64_t val,
                                                int memorder) {
    (void)memorder;
    return __sync_fetch_and_xor(ptr, val);
}

static __inline int64_t nlib_atomic_fetch_or64(int64_t* ptr, int64_t val,
                                               int memorder) {
    (void)memorder;
    return __sync_fetch_and_or(ptr, val);
}

static __inline void* nlib_atomic_loadptr(void* const* ptr, int memorder) {
    void* rval = *(void* volatile *)ptr;  // NOLINT
    (void)memorder;
#if !defined(__i386__) && !defined(__x86_64__)
    if (memorder & NLIB_ATOMIC_ACQUIRE)
        NLIB_MEMORY_ORDER_ACQUIRE;
#endif
    return rval;
}

static NLIB_ALWAYS_INLINE void nlib_atomic_storeptr(void** ptr, void* val, int memorder) {
    if (memorder == NLIB_ATOMIC_SEQ_CST)
        __sync_synchronize();
    else if (memorder & NLIB_ATOMIC_RELEASE)
        NLIB_MEMORY_ORDER_RELEASE;
    void* tmp = __sync_lock_test_and_set(ptr, val);
    (void)tmp;
}

static __inline int nlib_atomic_compare_exchangeptr(void** ptr, void** expected,
                                                    void* desired, int weak,
                                                    int success_memorder, int failure_memorder) {
    void* old = __sync_val_compare_and_swap(ptr, *expected, desired);
    if (old == *expected) return 1;
    *expected = old;

    (void)weak;
    (void)success_memorder;
    (void)failure_memorder;
    return 0;
}

static __inline void nlib_atomic_thread_fence(int memorder) {
    switch (memorder) {
    case NLIB_ATOMIC_RELAXED:
        break;
    case NLIB_ATOMIC_ACQUIRE:
        NLIB_MEMORY_ORDER_ACQUIRE;
        break;
    case NLIB_ATOMIC_RELEASE:
        NLIB_MEMORY_ORDER_RELEASE;
        break;
    case NLIB_ATOMIC_ACQ_REL:
        NLIB_MEMORY_ORDER_ACQ_REL;
        break;
    default:
        __sync_synchronize();
        break;
    }
}
#endif


#ifdef __cplusplus
}
#endif

#endif
#endif  // INCLUDE_NN_NLIB_PLATFORM_UNIX_H_
