﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SINGLETON_H_
#define INCLUDE_NN_NLIB_SINGLETON_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN

template <class T>
class Singleton NLIB_FINAL {
 public:
    static T& Instance() {
        if (!m_Flag) {
            if (!nlib_atomic_load32(&m_Flag, NLIB_ATOMIC_ACQUIRE))
                nlib_once(&m_OnceFlag, Init);
        }
        T* ptr = reinterpret_cast<T*>(&m_Memory[0]);
        return *ptr;
    }
    static T& GetInstance() { return Instance(); }

 private:
    static void Init();
    static int32_t m_Flag;
    static nlib_onceflag m_OnceFlag;
    // Memory leak would be mis-detected if dynamically allocated.
    NLIB_ALIGNAS(8) static unsigned char m_Memory[sizeof(T)];

 private:
    Singleton() {}
    NLIB_DISALLOW_COPY_AND_ASSIGN(Singleton);
};

template <class T>
void Singleton<T>::Init() {
    new (m_Memory) T();
    nlib_atomic_store32(&m_Flag, 1, NLIB_ATOMIC_RELEASE);
}

template <class T>
nlib_onceflag Singleton<T>::m_OnceFlag = NLIB_ONCE_INIT;
template <class T>
int32_t Singleton<T>::m_Flag = 0;
template <class T>

#ifndef NLIB_DOXYGEN
#if !defined(NN_PLATFORM_CTR)
NLIB_ALIGNAS(8) unsigned char Singleton<T>::m_Memory[sizeof(T)];
#else
unsigned char Singleton<T>::m_Memory[sizeof(T)];
#endif
#endif

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SINGLETON_H_
