﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_STRTO_H_
#define INCLUDE_NN_NLIB_STRTO_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN

// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, int base, int32_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, int base, int64_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, int base, uint32_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, int base, uint64_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, double* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrTo(const char* s, float* v) NLIB_NOEXCEPT NLIB_NONNULL;

// ERANGE if overflow, EILSEQ if an invalid char found
inline errno_t StrTo(const char* s, int base, int8_t* v) NLIB_NOEXCEPT {
    int32_t tmp;
    errno_t e = StrTo(s, base, &tmp);
    if (e != 0) return e;
    if (tmp > 127 || tmp < -128) return ERANGE;
    *v = static_cast<int8_t>(tmp);
    return 0;
}

// ERANGE if overflow, EILSEQ if an invalid char found
inline errno_t StrTo(const char* s, int base, int16_t* v) NLIB_NOEXCEPT {
    int32_t tmp;
    errno_t e = StrTo(s, base, &tmp);
    if (e != 0) return e;
    if (tmp > 32767 || tmp < -32768) return ERANGE;
    *v = static_cast<int16_t>(tmp);
    return 0;
}

// ERANGE if overflow, EILSEQ if an invalid char found
inline errno_t StrTo(const char* s, int base, uint8_t* v) NLIB_NOEXCEPT {
    uint32_t tmp;
    errno_t e = StrTo(s, base, &tmp);
    if (e != 0) return e;
    if (tmp > 255) return ERANGE;
    *v = static_cast<uint8_t>(tmp);
    return 0;
}

// ERANGE if overflow, EILSEQ if an invalid char found
inline errno_t StrTo(const char* s, int base, uint16_t* v) NLIB_NOEXCEPT {
    uint32_t tmp;
    errno_t e = StrTo(s, base, &tmp);
    if (e != 0) return e;
    if (tmp > 65535) return ERANGE;
    *v = static_cast<uint16_t>(tmp);
    return 0;
}

// ERANGE if overflow, EILSEQ if an invalid char found
template <class T>
NLIB_VIS_HIDDEN errno_t StrTo(const char* s, int base, T* v) NLIB_NOEXCEPT;

#define NLIB_DEF_STRTO(from_type, to_type)                                            \
template<>                                                                            \
NLIB_ALWAYS_INLINE errno_t StrTo(const char* s, int base, from_type* v) NLIB_NOEXCEPT \
{                                                                                     \
        return StrTo(s, base, reinterpret_cast<to_type*>(v));                         \
}

NLIB_DEF_STRTO(int, int32_t)
// NLIB_DEF_STRTO(char, int32_t)
NLIB_DEF_STRTO(signed char, int8_t)
NLIB_DEF_STRTO(short, int16_t)                // NOLINT
NLIB_DEF_STRTO(long, nlib_long_compatible_t)  // NOLINT
NLIB_DEF_STRTO(long long, int64_t)            // NOLINT

NLIB_DEF_STRTO(unsigned int, uint32_t)
NLIB_DEF_STRTO(unsigned char, uint8_t)
NLIB_DEF_STRTO(unsigned short, uint16_t)                // NOLINT
NLIB_DEF_STRTO(unsigned long, nlib_ulong_compatible_t)  // NOLINT
NLIB_DEF_STRTO(unsigned long long, uint64_t)            // NOLINT

// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, int base,
                                      int32_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, int base,
                                      int64_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, int base,
                                      uint32_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, int base,
                                      uint64_t* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, double* v) NLIB_NOEXCEPT NLIB_NONNULL;
// ERANGE if overflow, EILSEQ if an invalid char found
NLIB_VIS_PUBLIC errno_t StrToFallback(const char* s, float* v) NLIB_NOEXCEPT NLIB_NONNULL;

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_STRTO_H_
