﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_SOCKET_ENABLED

#include "nn/nlib/Config.h"
#include "nn/nlib/InputStream.h"
#include "nn/nlib/UniquePtr.h"

NLIB_NAMESPACE_BEGIN

struct TcpInputStreamSettings {
    void* buf;
    size_t buffer_size;
    TcpInputStreamSettings() : buf(NULL), buffer_size(8192) {}
};

#if defined(NLIB_WINDLL) && defined(NLIB_CXX11_UNIQUEPTR) && defined(NLIB_CXX11_TEMPLATE_ALIAS)
# ifdef nx_misc_EXPORTS
template class __declspec(dllexport) std::unique_ptr<uint8_t[]>;
# else
extern template class __declspec(dllimport) std::unique_ptr<uint8_t[]>;
# endif
#endif

#ifdef _MSC_VER
class TcpInputStream NLIB_FINAL : public InputStream {
#else
class NLIB_VIS_PUBLIC TcpInputStream NLIB_FINAL : public InputStream {
#endif

 public:
    TcpInputStream() NLIB_NOEXCEPT : m_Socket(NLIB_SOCKET_INVALID), m_Buf(NULL), m_BufSize(0) {}
    NLIB_VIS_PUBLIC virtual ~TcpInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(nlib_sock sockfd) NLIB_NOEXCEPT {
        TcpInputStreamSettings settings;
        return this->Init(sockfd, settings);
    }
    NLIB_VIS_PUBLIC errno_t Init(nlib_sock sockfd,
                                 const TcpInputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    NLIB_VIS_PUBLIC virtual size_t FillBuffer_(void* p,
                                               size_t nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    NLIB_VIS_PUBLIC virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;
    NLIB_VIS_PUBLIC virtual void* GetWorkBuffer_(size_t* nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_sock m_Socket;
    uint8_t* m_Buf;
    size_t m_BufSize;
    UniquePtr<uint8_t[]> m_BufferHolder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(TcpInputStream);
};

NLIB_NAMESPACE_END

#endif

#endif  // INCLUDE_NN_NLIB_TCPINPUTSTREAM_H_
