﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_

#include "nn/nlib/Platform.h"
#ifdef NLIB_SOCKET_ENABLED

#include "nn/nlib/Config.h"
#include "nn/nlib/OutputStream.h"
#include "nn/nlib/UniquePtr.h"

NLIB_NAMESPACE_BEGIN

struct TcpOutputStreamSettings {
    void* buf;
    size_t buffer_size;
    TcpOutputStreamSettings() : buf(NULL), buffer_size(8192) {}
};

#ifdef _MSC_VER
class TcpOutputStream NLIB_FINAL : public OutputStream {
#else
class NLIB_VIS_PUBLIC TcpOutputStream NLIB_FINAL : public OutputStream {
#endif

 public:
    TcpOutputStream() NLIB_NOEXCEPT : m_Socket(NLIB_SOCKET_INVALID), m_Buf(NULL), m_BufSize(0) {}
    NLIB_VIS_PUBLIC virtual ~TcpOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(nlib_sock sockfd) NLIB_NOEXCEPT {
        TcpOutputStreamSettings settings;
        return this->Init(sockfd, settings);
    }
    NLIB_VIS_PUBLIC errno_t Init(nlib_sock sockfd,
                                 const TcpOutputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    NLIB_VIS_PUBLIC virtual bool PushBuffer_(const void* p, size_t nBytes,
                                             bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    NLIB_VIS_PUBLIC virtual bool Close_() NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    nlib_sock m_Socket;
    uint8_t* m_Buf;
    size_t m_BufSize;
    UniquePtr<uint8_t[]> m_BufferHolder;
};

NLIB_NAMESPACE_END

#endif
#endif  // INCLUDE_NN_NLIB_TCPOUTPUTSTREAM_H_
