﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_URITEMPLATE_H_
#define INCLUDE_NN_NLIB_URITEMPLATE_H_

#include "nn/nlib/Uri.h"
#include "nn/nlib/Nlist.h"
#include "nn/nlib/UniquePtr.h"

NLIB_NAMESPACE_BEGIN

class OutputStream;

// See RFC6570: https://tools.ietf.org/html/rfc6570
class UriTemplate NLIB_FINAL {
 public:
    UriTemplate() NLIB_NOEXCEPT {}
    NLIB_VIS_PUBLIC ~UriTemplate() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC errno_t SetTemplate(const char* str) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC errno_t SetParameter(const char* param, const char* value) NLIB_NOEXCEPT;
    void ResetParameter() NLIB_NOEXCEPT {
        StringList<> tmp;
        tmp.swap(m_Pv);
    }
    NLIB_VIS_PUBLIC errno_t Resolve(OutputStream* os) const NLIB_NOEXCEPT;

    NLIB_VIS_PUBLIC bool Resolve(char* str, size_t n) const NLIB_NOEXCEPT;
    template <size_t N>
    bool Resolve(char (&str)[N]) const NLIB_NOEXCEPT {
        return Resolve(&str[0], N);
    }

 private:
    UniquePtr<char[]> m_Template;
    StringList<> m_Pv;
    NLIB_DISALLOW_COPY_AND_ASSIGN(UriTemplate);
};

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_URITEMPLATE_H_
