﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_WCHARINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_WCHARINPUTSTREAM_H_

#include "nn/nlib/InputConverterStream.h"

NLIB_NAMESPACE_BEGIN

class InputStream;

namespace detail {

class NLIB_VIS_PUBLIC UtfInputTransform : public InputTransform {
 public:
    typedef errno_t (*Func)(InputStream* istr, char (&utf8)[4], int* n);

    explicit UtfInputTransform(Func func) NLIB_NOEXCEPT;
    virtual errno_t Transform(InputStream* is, void* p, size_t size,
                              size_t* nBytes) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(InputStream* is) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose() NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    Func m_Func;
    unsigned char m_NumUtf8;
    char m_Utf8[3];
    unsigned char m_Buf[128];

    NLIB_DISALLOW_COPY_AND_ASSIGN(UtfInputTransform);
};

class NLIB_VIS_PUBLIC Utf32InputTransform NLIB_FINAL : public UtfInputTransform {
 public:
    Utf32InputTransform() NLIB_NOEXCEPT;
};

class NLIB_VIS_PUBLIC Utf16InputTransform NLIB_FINAL : public UtfInputTransform {
 public:
    Utf16InputTransform() NLIB_NOEXCEPT;
};

}  // namespace detail

#ifndef NLIB_DOXYGEN
typedef InputConverterStreamTempl<detail::Utf32InputTransform> Utf32InputStream;
typedef InputConverterStreamTempl<detail::Utf16InputTransform> Utf16InputStream;
#if NLIB_WCHAR_SIZE == 4
typedef Utf32InputStream WcharInputStream;
#else
typedef Utf16InputStream WcharInputStream;
#endif
#else
// Do not typedef for doxygen
class Utf32InputStream :
    public InputConverterStreamTempl<detail::Utf32InputTransform> {
 public:
};

class Utf16InputStream :
    public InputConverterStreamTempl<detail::Utf16InputTransform> {
 public:
};

#if NLIB_WCHAR_SIZE == 4
class WcharInputStream NLIB_FINAL : public Utf32InputStream {
 public:
};
#else
class WcharInputStream NLIB_FINAL : public Utf16InputStream {
 public:
};
#endif
#endif

NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_WCHARINPUTSTREAM_H_
