﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_
#define INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_
#ifndef NLIB_NO_ZLIB

#include <stdlib.h>
#include <zlib.h>
#include "nn/nlib/InputStream.h"
#include "nn/nlib/InputConverterStream.h"

NLIB_NAMESPACE_BEGIN

struct ZlibInputStreamSettings {
    typedef void* (*ReallocFunc)(void* p, size_t n);
    void* buf_in;
    void* buf_out;
    size_t buffer_size;
    ReallocFunc realloc_func;

 public:
    NLIB_CEXPR ZlibInputStreamSettings() NLIB_NOEXCEPT : buf_in(NULL),
                                                         buf_out(NULL),
                                                         buffer_size(64 * 1024),
                                                         realloc_func(::nlib_realloc) {}
    NLIB_CEXPR ZlibInputStreamSettings(void* buf_in_, void* buf_out_,
                                       size_t buffer_size_) NLIB_NOEXCEPT
        : buf_in(buf_in_),
          buf_out(buf_out_),
          buffer_size(buffer_size_),
          realloc_func(::nlib_realloc) {}
    NLIB_CEXPR ZlibInputStreamSettings(void* buf_in_, void* buf_out_, size_t buffer_size_,
                                       ReallocFunc realloc_func_) NLIB_NOEXCEPT
        : buf_in(buf_in_),
          buf_out(buf_out_),
          buffer_size(buffer_size_),
          realloc_func(realloc_func_) {}
};

namespace detail {

class NLIB_VIS_PUBLIC ZlibInputTransform NLIB_FINAL : public InputTransform {
 public:
    typedef ZlibInputStreamSettings::ReallocFunc ReallocFunc;
    ZlibInputTransform() NLIB_NOEXCEPT;
    virtual ~ZlibInputTransform() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(const ZlibInputStreamSettings& settings) NLIB_NOEXCEPT;
    virtual errno_t Transform(InputStream* is, void* p, size_t nBytes,
                              size_t* written) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(InputStream* is) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose() NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    z_stream m_Strm;
    ReallocFunc m_Realloc;
    unsigned char* m_BufIn;   // to z_stream::next_in
    unsigned char* m_BufOut;  // to z_stream::next_out
    size_t m_BufSize;         // for both m_BufIn and m_BufOut
    size_t m_BufOutCur;
    size_t m_BufOutDataSize;
    bool m_IsInternalBuffer;
    bool m_IsZStreamEnd;
    unsigned char m_BaseBuf[256];  // buffer for InputConverterStream

    NLIB_DISALLOW_COPY_AND_ASSIGN(ZlibInputTransform);
};

}  // namespace detail

class NLIB_VIS_PUBLIC ZlibInputStream NLIB_FINAL
    : public InputConverterStreamTempl<detail::ZlibInputTransform> {
 public:
    ZlibInputStream() NLIB_NOEXCEPT;
    virtual ~ZlibInputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR ZlibInputStreamSettings settings;
        return this->Init(settings);
    }
    errno_t Init(const ZlibInputStreamSettings& settings) NLIB_NOEXCEPT {
        return m_Transform.Init(settings);
    }

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(ZlibInputStream);
};

NLIB_NAMESPACE_END

#endif
#endif  // INCLUDE_NN_NLIB_ZLIBINPUTSTREAM_H_
