﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_ZLIBOUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_ZLIBOUTPUTSTREAM_H_
#ifndef NLIB_NO_ZLIB

#include <stdlib.h>
#include <zlib.h>
#include "nn/nlib/OutputConverterStream.h"

NLIB_NAMESPACE_BEGIN

class OutputStream;

struct ZlibOutputStreamSettings {
    typedef void* (*ReallocFunc)(void* p, size_t n);
    void* buf_in;
    void* buf_out;
    size_t buffer_size;
    int level;
    ReallocFunc realloc_func;

 public:
    NLIB_CEXPR ZlibOutputStreamSettings() NLIB_NOEXCEPT : buf_in(NULL),
                                                          buf_out(NULL),
                                                          buffer_size(64 * 1024),
                                                          level(Z_DEFAULT_COMPRESSION),
                                                          realloc_func(::nlib_realloc) {}
    NLIB_CEXPR ZlibOutputStreamSettings(void* buf_in_, void* buf_out_, size_t buffer_size_,
                                        int level_) NLIB_NOEXCEPT : buf_in(buf_in_),
                                                                    buf_out(buf_out_),
                                                                    buffer_size(buffer_size_),
                                                                    level(level_),
                                                                    realloc_func(::nlib_realloc) {}
    NLIB_CEXPR ZlibOutputStreamSettings(void* buf_in_, void* buf_out_, size_t buffer_size_,
                                        int level_, ReallocFunc realloc_func_) NLIB_NOEXCEPT
        : buf_in(buf_in_),
          buf_out(buf_out_),
          buffer_size(buffer_size_),
          level(level_),
          realloc_func(realloc_func_) {}
};

namespace detail {

class NLIB_VIS_PUBLIC ZlibOutputTransform NLIB_FINAL : public OutputTransform {
 public:
    typedef ZlibOutputStreamSettings::ReallocFunc ReallocFunc;
    ZlibOutputTransform() NLIB_NOEXCEPT;
    virtual ~ZlibOutputTransform() NLIB_NOEXCEPT NLIB_OVERRIDE;

    virtual errno_t Transform(OutputStream* os, const void* p, size_t nBytes,
                              bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;

    errno_t Init(const ZlibOutputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    z_stream m_Strm;
    ReallocFunc m_Realloc;
    unsigned char* m_BufIn;   // to z_stream::next_in
    unsigned char* m_BufOut;  // to z_stream::next_out
    size_t m_BufSize;         // for both m_BufIn and m_BufOut
    size_t m_BufInCur;
    int m_Level;
    bool m_IsInternalBuffer;
    unsigned char m_BaseBuf[256];  // buffer for the base class
};

}  // namespace detail

class NLIB_VIS_PUBLIC ZlibOutputStream NLIB_FINAL
    : public OutputConverterStreamTempl<detail::ZlibOutputTransform> {
 public:
    ZlibOutputStream() NLIB_NOEXCEPT;
    virtual ~ZlibOutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        NLIB_CEXPR ZlibOutputStreamSettings settings;
        return this->Init(settings);
    }
    errno_t Init(const ZlibOutputStreamSettings& settings) NLIB_NOEXCEPT {
        return m_Transform.Init(settings);
    }

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(ZlibOutputStream);
};

NLIB_NAMESPACE_END

#endif
#endif  // INCLUDE_NN_NLIB_ZLIBOUTPUTSTREAM_H_
