﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_
#define INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_

#include "nn/nlib/exi/ExiAllocator.h"
#include "nn/nlib/exi/Types.h"

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace exi {

class NLIB_VIS_PUBLIC ExiErrorStatus NLIB_FINAL {
 public:
    enum ErrorValue {
        OK = 0,
        OUT_OF_MEMORY,
        ARGUMENT_ERROR,
        IO_ERROR,
        UNEXPECTED_EOS,
        STREAM_DETACHED,
        ILLEGAL_CHARACTER,
        INTERNAL_ERROR,
        NOT_SUPPORTED,
        DUPLICATE_ATTRIBUTE,
        RESERVED_PREFIX,
        UNDECLARING_PREFIX,
        ATTRIBUTE_DEFAULT_NS,
        ILLEGAL_COMMENT,
        ILLEGAL_PI,
        ILLEGAL_BINARY,
        ILLEGAL_BINARY_IDX,
        ILLEGAL_BINARY_EVT,
        SYNTAX_ERROR,
        ILLEGAL_BINARY_HDR,
        FUNC_NOT_AVAILABLE
    };

 public:
    ExiErrorStatus() NLIB_NOEXCEPT : m_Error(OK), m_Line(0), m_FileName(NULL) {}
    bool IsOk() const NLIB_NOEXCEPT { return m_Error == OK; }
    ErrorValue GetErrorValue() const NLIB_NOEXCEPT { return static_cast<ErrorValue>(m_Error); }
    int GetLineNo() const NLIB_NOEXCEPT { return m_Line; }
    const char* GetFileName() const NLIB_NOEXCEPT { return m_FileName; }
    bool SetError(ErrorValue e, int line, const char* filename) NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT;
    NLIB_SAFE_BOOL(ExiErrorStatus, IsOk())

 private:
    int m_Error;             // ErrorValue
    int m_Line;              // __LINE__
    const char* m_FileName;  // __FILE__
    NEXI_DISALLOW_NEW_DELETE(ExiErrorStatus);
    NLIB_DISALLOW_COPY_AND_ASSIGN(ExiErrorStatus);
};

// for unit-tests, make NEXI_SET_ERROR() work if ErrorStatus is not set.
#define NEXI_SET_ERROR(e)                                                                     \
    (this->GetExiErrorStatus() ? this->GetExiErrorStatus()->SetError((e), __LINE__, __FILE__) \
                               : false)
#define NEXI_IS_ERROR (!this->GetExiErrorStatus()->IsOk())
#define NEXI_RESET_ERROR this->GetExiErrorStatus()->Reset()
#define NEXI_CHECK_OUT_OF_MEMORY \
    if (ExiAllocator::IsOutOfMemory()) NEXI_SET_ERROR(ExiErrorStatus::OUT_OF_MEMORY)

}  // namespace exi
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_exi_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_EXI_EXIERRORSTATUS_H_
