﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HEAP_CACHEDHEAP_H_
#define INCLUDE_NN_NLIB_HEAP_CACHEDHEAP_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace heap {

class TlsHeapCache;

class NLIB_VIS_PUBLIC CachedHeap NLIB_FINAL {
 public:
    CachedHeap() NLIB_NOEXCEPT : m_Body(NULL) {}
    ~CachedHeap() NLIB_NOEXCEPT { this->Finalize(); }
    NLIB_MOVE_MEMBER_HELPER_1(CachedHeap, m_Body);
    void swap(CachedHeap& rhs) NLIB_NOEXCEPT {
        using std::swap;
        swap(m_Body, rhs.m_Body);
    }

    void* Alloc(size_t n) NLIB_NOEXCEPT;                              // nmalloc()
    void* Alloc(size_t n, size_t algn) NLIB_NOEXCEPT;                 // nmalloc_aligned()
    size_t GetAllocSize(const void* p) NLIB_NOEXCEPT;                 // nmalloc_size()
    errno_t Free(void* p) NLIB_NOEXCEPT;                              // nfree()
    errno_t FreeWithSize(void* p, size_t size) NLIB_NOEXCEPT;         // nfree_size()
    errno_t Realloc(void* ptr, size_t size, void** p) NLIB_NOEXCEPT;  // nrealloc() minus
    errno_t SetMark1(void* ptr, uint16_t mark1) NLIB_NOEXCEPT;
    errno_t SetMark2(void* ptr, uint16_t mark2) NLIB_NOEXCEPT;
    errno_t GetMark(const void* ptr, uint16_t* mark1, uint16_t* mark2) NLIB_NOEXCEPT;
    void* GetObjPtr(void* raw_ptr) NLIB_NOEXCEPT;

    void ReleaseAllCache() NLIB_NOEXCEPT;  // nmalloc_clear_tls()
    void Finalize() NLIB_NOEXCEPT;         // nmalloc_finalize_tls()
    bool CheckCache() NLIB_NOEXCEPT;

    // Internal use only
    NLIB_VIS_HIDDEN void Reset(TlsHeapCache* body) NLIB_NOEXCEPT;
    NLIB_VIS_HIDDEN TlsHeapCache* Release() NLIB_NOEXCEPT;

    NLIB_SAFE_BOOL(CachedHeap, (m_Body != NULL))

 private:
    TlsHeapCache* m_Body;
    NLIB_DISALLOW_COPY_AND_ASSIGN(CachedHeap);
};

}  // namespace heap
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::heap::CachedHeap)

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_HEAP_CACHEDHEAP_H_
