﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_
#define INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_

#include "nn/nlib/heap/NMalloc.h"

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace heap {

class CachedHeap;
class TlsHeapCentral;
class NLIB_VIS_PUBLIC CentralHeap NLIB_FINAL {
 public:
    typedef void (*DestructorHandler)(void* start, void* end);

    CentralHeap() NLIB_NOEXCEPT : m_Central(NULL),
                                  m_UseVirtualMemory(false),
                                  m_HeapOption(0),
                                  m_Start(NULL),
                                  m_End(NULL) {}
    ~CentralHeap() NLIB_NOEXCEPT { this->Finalize(); }
    errno_t Initialize(void* start, size_t size, uint32_t heap_option) NLIB_NOEXCEPT;
    void Finalize() NLIB_NOEXCEPT;
    NLIB_ALWAYS_INLINE void* Alloc(size_t n) NLIB_NOEXCEPT { return this->Alloc(n, 8); }
    void* Alloc(size_t n, size_t algn) NLIB_NOEXCEPT;
    errno_t Free(void* p) NLIB_NOEXCEPT;

    bool IsClean() NLIB_NOEXCEPT;
    bool MakeCache(CachedHeap* ptr) NLIB_NOEXCEPT;
    void CalcHeapHash(HeapHash* hash) NLIB_NOEXCEPT;
    void DumpEx(NMallocDumpMode mode, nlib_fd fd) const NLIB_NOEXCEPT;
    void DumpEx(NMallocDumpMode mode) const NLIB_NOEXCEPT { DumpEx(mode, 1); }
    void Dump() const NLIB_NOEXCEPT { DumpEx(NMALLOC_DUMP_BASIC, 1); }
    errno_t WalkAllocatedPtrs(nmalloc_heapwalk_callback func, void* user_ptr) NLIB_NOEXCEPT;
    static const size_t kPageSize;

 private:
    TlsHeapCentral* m_Central;
    bool m_UseVirtualMemory;
    uint32_t m_HeapOption;
    uint8_t* m_Start;
    uint8_t* m_End;
    NLIB_DISALLOW_COPY_AND_ASSIGN(CentralHeap);
};

}  // namespace heap
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_HEAP_CENTRALHEAP_H_
