﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_
#define INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/DynamicAlignedStorage.h"

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace heap {

class NLIB_VIS_PUBLIC UnitHeap NLIB_FINAL {
 public:
    typedef void (*DestructorHandler)(void* start, void* end);
    UnitHeap() NLIB_NOEXCEPT;
    ~UnitHeap() NLIB_NOEXCEPT {
        // Do not assert by GetAllocatedCount().
        // Allow deallote without destroy.
        if (m_Handler) {
            (*m_Handler)(m_Start, m_End);
        }
    }
    errno_t Initialize(void* start, size_t size, DestructorHandler handler, size_t unit,
                       size_t algn, uint32_t heap_option) NLIB_NOEXCEPT;
    void* Alloc() NLIB_NOEXCEPT;
    errno_t Free(void* p) NLIB_NOEXCEPT;
    size_t GetUnitSize() const NLIB_NOEXCEPT { return m_UnitSize; }
    size_t GetUnitCount() const NLIB_NOEXCEPT { return m_UnitCount; }
    size_t GetAllocatedCount() const NLIB_NOEXCEPT { return m_AllocatedCount; }
    void Dump() NLIB_NOEXCEPT;

 private:
    struct Node {
        Node* next;
    };

    size_t m_UnitSize;
    size_t m_BaseUnitCount;
    Node* m_FreeList;
    size_t m_UnitCount;
    size_t m_AllocatedCount;

    uint32_t m_HeapOption;
    uint8_t* m_Start;
    uint8_t* m_End;
    DestructorHandler m_Handler;
};

}  // namespace heap
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_heap_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_HEAP_UNITHEAP_H_
