﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_
#define INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_

#include "nn/nlib/msgpack/MpObject.h"
#include "nn/nlib/InputStream.h"
#include "nn/nlib/TextReader.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {

class NLIB_VIS_PUBLIC CsvReader NLIB_FINAL {
 public:
    // In Conformance with RFC4180(https://www.ietf.org/rfc/rfc4180.txt).
    // except that
    //  pass UTF-8, the UTF-8 string is checked validity.
    //  pass CR, LF, CRLF as a new line.
    CsvReader() NLIB_NOEXCEPT : m_ErrorValue(0) {}
    ~CsvReader() NLIB_NOEXCEPT { m_Reader.Close(); }
    bool Init(InputStream* stream) NLIB_NOEXCEPT NLIB_NONNULL;
    bool Close() NLIB_NOEXCEPT { return m_Reader.Close(); }
    errno_t GetErrorValue() const NLIB_NOEXCEPT { return m_ErrorValue; }

 public:
    static bool Read(MpObject* obj, const char* csvtext) NLIB_NOEXCEPT NLIB_NONNULL;
    bool Read(MpObject* obj) NLIB_NOEXCEPT NLIB_NONNULL;
    InputStream* GetStream() NLIB_NOEXCEPT { return m_Reader.GetStream(); }
    NLIB_SAFE_BOOL(CsvReader, GetErrorValue() == 0)

 private:
    void SetError(errno_t e) NLIB_NOEXCEPT {
        if (m_ErrorValue == 0) m_ErrorValue = e;
    }

 private:
    errno_t m_ErrorValue;
    TextReader m_Reader;
    NLIB_DISALLOW_COPY_AND_ASSIGN(CsvReader);
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_CSVREADER_H_
