﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONREADER_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONREADER_H_

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/TextReader.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
class InputStream;
namespace msgpack {
class MpObject;

struct NLIB_VIS_PUBLIC JsonReaderSettings {
    size_t maxArraySize;
    size_t maxMapSize;
    size_t maxStrLen;
    size_t maxDepth;

 public:
    NLIB_CEXPR JsonReaderSettings(size_t max_array_size_, size_t max_map_size_,
                                  size_t max_str_len_, size_t max_depth_) NLIB_NOEXCEPT
                                  : maxArraySize(max_array_size_),
                                    maxMapSize(max_map_size_),
                                    maxStrLen(max_str_len_),
                                    maxDepth(max_depth_) {}
    NLIB_CEXPR JsonReaderSettings() NLIB_NOEXCEPT : maxArraySize(8192),
                                                    maxMapSize(256),
                                                    maxStrLen(2047),
                                                    maxDepth(7) {}
};

class JsonReader NLIB_FINAL {
 public:
    enum Option {
        OPTION_DEFAULT = 0x00000000,
        OPTION_RELAXED = 0x00000001
    };
    NLIB_VIS_PUBLIC JsonReader() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC ~JsonReader() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Init(InputStream* stream,
                              const JsonReaderSettings& settings) NLIB_NOEXCEPT NLIB_NONNULL;
    bool Init(InputStream* stream) NLIB_NOEXCEPT NLIB_NONNULL {
        NLIB_CEXPR JsonReaderSettings settings;
        return this->Init(stream, settings);
    }
    bool Close() NLIB_NOEXCEPT { return true; }
    errno_t GetErrorValue() const NLIB_NOEXCEPT { return m_ErrorValue; }
    InputStream* GetStream() NLIB_NOEXCEPT { return m_Reader.GetStream(); }
    int GetLine() const NLIB_NOEXCEPT { return m_Reader.GetLine(); }
    int GetColumn() const NLIB_NOEXCEPT { return m_Reader.GetColumn(); }

 public:
    NLIB_VIS_PUBLIC static errno_t ReadEx(MpObject* obj, const void* jsontext, uint32_t option = 0,
                                          size_t n = RSIZE_MAX) NLIB_NOEXCEPT NLIB_NONNULL;
    static bool Read(MpObject* obj, const char* jsontext,
                     uint32_t option = 0) NLIB_NOEXCEPT NLIB_NONNULL {
        // This can be used if jsontext is NULL terminated.
        return ReadEx(obj, jsontext, option, RSIZE_MAX) == 0;
    }
    NLIB_VIS_PUBLIC bool Read(MpObject* obj, uint32_t option = 0) NLIB_NOEXCEPT NLIB_NONNULL;
    NLIB_SAFE_BOOL(JsonReader, GetErrorValue() == 0);

 private:
    void SetError(errno_t e) NLIB_NOEXCEPT {
        if (m_ErrorValue == 0) m_ErrorValue = e;
    }
    NLIB_VIS_HIDDEN bool GetValue(MpObject* obj) NLIB_NOEXCEPT;
    NLIB_VIS_HIDDEN bool GetString(MpObject* obj) NLIB_NOEXCEPT;
    NLIB_VIS_HIDDEN bool GetNumber(MpObject* obj) NLIB_NOEXCEPT;

 private:
    errno_t m_ErrorValue;
    size_t m_MaxArraySize;
    size_t m_MaxMapSize;
    size_t m_MaxStrLen;
    size_t m_MaxDepth;
    UniquePtr<char[]> m_StrBuf;
    size_t m_Depth;
    TextReader m_Reader;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonReader);
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONREADER_H_
