﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONWRITER_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONWRITER_H_

#include "nn/nlib/Config.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
class OutputStream;
namespace msgpack {
class MpObject;

class NLIB_VIS_PUBLIC JsonWriter NLIB_FINAL {
 public:
    enum Option {
        OPTION_DEFAULT = 0x00000000,
        OPTION_RELAXED = 0x00000001
    };
    JsonWriter() NLIB_NOEXCEPT : m_ErrorValue(0), m_Stream(NULL) {}
    ~JsonWriter() NLIB_NOEXCEPT { this->Close(); }
    bool Init(OutputStream* stream) NLIB_NOEXCEPT NLIB_NONNULL;
    bool Flush() NLIB_NOEXCEPT;
    bool Close() NLIB_NOEXCEPT;
    errno_t GetErrorValue() const NLIB_NOEXCEPT { return m_ErrorValue; }

 public:
    bool Write(const MpObject& obj, uint32_t option = 0) NLIB_NOEXCEPT;
    OutputStream* GetStream() NLIB_NOEXCEPT { return m_Stream; }

    static bool Write(char* str, size_t n, const MpObject& obj,
                      uint32_t option = 0) NLIB_NOEXCEPT NLIB_NONNULL;
    template <size_t n>
    static bool Write(char (&str)[n], const MpObject& obj, uint32_t option = 0) NLIB_NOEXCEPT {
        return Write(&str[0], n, obj, option);
    }
    NLIB_SAFE_BOOL(JsonWriter, GetErrorValue() == 0);

 private:
    NLIB_VIS_HIDDEN bool Write_(const MpObject& obj) NLIB_NOEXCEPT;
    void SetError(errno_t e) NLIB_NOEXCEPT {
        if (m_ErrorValue == 0) m_ErrorValue = e;
    }
    NLIB_VIS_HIDDEN bool WriteString(const char* p, size_t n) NLIB_NOEXCEPT NLIB_NONNULL;

 private:
    errno_t m_ErrorValue;
    OutputStream* m_Stream;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonWriter);
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONWRITER_H_
