﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
#define INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_

#include "nn/nlib/Config.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {

class MpObject;
class NLIB_VIS_PUBLIC MpWalker NLIB_FINAL {
 public:
    MpWalker() NLIB_NOEXCEPT : m_Cur(NULL), m_nBytes(0) {}
    ~MpWalker() NLIB_NOEXCEPT {}
    MpWalker(const void* p, size_t n) NLIB_NOEXCEPT;
    bool Init(const void* p, size_t n) NLIB_NOEXCEPT NLIB_NONNULL {
        if (n > RSIZE_MAX) return false;
        m_Cur = reinterpret_cast<const uint8_t*>(p);
        m_nBytes = n;
        return true;
    }

    MpWalker operator[](size_t array_idx) const NLIB_NOEXCEPT;
    MpWalker operator[](const char* key) const NLIB_NOEXCEPT NLIB_NONNULL;
    MpWalker operator[](int array_idx) const NLIB_NOEXCEPT {
        if (array_idx < 0)
            return MpWalker();
        else
            return operator[](static_cast<size_t>(array_idx));
    }
    const void* GetRaw(uint32_t* n) const NLIB_NOEXCEPT NLIB_NONNULL;

    const void* GetPtr() const NLIB_NOEXCEPT { return m_Cur; }
    size_t GetSize() const NLIB_NOEXCEPT { return m_nBytes; }

    NLIB_SAFE_BOOL(MpWalker, IsOk())

 private:
    bool IsOk() const NLIB_NOEXCEPT { return m_Cur != NULL && m_nBytes > 0; }

    const uint8_t* m_Cur;
    size_t m_nBytes;
};

}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_MPWALKER_H_
