﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/threading/Future.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {
namespace jsonrpc {
typedef uint32_t reqid_t;

class JsonRpcResponseHandler;
class JsonRpcResponse;

class NLIB_VIS_PUBLIC JsonRpcClient NLIB_FINAL {
 public:
    static JsonRpcClient OpenClient(const char* name);
    static JsonRpcClient SearchClient(const char* name);
    static void CloseClient(const JsonRpcClient& client);
    static void Shutdown();

 public:
    typedef void (*NoTargetResponseHandler)(UniquePtr<JsonRpcResponse>& rhs);  // NOLINT
    JsonRpcClient() : m_Handle(0) {}
    JsonRpcClient(const JsonRpcClient& rhs) : m_Handle(rhs.m_Handle) {}
    JsonRpcClient& operator=(const JsonRpcClient& rhs) {
        m_Handle = rhs.m_Handle;
        return *this;
    }
    void swap(JsonRpcClient& rhs) {  // NOLINT
        using std::swap;
        swap(m_Handle, rhs.m_Handle);
    }

 public:
    reqid_t GenerateId();

    // future becomes valid after response or timeout
    typedef ::nlib_ns::threading::Future<UniquePtr<JsonRpcResponse> > FutureType;
    errno_t GetFutureForId(FutureType* future, reqid_t reqid);
    errno_t ResolveResponse(const void* buf, size_t nbytes);
    errno_t Abort(reqid_t reqid);
    errno_t SetNoTargetResponseHandler(NoTargetResponseHandler handler);

    bool operator==(const JsonRpcClient& rhs) const { return m_Handle == rhs.m_Handle; }
    bool operator!=(const JsonRpcClient& rhs) const { return m_Handle != rhs.m_Handle; }
    NLIB_SAFE_BOOL(JsonRpcClient, m_Handle != 0);

 private:
    explicit NLIB_VIS_HIDDEN JsonRpcClient(uint32_t handle) : m_Handle(handle) {}

 private:
    uint32_t m_Handle;
    friend class JsonRpcClientImpl;
};

}  // namespace jsonrpc
}  // namespace msgpack
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::msgpack::jsonrpc::JsonRpcClient)

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCCLIENT_H_
