﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCREQUEST_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCREQUEST_H_

#include <iterator>

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/ReallocOutputStream.h"
#include "nn/nlib/Cstring.h"
#include "nn/nlib/msgpack/MpObject.h"
#include "nn/nlib/msgpack/MpWalker.h"
#include "nn/nlib/msgpack/jsonrpc/JsonRpcResponse.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {
namespace jsonrpc {

namespace fortest {
class JsonRpcErrorRequestTest;
}  // namespace fortest

typedef uint32_t reqid_t;

class JsonRpcRequest NLIB_FINAL {
 public:
    // NOTE:
    // If m_Id is 0, the request becomes notification(ID is not sent.).
    JsonRpcRequest() : m_Id(0) {}
    reqid_t GetId() const { return m_Id; }
    const char* GetMethod() const { return m_Method.get(); }
    const MpObject& GetParams() const { return m_Params; }
    MpObject& GetParams() { return m_Params; }

    void SetId(reqid_t id) { m_Id = id; }
    NLIB_VIS_PUBLIC bool SetMethod(const char* method);
    void MoveParamsFrom(MpObject& params) {  // NOLINT
        MpObject tmp;
        tmp.swap(params);
        tmp.swap(m_Params);
    }

 private:
    NLIB_VIS_HIDDEN JsonRpcResponse& GetResponse() { return m_Response; }
    NLIB_VIS_HIDDEN bool SetMethodRaw(const void* p, size_t n);
    NLIB_VIS_HIDDEN errno_t Init(MpObject* rhs);
    NLIB_VIS_HIDDEN errno_t Init(MpWalker* walker);

 private:
    reqid_t m_Id;
    UniquePtr<char[]> m_Method;
    MpObject m_Params;
    JsonRpcResponse m_Response;

    friend NLIB_VIS_PUBLIC size_t JsonRpcServerExec(const void* p, size_t n,
                                                    ReallocOutputStream::UniquePtrType* ptr);
    friend class JsonRpcRequestReader;
    friend class fortest::JsonRpcErrorRequestTest;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcRequest);
};

class JsonRpcRequestWriter NLIB_FINAL {
 public:
    JsonRpcRequestWriter() : m_IsMsgpack(false), m_Cur(0), m_Count(0) {}
    explicit JsonRpcRequestWriter(bool use_msgpack)
        : m_IsMsgpack(use_msgpack), m_Cur(0), m_Count(0) {}
    NLIB_VIS_PUBLIC errno_t BeginWriteRequest(uint32_t n);
    NLIB_VIS_PUBLIC errno_t WriteRequest(const char* method, reqid_t id, const MpObject& params);
    errno_t WriteNotification(const char* method, const MpObject& params) {
        return WriteRequest(method, 0, params);
    }
    errno_t WriteRequest(const JsonRpcRequest& req) {
        return WriteRequest(req.GetMethod(), req.GetId(), req.GetParams());
    }
    NLIB_VIS_PUBLIC errno_t EndWriteRequest(ReallocOutputStream::UniquePtrType* ptr, size_t* n);

 private:
    NLIB_VIS_HIDDEN errno_t WriteJsonRequest(const char* method, reqid_t id,
                                             const MpObject& params);
    NLIB_VIS_HIDDEN errno_t WriteMsgpackRequest(const char* method, reqid_t id,
                                                const MpObject& params);

 private:
    bool m_IsMsgpack;
    size_t m_Cur;
    size_t m_Count;
    ReallocOutputStream m_Ostr;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcRequestWriter);
};

class NLIB_VIS_PUBLIC JsonRpcRequestReader NLIB_FINAL {
 public:
    typedef Nlist<JsonRpcRequest> ListType;
    static errno_t ReadRequest(const void* p, size_t n, ListType* request_list);

 private:
    NLIB_VIS_HIDDEN static errno_t ReadReqJson(const char* p, size_t n,
                                               ListType* request_list);
    NLIB_VIS_HIDDEN static errno_t ReadReqMsgpack(const uint8_t* p, size_t n,
                                                  ListType* request_list);

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcRequestReader);
};

}  // namespace jsonrpc
}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCREQUEST_H_
