﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_
#define INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/Swap.h"
#include "nn/nlib/Nlist.h"
#include "nn/nlib/ReallocOutputStream.h"
#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/msgpack/MpObject.h"
#include "nn/nlib/msgpack/MpWalker.h"

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace msgpack {
namespace jsonrpc {
typedef uint32_t reqid_t;

class JsonRpcResponse NLIB_FINAL {
 public:
    enum ErrorCode {
        OK = 0,
        PARSE_ERROR = -32700,
        INVALID_REQUEST = -32600,
        METHOD_NOT_FOUND = -32601,
        INVALID_PARAMS = -32602,
        INTERNAL_ERROR = -32603,
        CLIENT_ABORT = -31000
    };
    JsonRpcResponse() : m_Id(0), m_IsError(false), m_ErrorCode(0) {}

    NLIB_VIS_PUBLIC void MoveResultFrom(MpObject& obj);           // NOLINT
    NLIB_VIS_PUBLIC void SetError(int errcode, const char* msg, MpObject& data);  // NOLINT
    void SetError(int errcode, const char* msg) {
        MpObject data;
        this->SetError(errcode, msg, data);
    }

    bool IsError() const { return m_IsError; }
    int GetErrorCode() const { return m_ErrorCode; }
    reqid_t GetId() const { return m_Id; }
    const char* GetErrorMessage() const {
        const char* msg = m_ErrorMsg.get();
        return msg ? msg : "";
    }
    MpObject& GetMpObject() { return m_Obj; }
    const MpObject& GetMpObject() const { return m_Obj; }

    NLIB_SAFE_BOOL(JsonRpcResponse, !IsError());

 private:
    void SetId(reqid_t id) { m_Id = id; }  // from JsonRpcRequest::Init()
    NLIB_VIS_HIDDEN errno_t Init(MpObject* rhs);           // from JsonRpcResponseReader
    NLIB_VIS_HIDDEN errno_t Init(MpWalker* walker);        // from JsonRpcResponseReader

 private:
    reqid_t m_Id;
    bool m_IsError;
    int m_ErrorCode;
    UniquePtr<char[]> m_ErrorMsg;
    MpObject m_Obj;

    friend class JsonRpcResponseReader;
    friend class JsonRpcRequest;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcResponse);
};

class NLIB_VIS_PUBLIC JsonRpcResponseReader {
 public:
    typedef UniquePtr<JsonRpcResponse> ValueType;
    typedef Nlist<ValueType> ListType;
    static errno_t ReadResponse(const void* p, size_t n, ListType* result_list);

 private:
    NLIB_VIS_HIDDEN static errno_t ReadJsonResponse(const char* p, size_t n,
                                                    ListType* result_list);
    NLIB_VIS_HIDDEN static errno_t ReadMsgpackResponse(const uint8_t* p, size_t n,
                                                       ListType* result_list);
};

class JsonRpcResponseWriter NLIB_FINAL {
 public:
    JsonRpcResponseWriter() : m_IsMsgpack(false), m_Cur(0), m_Count(0) {}
    explicit JsonRpcResponseWriter(bool msgpack) : m_IsMsgpack(msgpack), m_Cur(0), m_Count(0) {}
    NLIB_VIS_PUBLIC errno_t BeginWriteResponse(uint32_t n);
    NLIB_VIS_PUBLIC errno_t WriteResponse(JsonRpcResponse* result);
    NLIB_VIS_PUBLIC errno_t EndWriteResponse(ReallocOutputStream::UniquePtrType* ptr,
                                             size_t* n);

 private:
    NLIB_VIS_HIDDEN errno_t WriteJsonResult(JsonRpcResponse* result);
    NLIB_VIS_HIDDEN errno_t WriteMsgpackResult(JsonRpcResponse* result);

 private:
    bool m_IsMsgpack;
    size_t m_Cur;
    size_t m_Count;
    ReallocOutputStream m_Ostr;
    NLIB_DISALLOW_COPY_AND_ASSIGN(JsonRpcResponseWriter);
};

}  // namespace jsonrpc
}  // namespace msgpack
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_msgpack_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_MSGPACK_JSONRPC_JSONRPCRESPONSE_H_
