﻿
#pragma once
#ifndef INCLUDE_NN_NLIB_OSS_LZ4OUTPUTSTREAM_H_
#define INCLUDE_NN_NLIB_OSS_LZ4OUTPUTSTREAM_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/InputStream.h"
#include "nn/nlib/InputConverterStream.h"

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace oss {

class Lz4InputStreamSettings {
 public:
    NLIB_CEXPR Lz4InputStreamSettings() NLIB_NOEXCEPT {}
};

namespace detail {

class NLIB_VIS_PUBLIC Lz4InputTransform NLIB_FINAL : public InputTransform {
 public:
    Lz4InputTransform() NLIB_NOEXCEPT;
    virtual ~Lz4InputTransform() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init(const Lz4InputStreamSettings& settings) NLIB_NOEXCEPT;
    virtual errno_t Transform(InputStream* is, void* p, size_t nBytes,
                              size_t* written) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(InputStream* is) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose() NLIB_NOEXCEPT NLIB_OVERRIDE;

 private:
    LZ4F_decompressionContext_t m_Ctx;
    unsigned char* m_BufIn;
    size_t m_BufInCur;
    size_t m_BufInDataSize;
    unsigned char m_BaseBuf[256];

    NLIB_DISALLOW_COPY_AND_ASSIGN(Lz4InputTransform);
};

}  // namespace detail

class NLIB_VIS_PUBLIC Lz4InputStream NLIB_FINAL
    : public InputConverterStreamTempl<detail::Lz4InputTransform> {
 public:
     Lz4InputStream() NLIB_NOEXCEPT;
     virtual ~Lz4InputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
     errno_t Init() NLIB_NOEXCEPT {
         NLIB_CEXPR Lz4InputStreamSettings settings;
         return this->Init(settings);
     }
     errno_t Init(const Lz4InputStreamSettings& settings) NLIB_NOEXCEPT {
         return m_Transform.Init(settings);
     }

 private:
     NLIB_DISALLOW_COPY_AND_ASSIGN(Lz4InputStream);
};

}  // namespace oss
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_OSS_LZ4OUTPUTSTREAM_H_
