﻿
#pragma once
#ifndef INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_
#define INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/OutputStream.h"
#include "nn/nlib/OutputConverterStream.h"

#include "lz4frame.h"

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace oss {

class Lz4OutputStreamSettings;

namespace detail {

class NLIB_VIS_PUBLIC Lz4OutputTransform NLIB_FINAL : public OutputTransform {
 public:
    Lz4OutputTransform() NLIB_NOEXCEPT;
    virtual ~Lz4OutputTransform() NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t Transform(OutputStream* os, const void* p, size_t nBytes,
                              bool doFlush) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual void* GetWorkBuffer(size_t* n) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnSetStream(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;
    virtual errno_t OnClose(OutputStream* os) NLIB_NOEXCEPT NLIB_OVERRIDE;

    errno_t Init(const Lz4OutputStreamSettings& settings) NLIB_NOEXCEPT;

 private:
    LZ4F_compressionContext_t m_Ctx;
    unsigned char* m_BufOut;
    size_t m_BufOutSize;
    size_t m_BlockSize;
    size_t m_HeaderSize;
    unsigned char m_Header[16];  // LZ4F_MAXHEADERFRAME_SIZE <= sizeof(m_Header)
    unsigned char m_BaseBuf[256];
    NLIB_DISALLOW_COPY_AND_ASSIGN(Lz4OutputTransform);
};

}  // namespace detail

class Lz4OutputStreamSettings {
 public:
    Lz4OutputStreamSettings() NLIB_NOEXCEPT {
        nlib_memset(&m_Pref, 0, sizeof(m_Pref));
        m_Pref.autoFlush = 1;
        m_Pref.frameInfo.blockMode = LZ4F_blockLinked;
        m_Pref.compressionLevel = 9;  // hc
    }
    ~Lz4OutputStreamSettings() NLIB_NOEXCEPT {}
    void SetContentSize(uint64_t size) NLIB_NOEXCEPT {
        m_Pref.frameInfo.contentSize = size;
    }
    uint64_t GetContentSize() const NLIB_NOEXCEPT {
        return m_Pref.frameInfo.contentSize;
    }
    void SetCompressionLevel(int level) NLIB_NOEXCEPT {
        if (level < 0)
            level = 0;
        else if (level > 16)
            level = 16;
        m_Pref.compressionLevel = level;
    }
    int GetCompressionLevel() NLIB_NOEXCEPT {
        return m_Pref.compressionLevel;
    }
    void SetMaxBlockSize(LZ4F_blockSizeID_t max_block_size) NLIB_NOEXCEPT {
        m_Pref.frameInfo.blockSizeID = max_block_size;
    }
    LZ4F_blockSizeID_t GetMaxBlockSize() const NLIB_NOEXCEPT {
        return m_Pref.frameInfo.blockSizeID;
    }
    void SetBlockIndependence(bool independence) NLIB_NOEXCEPT {
        m_Pref.frameInfo.blockMode = independence ? LZ4F_blockIndependent : LZ4F_blockLinked;
    }
    bool GetBlockIndependence() const NLIB_NOEXCEPT {
        return (m_Pref.frameInfo.blockMode != LZ4F_blockLinked);
    }

 private:
    LZ4F_preferences_t m_Pref;
    friend class detail::Lz4OutputTransform;
};

class NLIB_VIS_PUBLIC Lz4OutputStream NLIB_FINAL
    : public OutputConverterStreamTempl<detail::Lz4OutputTransform> {
 public:
    Lz4OutputStream() NLIB_NOEXCEPT;
    virtual ~Lz4OutputStream() NLIB_NOEXCEPT NLIB_OVERRIDE;
    errno_t Init() NLIB_NOEXCEPT {
        Lz4OutputStreamSettings settings;
        return this->Init(settings);
    }
    errno_t Init(const Lz4OutputStreamSettings& settings) NLIB_NOEXCEPT {
        return m_Transform.Init(settings);
    }

 private:
    NLIB_DISALLOW_COPY_AND_ASSIGN(Lz4OutputStream);
};

}  // namespace oss
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_oss_lz4_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_OSS_LZ4INPUTSTREAM_H_
