﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMD_SIMDALGORITHM_H_
#define INCLUDE_NN_NLIB_SIMD_SIMDALGORITHM_H_

#include "nn/nlib/simd/SimdInt.h"

#if defined(NLIB_SIMD)

NLIB_NAMESPACE_BEGIN
namespace simd {

namespace detail {
NLIB_VIS_PUBLIC void sortUint32A16(uint32_t* buf, uint32_t* src, size_t N);
NLIB_VIS_PUBLIC void sortUint32A16_1(uint32_t* buf, uint32_t* src, size_t N);

template <size_t NumElem, bool flag>
struct MergeSortHelper {
    static NLIB_ALWAYS_INLINE void Sort(uint32_t* data) {
        NLIB_ALIGNAS(16) uint32_t tmp[NumElem];
        sortUint32A16(tmp, data, NumElem);
    }
};

template <size_t NumElem>
struct MergeSortHelper<NumElem, false> {
    static NLIB_ALWAYS_INLINE void Sort(uint32_t* data) {
        NLIB_ALIGNAS(16) uint32_t tmp[NumElem];
        sortUint32A16_1(tmp, data, NumElem);
    }
};

}  // namespace detail

template <size_t NumElem>
// data must be 16 bytes aligned, NumElem must be multiple of 16
NLIB_ALWAYS_INLINE void MergeSortUint32A16(uint32_t* data) {
    NLIB_STATIC_ASSERT((NumElem > 0) && (NumElem % 16 == 0));
    NLIB_ASSERT(!(reinterpret_cast<uintptr_t>(data) & 15));
    detail::MergeSortHelper<NumElem, ((NumElem & (NumElem - 1)) == 0)>::Sort(data);
}

template <class PRED>
// i128 PRED(i128 c) { mask(c.u8[i]) if matched }
const void* nlib_memchr_pred(const void* s, PRED pred, size_t n) {
    if (!s) return NULL;

    const unsigned char* p = reinterpret_cast<const unsigned char*>(s);
    i128 a1, a2;
    i128 cmp1, cmp2;
    uint32_t mask;
    if (reinterpret_cast<uintptr_t>(p) & 15) {
        size_t r = reinterpret_cast<uintptr_t>(p) & 15;
        a1 = I128::LoadA16(p - r);
        cmp1 = pred(a1);
        mask = I128::MoveMask8(cmp1);
        mask = mask >> r;
        size_t rr = 16 - r;
        if (n < rr) {
            mask &= (1 << n) - 1;
            if (mask) return p + nlib_ctz(mask);
            return NULL;
        }
        if (mask) return p + nlib_ctz(mask);
        p += rr;
        n -= rr;
    }
    if (n >= 16) {
        if ((reinterpret_cast<uintptr_t>(p) & 32)) {
            a1 = I128::LoadA16(p);
            cmp1 = pred(a1);
            if (!I128::IsZero(cmp1)) {
                mask = I128::MoveMask8(cmp1);
                return p + nlib_ctz(mask);
            }
            p += 16;
            n -= 16;
        }
        while (n >= 32) {
            a1 = I128::LoadA16(p);
            a2 = I128::LoadA16(p + 16);
            cmp1 = I128::SetZero();
            cmp2 = I128::SetZero();
            cmp1 = pred(a1);
            cmp2 = pred(a2);
            if (!I128::IsZero(I128::Or(cmp1, cmp2))) {
                mask = I128::MoveMask8(cmp1) | (I128::MoveMask8(cmp2) << 16);
                return p + nlib_ctz(mask);
            }
            p += 32;
            n -= 32;
        }
        if (n >= 16) {
            a1 = I128::LoadA16(p);
            cmp1 = pred(a1);
            if (!I128::IsZero(cmp1)) {
                mask = I128::MoveMask8(cmp1);
                return p + nlib_ctz(mask);
            }
            p += 16;
            n -= 16;
        }
    }
    if (n > 0) {
        a1 = I128::LoadA16(p);
        cmp1 = pred(a1);
        mask = I128::MoveMask8(cmp1);
        mask &= (1 << n) - 1;
        if (mask) return p + nlib_ctz(mask);
    }
    return NULL;
}

template <class PRED>
// i128 PRED(i128 c) { mask(c.u8[i]) if matched }
const void* nlib_memchr_pred_not(const void* s, PRED pred, size_t n) {
    if (!s) return NULL;

    const unsigned char* p = reinterpret_cast<const unsigned char*>(s);
    i128 a1, a2;
    i128 cmp1, cmp2;
    uint32_t mask;
    if (reinterpret_cast<uintptr_t>(p) & 15) {
        size_t r = reinterpret_cast<uintptr_t>(p) & 15;
        a1 = I128::LoadA16(p - r);
        cmp1 = I128::Not(pred(a1));
        mask = I128::MoveMask8(cmp1);
        mask = mask >> r;
        size_t rr = 16 - r;
        if (n < rr) {
            mask &= (1 << n) - 1;
            if (mask) return p + nlib_ctz(mask);
            return NULL;
        }
        if (mask) return p + nlib_ctz(mask);
        p += rr;
        n -= rr;
    }
    if (n >= 16) {
        if ((reinterpret_cast<uintptr_t>(p) & 32)) {
            a1 = I128::LoadA16(p);
            cmp1 = pred(a1);
            if (!I128::IsFull(cmp1)) {
                mask = I128::MoveMask8(I128::Not(cmp1));
                return p + nlib_ctz(mask);
            }
            p += 16;
            n -= 16;
        }
        while (n >= 32) {
            a1 = I128::LoadA16(p);
            a2 = I128::LoadA16(p + 16);
            cmp1 = pred(a1);
            cmp2 = pred(a2);
            if (!I128::IsFull(I128::And(cmp1, cmp2))) {
                mask = I128::MoveMask8(I128::Not(cmp1)) | (I128::MoveMask8(I128::Not(cmp2)) << 16);
                return p + nlib_ctz(mask);
            }
            p += 32;
            n -= 32;
        }
        if (n >= 16) {
            a1 = I128::LoadA16(p);
            cmp1 = pred(a1);
            if (!I128::IsFull(cmp1)) {
                mask = I128::MoveMask8(I128::Not(cmp1));
                return p + nlib_ctz(mask);
            }
            p += 16;
            n -= 16;
        }
    }
    if (n > 0) {
        a1 = I128::LoadA16(p);
        cmp1 = I128::Not(pred(a1));
        mask = I128::MoveMask8(cmp1);
        mask &= (1 << n) - 1;
        if (mask) return p + nlib_ctz(mask);
    }
    return NULL;
}

// mask if c.u8[i] is a-zA-Z
inline i128 __vectorcall IsAlpha(i128 c) NLIB_NOEXCEPT {
    i128 result = I128::CmpLtInt8(c, I128::SetValue('{', each_int8));
    result = I128::And(result, I128::CmpGtInt8(c, I128::SetValue('`', each_int8)));

    i128 tmp = I128::CmpLtInt8(c, I128::SetValue('[', each_int8));
    tmp = I128::And(tmp, I128::CmpGtInt8(c, I128::SetValue('@', each_int8)));
    result = I128::Or(result, tmp);

    return result;
}

// mask if c.u8[i] is 0-9
inline i128 __vectorcall IsDigit(i128 c) NLIB_NOEXCEPT {
    i128 result = I128::CmpLtInt8(c, I128::SetValue(':', each_int8));
    result = I128::And(result, I128::CmpGtInt8(c, I128::SetValue('/', each_int8)));

    return result;
}

// mask if c.u8[i] is a-zA-Z0-9
inline i128 __vectorcall IsAlnum(i128 c) NLIB_NOEXCEPT { return I128::Or(IsDigit(c), IsAlpha(c)); }

// mask if c.u8[i] is space, CR, LF, or tab
inline i128 __vectorcall IsSpace(i128 c) NLIB_NOEXCEPT {
    i128 result = I128::CmpEq8(c, I128::SetValue(' ', each_int8));
    result = I128::Or(result, I128::CmpEq8(c, I128::SetValue('\r', each_int8)));
    result = I128::Or(result, I128::CmpEq8(c, I128::SetValue('\n', each_int8)));
    result = I128::Or(result, I128::CmpEq8(c, I128::SetValue('\t', each_int8)));

    return result;
}

// mask if c.u8[i] is 0-9A-Fa-f
inline i128 __vectorcall IsXdigit(i128 c) NLIB_NOEXCEPT {
    i128 tmp;
    i128 result = I128::CmpLtInt8(c, I128::SetValue(':', each_int8));
    result = I128::And(result, I128::CmpGtInt8(c, I128::SetValue('/', each_int8)));

    tmp = I128::CmpLtInt8(c, I128::SetValue('G', each_int8));
    tmp = I128::And(tmp, I128::CmpGtInt8(c, I128::SetValue('@', each_int8)));
    result = I128::Or(result, tmp);

    tmp = I128::CmpLtInt8(c, I128::SetValue('g', each_int8));
    tmp = I128::And(tmp, I128::CmpGtInt8(c, I128::SetValue('`', each_int8)));
    result = I128::Or(result, tmp);

    return result;
}

}  // namespace simd
NLIB_NAMESPACE_END

#endif

#endif  // INCLUDE_NN_NLIB_SIMD_SIMDALGORITHM_H_
