﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMD_SIMDVECTOR3_H_
#define INCLUDE_NN_NLIB_SIMD_SIMDVECTOR3_H_

#include "nn/nlib/simd/SimdFloat.h"
#include "nn/nlib/simd/SimdVector4.h"
#include "nn/nlib/simd/SimdQuaternion.h"

NLIB_NAMESPACE_BEGIN
namespace simd {

class NLIB_VIS_HIDDEN Vector3 {
 public:
    static SimdVector __vectorcall LoadFloat3(const Float3* p) NLIB_NOEXCEPT;
    template<typename MyVector3>
    static SimdVector __vectorcall LoadFloat3(const MyVector3* p) NLIB_NOEXCEPT;
    static void __vectorcall StoreFloat3(Float3* p, SimdVectorArg vec) NLIB_NOEXCEPT;
    template<typename MyVector3>
    static void __vectorcall StoreFloat3(MyVector3* p, SimdVectorArg vec) NLIB_NOEXCEPT;
    static bool __vectorcall CmpEq(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall CmpLt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall CmpLe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall CmpGt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall CmpGe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall CmpNe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static bool __vectorcall
        CmpNearEq(SimdVectorArg vec1, SimdVectorArg vec2, SimdVectorArg eps) NLIB_NOEXCEPT;
    static bool __vectorcall IsNaN(SimdVectorArg vec) NLIB_NOEXCEPT;
    static bool __vectorcall IsInfinite(SimdVectorArg vec) NLIB_NOEXCEPT;
    static bool __vectorcall InBound(SimdVectorArg vec, SimdVectorArg bounds) NLIB_NOEXCEPT;
    static f128 __vectorcall Dot(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    template <bool SetLane0, bool SetLane1, bool SetLane2, bool SetLane3>
    static f128 __vectorcall DotEx(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Cross(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Normalize(SimdVectorArg vec) NLIB_NOEXCEPT;
    static SimdVector __vectorcall NormalizeEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall LengthSq(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall Length(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall LengthEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpLength(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall RecpLengthEst(SimdVectorArg vec) NLIB_NOEXCEPT;
    static f128 __vectorcall GetAngle(SimdVectorArg vec1_normalized,
                                      SimdVectorArg vec2_normalized) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Reflect(SimdVectorArg vec, SimdVectorArg normal) NLIB_NOEXCEPT;
    static SimdVector __vectorcall Orthogonal(SimdVector vec) NLIB_NOEXCEPT;  // not implemented yet

    static SimdVector __vectorcall Transform(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT;
    static SimdVector __vectorcall
        TransformCoord(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT;
    static SimdVector __vectorcall
        TransformNormal(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT;

    static SimdVector __vectorcall
        Rotate(SimdVectorArg vec, SimdQuaternionArg q_normalized) NLIB_NOEXCEPT;
    static SimdVector __vectorcall
        InvRotate(SimdVectorArg vec, SimdQuaternionArg q_normalized) NLIB_NOEXCEPT;

    // Transform****Stream
    // Project, Unproject

 private:
    Vector3();  // forbidden
};

#ifndef NLIB_DOXYGEN

#define NLIB_M(tp) inline tp __vectorcall
#define NLIB_MH(tp) inline tp __vectorcall

// p is 4 bytes aligned
NLIB_M(SimdVector) Vector3::LoadFloat3(const Float3* p) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    f128 ret;
    ret.vec.v[0] = p->x;
    ret.vec.v[1] = p->y;
    ret.vec.v[2] = p->z;
    ret.vec.v[3] = 0.f;
    return ret;
#elif defined(NLIB_SSE41)
    __m128 x = _mm_load_ss(&p->x);
    __m128 y = _mm_load_ss(&p->y);
    __m128 z = _mm_load_ss(&p->z);
    __m128 xy = _mm_unpacklo_ps(x, y);
    return _mm_shuffle_ps(xy, z, _MM_SHUFFLE(1, 0, 1, 0));
#elif defined(NLIB_NEON)
    float32x2_t xy = vld1_f32(&p->x);
    float32x2_t z = vld1_lane_f32(&p->z, xy, 0);
    return vcombine_f32(xy, z);
#elif defined(NLIB_CAFE_PPC)
    f128 ret;
    ret.vec.ps[0][0] = p->x;
    ret.vec.ps[0][1] = p->y;
    ret.vec.ps[1][0] = p->z;
    return ret;
#endif
}

template<typename MyVector3>
// MyVector is a structure like 'struct MyVector3 { float x, y, z; };'
// DirectX::XMFLOAT3, DirectX::XMFLOAT3A for example
NLIB_MH(SimdVector) Vector3::LoadFloat3(const MyVector3* p) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(sizeof(p->x) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->y) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->z) == 4);
    return Vector3::LoadFloat3(reinterpret_cast<const Float3*>(&p->x));
}

// p is 4 bytes aligned
NLIB_M(void) Vector3::StoreFloat3(Float3* p, SimdVectorArg vec) NLIB_NOEXCEPT {  // NOLINT
#ifdef NLIB_F128_SIMD_NOUSE
    p->x = vec.vec.v[0];
    p->y = vec.vec.v[1];
    p->z = vec.vec.v[2];
#elif defined(NLIB_SSE41)
    f128 y = _mm_shuffle_ps(vec, vec, _MM_SHUFFLE(1, 1, 1, 1));
    f128 z = _mm_shuffle_ps(vec, vec, _MM_SHUFFLE(2, 2, 2, 2));
    _mm_store_ss(&p->x, vec);
    _mm_store_ss(&p->y, y);
    _mm_store_ss(&p->z, z);
#elif defined(NLIB_NEON)
    float32x2_t lo = vget_low_f32(vec);
    vst1_f32(&p->x, lo);
    vst1q_lane_f32(&p->z, vec, 2);
#elif defined(NLIB_CAFE_PPC)
    p->x = vec.vec.ps[0][0];
    p->y = vec.vec.ps[0][1];
    p->z = vec.vec.ps[1][0];
#endif
}

template<typename MyVector3>
NLIB_MH(void) Vector3::StoreFloat3(MyVector3* p, SimdVectorArg vec) NLIB_NOEXCEPT {  // NOLINT
    // MyVector is a structure like 'struct MyVector3 { float x, y, z; };'
    // DirectX::XMFLOAT3, DirectX::XMFLOAT3A for example
    NLIB_STATIC_ASSERT(sizeof(p->x) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->y) == 4);
    NLIB_STATIC_ASSERT(sizeof(p->z) == 4);
    Vector3::StoreFloat3(reinterpret_cast<Float3*>(&p->x), vec);
}

#if defined(NLIB_NEON)
#define NLIB_NEON_CMPVector3(op, vec1, vec2)                        \
    uint8x16_t cmp = vreinterpretq_u8_u32(op(vec1, vec2));          \
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp)); \
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),   \
                                 vreinterpret_u16_u8(zip.val[1]));  \
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) == 0xFFFFFFU
#endif

// true if vec1[xyz] == vec2[xyz]
inline bool __vectorcall Vector3::CmpEq(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpEq(vec1, vec2));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    NLIB_NEON_CMPVector3(vceqq_f32, vec1, vec2);
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] == vec2.vec.ps[0][0] &&
           vec1.vec.ps[0][1] == vec2.vec.ps[0][1] &&
           vec1.vec.ps[1][0] == vec2.vec.ps[1][0];
#endif
}

// true if vec1[xyz] < vec2[xyz]
inline bool __vectorcall Vector3::CmpLt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpLt(vec1, vec2));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    NLIB_NEON_CMPVector3(vcltq_f32, vec1, vec2);
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] < vec2.vec.ps[0][0] &&
           vec1.vec.ps[0][1] < vec2.vec.ps[0][1] &&
           vec1.vec.ps[1][0] < vec2.vec.ps[1][0];
#endif
}

// true if vec1[xyz] <= vec2[xyz]
inline bool __vectorcall Vector3::CmpLe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpLe(vec1, vec2));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    NLIB_NEON_CMPVector3(vcleq_f32, vec1, vec2);
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] <= vec2.vec.ps[0][0] &&
           vec1.vec.ps[0][1] <= vec2.vec.ps[0][1] &&
           vec1.vec.ps[1][0] <= vec2.vec.ps[1][0];
#endif
}

// true if vec1[xyz] > vec2[xyz]
inline bool __vectorcall Vector3::CmpGt(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpGt(vec1, vec2));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    NLIB_NEON_CMPVector3(vcgtq_f32, vec1, vec2);
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] > vec2.vec.ps[0][0] &&
           vec1.vec.ps[0][1] > vec2.vec.ps[0][1] &&
           vec1.vec.ps[1][0] > vec2.vec.ps[1][0];
#endif
}

// true if vec1[xyz] >= vec2[xyz]
inline bool __vectorcall Vector3::CmpGe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpGe(vec1, vec2));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    NLIB_NEON_CMPVector3(vcgeq_f32, vec1, vec2);
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] >= vec2.vec.ps[0][0] &&
           vec1.vec.ps[0][1] >= vec2.vec.ps[0][1] &&
           vec1.vec.ps[1][0] >= vec2.vec.ps[1][0];
#endif
}

// vec1.x != vec2.x || vec1.y != vec2.y || vec1.z != vec2.z
inline bool __vectorcall Vector3::CmpNe(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41)
    int mask = F128::MoveMask(F128::CmpEq(vec1, vec2));
    return ((mask & 7) != 7);
#elif defined(NLIB_NEON)
    uint8x16_t cmp = vreinterpretq_u8_u32(vceqq_f32(vec1, vec2));
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp));
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),
                                 vreinterpret_u16_u8(zip.val[1]));
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) != 0xFFFFFFU;
#elif defined(NLIB_CAFE_PPC)
    return vec1.vec.ps[0][0] != vec2.vec.ps[0][0] ||
           vec1.vec.ps[0][1] != vec2.vec.ps[0][1] ||
           vec1.vec.ps[1][0] != vec2.vec.ps[1][0];
#endif
}

inline bool __vectorcall
Vector3::CmpNearEq(SimdVectorArg vec1, SimdVectorArg vec2, SimdVectorArg eps) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41) || defined(NLIB_CAFE_PPC)
    int mask = F128::MoveMask(F128::CmpNearEq(vec1, vec2, eps));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    f128 cmp_ = F128::CmpNearEq(vec1, vec2, eps);
    uint8x16_t cmp = vreinterpretq_u8_f32(cmp_);
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp));
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),
                                 vreinterpret_u16_u8(zip.val[1]));
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) == 0xFFFFFFU;
#endif
}

#if defined(NLIB_NEON)
#undef NLIB_NEON_CMPVector3
#endif

// isnan(vec.x) || isnan(vec.y) || isnan(vec.z)
inline bool __vectorcall Vector3::IsNaN(SimdVectorArg vec) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41) || defined(NLIB_CAFE_PPC)
    int mask = F128::MoveMask(F128::IsNaN(vec));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    f128 cmp_ = F128::IsNaN(vec);
    uint8x16_t cmp = vreinterpretq_u8_f32(cmp_);
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp));
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),
                                 vreinterpret_u16_u8(zip.val[1]));
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) == 0xFFFFFFU;
#endif
}

// isinf(vec.x) || isinf(vec.y) || isinf(vec.z)
inline bool __vectorcall Vector3::IsInfinite(SimdVectorArg vec) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41) || defined(NLIB_CAFE_PPC)
    int mask = F128::MoveMask(F128::IsInfinite(vec));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    f128 cmp_ = F128::IsInfinite(vec);
    uint8x16_t cmp = vreinterpretq_u8_f32(cmp_);
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp));
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),
                                 vreinterpret_u16_u8(zip.val[1]));
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) == 0xFFFFFFU;
#endif
}

// true if abs(vec[xyz]) <= bounds[xyz]
inline bool __vectorcall Vector3::InBound(SimdVectorArg vec, SimdVectorArg bounds) NLIB_NOEXCEPT {
#if defined(NLIB_F128_SIMD_NOUSE) || defined(NLIB_SSE41) || defined(NLIB_CAFE_PPC)
    int mask = F128::MoveMask(F128::InBound(vec, bounds));
    return ((mask & 7) == 7);
#elif defined(NLIB_NEON)
    f128 cmp_ = F128::InBound(vec, bounds);
    uint8x16_t cmp = vreinterpretq_u8_f32(cmp_);
    uint8x8x2_t zip = vzip_u8(vget_low_u8(cmp), vget_high_u8(cmp));
    uint16x4x2_t zip2 = vzip_u16(vreinterpret_u16_u8(zip.val[0]),
                                 vreinterpret_u16_u8(zip.val[1]));
    return (vget_lane_u32(vreinterpret_u32_u16(zip2.val[0]), 0) & 0xFFFFFFU) == 0xFFFFFFU;
#endif
}

// r = { dot, dot, dot, dot }
NLIB_M(f128) Vector3::Dot(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#ifdef NLIB_F128_SIMD_NOUSE
    float tmp = vec1.vec.v[0] * vec2.vec.v[0] + vec1.vec.v[1] * vec2.vec.v[1] +
                vec1.vec.v[2] * vec2.vec.v[2];
    f128 rval;
    rval.vec.v[0] = rval.vec.v[1] = rval.vec.v[2] = rval.vec.v[3] = tmp;
    return rval;
#elif defined(NLIB_SSE41)
    return _mm_dp_ps(vec1, vec2, 0x7F);
#elif defined(NLIB_NEON)
    f128 tmp = vmulq_f32(vec1, vec2);
    float32x2_t lo = vget_low_f32(tmp);
    lo = vpadd_f32(lo, lo);
    float32x2_t hi = vdup_lane_f32(vget_high_f32(tmp), 0);
    lo = vadd_f32(lo, hi);
    return vcombine_f32(lo, lo);
#elif defined(NLIB_CAFE_PPC)
    f128 tmp = F128::Mult(vec1, vec2);
    f32x2 val = __PS_SUM0(tmp.vec.ps[0], tmp.vec.ps[0], tmp.vec.ps[0]);
    val = __PS_ADD(val, tmp.vec.ps[1]);
    f128 ret;
    ret.vec.ps[0] = ret.vec.ps[1] = __PS_FDUP(val[0]);
    return ret;
#endif
}

template <bool SetLane0, bool SetLane1, bool SetLane2, bool SetLane3>
// r[i] = SetLane[i] ? dot : 0.f
NLIB_MH(f128) Vector3::DotEx(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
#if !defined(NLIB_F128_SIMD_NOUSE) && defined(NLIB_SSE41)
    return _mm_dp_ps(vec1, vec2,
                     (0x70 | (SetLane0 ? 1 : 0) | (SetLane1 ? 2 : 0) |
                             (SetLane2 ? 4 : 0) | (SetLane3 ? 8 : 0)));
#else
    return F128::Splat<SetLane0, SetLane1, SetLane2, SetLane3>(F128::SetZero(), Dot(vec1, vec2));
#endif
}

// 'w' component is arbitrary
NLIB_M(SimdVector) Vector3::Cross(SimdVectorArg vec1, SimdVectorArg vec2) NLIB_NOEXCEPT {
    SimdVector rval = F128::Mult(F128::Swizzle<1, 2, 0, 1>(vec1), F128::Swizzle<2, 0, 1, 1>(vec2));
    // yz, zx, xy, *
    rval = F128::MultSub(F128::Swizzle<2, 0, 1, 1>(vec1), F128::Swizzle<1, 2, 0, 1>(vec2), rval);
    // yz - zy, zx - xz, xy - yx, *
    return rval;
    // NOTE:
    // optimization for NEON may be needed.
}

// nan if |vec| is too big, 0 if |vec| = 0
NLIB_M(SimdVector) Vector3::Normalize(SimdVectorArg vec) NLIB_NOEXCEPT {
    f128 dot = Vector3::Dot(vec, vec);
    f128 rsqrt = F128::RecpSqrt(dot);
    f128 inf = F128::SetInfinity();
    f128 zero = F128::SetZero();
    f128 eqzero = F128::CmpEq(dot, zero);
    f128 eqinf = F128::CmpEq(dot, inf);
    SimdVector ret = F128::Mult(vec, rsqrt);
    f128 nan = F128::SetNaN();
    ret = F128::Select(eqzero, zero, ret);
    ret = F128::Select(eqinf, nan, ret);
    return ret;
}

// r = { lensq, lensq, lensq, lensq }
NLIB_M(f128) Vector3::LengthSq(SimdVectorArg vec) NLIB_NOEXCEPT { return Dot(vec, vec); }

// r = { len, len, len, len }
NLIB_M(f128) Vector3::Length(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::Sqrt(Dot(vec, vec));
}

// r = { len, len, len, len }
NLIB_M(f128) Vector3::LengthEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::SqrtEst(Dot(vec, vec));
}

// r = { recpLen, recpLen, recpLen, recpLen }
NLIB_M(f128) Vector3::RecpLength(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::RecpSqrt(Dot(vec, vec));
}

// r = { recpLen, recpLen, recpLen, recpLen }
NLIB_M(f128) Vector3::RecpLengthEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::RecpSqrtEst(Dot(vec, vec));
}

NLIB_M(SimdVector) Vector3::NormalizeEst(SimdVectorArg vec) NLIB_NOEXCEPT {
    return F128::Mult(vec, RecpLengthEst(vec));
}

// { radian, radian, radian, radian }
NLIB_M(f128) Vector3::GetAngle(SimdVectorArg vec1_normalized,
                               SimdVectorArg vec2_normalized) NLIB_NOEXCEPT {
    f128 ret = Dot(vec1_normalized, vec2_normalized);
    ret = F128::Clamp(ret, F128::SetNegativeOne(), F128::SetOne());
    return F128::ArcCos(ret);
}

NLIB_M(SimdVector) Vector3::Reflect(SimdVectorArg vec, SimdVectorArg normal) NLIB_NOEXCEPT {
    f128 s = Dot(vec, normal);
    s = F128::Add(s, s);
    return F128::MultSub(s, normal, vec);
}

/*
// Vector3::Transform(vec, m) is better in performance
NLIB_M(SimdVector) Vector3::Transform(SimdMatrixArg m, SimdVectorArg vec) NLIB_NOEXCEPT {
    f128 v = F128::SetFloatToLane<3>(vec, 1.f);
    return Vector4::Transform(m, v);
}
*/

// r = { vec.x, vec.y, vec.z, 1 } * m
NLIB_M(SimdVector) Vector3::Transform(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT {
    f128 ret = F128::Mult<0>(vec, m.r[0], each_select32);
    ret = F128::MultAdd<1>(vec, m.r[1], ret, each_select32);
    ret = F128::MultAdd<2>(vec, m.r[2], ret, each_select32);
    return F128::Add(m.r[3], ret);
}

// r = { vec.x, vec.y, vec.z, 1.f } * m, r = r / r.w
NLIB_M(SimdVector) Vector3::TransformCoord(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT {
    f128 tmp = Vector3::Transform(vec, m);
    return F128::Div(tmp, F128::SetValue<3>(tmp, each_select32));
}

/*
// Vector3::TransformNormal(vec, m) is better in performance
NLIB_M(SimdVector) Vector3::TransformNormal(SimdMatrixArg m, SimdVectorArg vec) NLIB_NOEXCEPT {
    f128 v = F128::SetZeroToLane<3>(vec);
    return Vector4::Transform(m, v);
}
*/

// r = { vec.x, vec.y, vec.z, 0 } * m
NLIB_M(SimdVector) Vector3::TransformNormal(SimdVectorArg vec, SimdMatrixArg m) NLIB_NOEXCEPT {
    f128 ret = F128::Mult<0>(vec, m.r[0], each_select32);
    ret = F128::MultAdd<1>(vec, m.r[1], ret, each_select32);
    ret = F128::MultAdd<2>(vec, m.r[2], ret, each_select32);
    return ret;
}

NLIB_M(SimdVector) Vector3::Rotate(SimdVectorArg vec,
                                   SimdQuaternionArg q_normalized) NLIB_NOEXCEPT {
    SimdVector v = F128::SetZeroToLane<3>(vec);
    SimdQuaternion conj = Quaternion::Conjugate(q_normalized);
    return Quaternion::Mult(Quaternion::Mult(conj, v), q_normalized);
}

NLIB_M(SimdVector) Vector3::InvRotate(SimdVectorArg vec,
                                      SimdQuaternionArg q_normalized) NLIB_NOEXCEPT {
    SimdVector v = F128::SetZeroToLane<3>(vec);
    SimdQuaternion conj = Quaternion::Conjugate(q_normalized);
    return Quaternion::Mult(Quaternion::Mult(q_normalized, v), conj);
}

#undef NLIB_M
#undef NLIB_MH

#endif  // NLIB_DOXYGEN

}  // namespace simd
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SIMD_SIMDVECTOR3_H_
