﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_
#define INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_

#include "nn/nlib/Swap.h"
#include "nn/nlib/succinct/Bp.h"
#include "nn/nlib/succinct/Sbv.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {
namespace detail {

class GotoArcHolder NLIB_FINAL {
 public:
    uint64_t GetPos(uint32_t nodeid, unsigned char c) NLIB_NOEXCEPT {
        uint64_t rval = m_NumNodeId;
        rval = rval * c + nodeid;
        return rval;
    }

 public:
    GotoArcHolder() NLIB_NOEXCEPT;
    bool Init(uint32_t num_nodeid) NLIB_NOEXCEPT;
    bool TurnOn(uint32_t nodeid, unsigned char c) NLIB_NOEXCEPT {
        uint64_t idx = this->GetPos(nodeid, c);
        return m_Set.TurnOn(idx);
    }
    bool Build() NLIB_NOEXCEPT { return m_Set.Build(); }
    int32_t Goto(uint32_t nodeid, unsigned char c) NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT;
    size_t MemSize() const NLIB_NOEXCEPT { return sizeof(m_NumNodeId) + m_Set.MemSize(); }
    bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    bool Import(BinaryReader* r) NLIB_NOEXCEPT;
    void swap(GotoArcHolder& rhs) NLIB_NOEXCEPT {  // NOLINT
        using std::swap;
        swap(m_Set, rhs.m_Set);
        swap(m_NumNodeId, rhs.m_NumNodeId);
    }

 private:
    SparseSet m_Set;
    uint32_t m_NumNodeId;
    NLIB_DISALLOW_COPY_AND_ASSIGN(GotoArcHolder);
};

}  // namespace detail

class AhoCorasick NLIB_FINAL {
 public:
    static const size_t LABEL_WIDTH = 8;  // 1,2,4,8
    static const int LABEL_MASK = (1 << LABEL_WIDTH) - 1;
    static const int NUMLABEL_PER_BYTE = 8 / LABEL_WIDTH;

 public:
    NLIB_VIS_PUBLIC AhoCorasick() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC ~AhoCorasick() NLIB_NOEXCEPT;
    NLIB_MOVE_MEMBER_HELPER_4(AhoCorasick, m_LenHolder, m_ReportTreeHolder, m_GotoArcHolder,
                              m_FailureTreeHolder);
    NLIB_VIS_PUBLIC void swap(AhoCorasick& rhs) NLIB_NOEXCEPT;
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    void Match(const char* doc, MatchCallback callback, void* userobj = NULL) NLIB_NOEXCEPT {
        this->Match(doc, nlib_strlen(doc), callback, userobj);
    }
    NLIB_VIS_PUBLIC void Match(const void* data, size_t n, MatchCallback callback,
                               void* userobj = NULL) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC size_t MemSize() const NLIB_NOEXCEPT;

    NLIB_VIS_PUBLIC void MemSize(size_t* len_store, size_t* goto_arc, size_t* report_arc,
                                 size_t* failure_arc) const NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC void Reset() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Import(BinaryReader* r) NLIB_NOEXCEPT;

 protected:
    Map m_LenHolder;         // holds len
    Map m_ReportTreeHolder;  // holds nodeid
    detail::GotoArcHolder m_GotoArcHolder;
    Bp m_FailureTreeHolder;

 private:
    friend class AhoCorasickBuilder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AhoCorasick);
};

}  // namespace succinct
NLIB_NAMESPACE_END

NLIB_DEFINE_STD_SWAP(::nlib_ns::succinct::AhoCorasick)

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_AHOCORASICK_H_
