﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_TRIE_H_
#define INCLUDE_NN_NLIB_SUCCINCT_TRIE_H_

#include <string.h>

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/succinct/Louds.h"
#include "nn/nlib/succinct/Sbv.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {

class Trie NLIB_FINAL {
 public:
    Trie() NLIB_NOEXCEPT {}
    ~Trie() NLIB_NOEXCEPT {}
    NLIB_MOVE_MEMBER_HELPER_3(Trie, m_Louds, m_IsTerminal, m_Labels);
    void swap(Trie& rhs) NLIB_NOEXCEPT {
        m_Louds.swap(rhs.m_Louds);
        m_IsTerminal.swap(rhs.m_IsTerminal);
        m_Labels.swap(rhs.m_Labels);
    }
    bool Init() NLIB_NOEXCEPT {
        this->Reset();
        return true;
    }
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    void Match(const char* cstr, MatchCallback callback, void* user_obj = NULL) NLIB_NOEXCEPT {
        this->Match(cstr, nlib_strlen(cstr), callback, user_obj);
    }
    NLIB_VIS_PUBLIC void Match(const void* data, size_t n, MatchCallback callback,
                               void* user_obj = NULL) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC void MatchBackward(const void* data, size_t n, MatchCallback callback,
                                       void* user_obj = NULL) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC errno_t GetCommonPrefixWords(const void* prefix, size_t n,
                                                 ReallocCstringVec* vec) NLIB_NOEXCEPT;
    void Reset() NLIB_NOEXCEPT {
        m_Louds.Reset();
        m_IsTerminal.Reset();
        m_Labels.Reset();
    }
    NLIB_VIS_PUBLIC bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Import(BinaryReader* r) NLIB_NOEXCEPT;

 private:
    NLIB_VIS_HIDDEN int Match_(const void* data, const unsigned char* p, uint32_t pos,
                               MatchCallback callback, void* user_obj, bool isfwd) NLIB_NOEXCEPT;

 private:
    Louds m_Louds;
    detail::BitVector32 m_IsTerminal;
    detail::PlainArray32<unsigned char> m_Labels;
    friend class TrieBuilder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(Trie);
};

class TrieBuilder NLIB_FINAL {
 public:
    TrieBuilder() NLIB_NOEXCEPT {}
    ~TrieBuilder() NLIB_NOEXCEPT {}
    NLIB_VIS_PUBLIC bool Init() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC Trie* Build(ReallocVec<uint32_t>* keyids = NULL) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddWord(const char* str) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddPattern(const void* p, size_t n) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddWords(const char* str) NLIB_NOEXCEPT {
        return AddWords(str, strlen(str));
    }

 private:
    ReallocCstringVec m_Words;
    NLIB_DISALLOW_COPY_AND_ASSIGN(TrieBuilder);
};

}  // namespace succinct
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_TRIE_H_
