﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_
#define INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_

#include <string.h>

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/succinct/AhoCorasick.h"
#include "nn/nlib/succinct/Trie.h"

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace succinct {

class WordFilterImpl;
class WordFilter NLIB_FINAL {
 public:
    typedef bool (*MatchCallback)(const char* first, const char* last, uint32_t nodeid,
                                  void* user_obj);
    NLIB_VIS_PUBLIC WordFilter() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC ~WordFilter() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC WordFilter(WordFilter& rhs, move_tag) NLIB_NOEXCEPT;  // NOLINT
    WordFilter& assign(WordFilter& rhs, move_tag) NLIB_NOEXCEPT {  // NOLINT
        WordFilter().swap(*this);
        this->swap(rhs);
        return *this;
    }
#ifdef NLIB_CXX11_RVALUE_REFERENCES
    NLIB_MOVE_MEMBER_HELPER_X_COMMON(WordFilter)
    NLIB_VIS_PUBLIC WordFilter(WordFilter&& rhs) NLIB_NOEXCEPT;  // NOLINT
#endif
    void swap(WordFilter& rhs) NLIB_NOEXCEPT { m_Impl.swap(rhs.m_Impl); }
    NLIB_VIS_PUBLIC bool Init() NLIB_NOEXCEPT;
    void Match(const char* doc, MatchCallback callback, void* user_obj = NULL) NLIB_NOEXCEPT {
        this->Match(doc, nlib_strlen(doc), callback, user_obj);
    }
    NLIB_VIS_PUBLIC void Match(const char* data, size_t n, MatchCallback callback,
                               void* user_obj = NULL) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC void Reset() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Export(BinaryWriter* w) const NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool Import(BinaryReader* r) NLIB_NOEXCEPT;

 private:
    UniquePtr<WordFilterImpl> m_Impl;
    friend class WordFilterBuilder;
    NLIB_DISALLOW_COPY_AND_ASSIGN(WordFilter);
};

class WordFilterBuilder NLIB_FINAL {
 public:
    WordFilterBuilder() NLIB_NOEXCEPT {}
    ~WordFilterBuilder() NLIB_NOEXCEPT {}
    NLIB_VIS_PUBLIC bool Init() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC WordFilter* Build() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddWord(const char* str) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddWords(const char* str) NLIB_NOEXCEPT {
        return AddWords(str, strlen(str));
    }
    NLIB_VIS_PUBLIC bool AddExcludeWord(const char* str) NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC bool AddExcludeWords(const char* str, size_t len) NLIB_NOEXCEPT;
    bool AddExcludeWords(const char* str) NLIB_NOEXCEPT {
        return AddExcludeWords(str, strlen(str));
    }

 private:
    ReallocCstringVec m_MatchWords;
    ReallocCstringVec m_ExcludeWords;
    NLIB_DISALLOW_COPY_AND_ASSIGN(WordFilterBuilder);
};

}  // namespace succinct
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_succinct_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_SUCCINCT_WORDFILTER_H_
