﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_TESTING_PRINTER_H_
#define INCLUDE_NN_NLIB_TESTING_PRINTER_H_

#include <string>
#include <utility>
#include <vector>

#include "nn/nlib/Config.h"

#if defined(_MSC_VER) && defined(nx_testing_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINEXPORT
#endif

NLIB_NAMESPACE_BEGIN
namespace testing {

namespace detail_func {

class DummyStream NLIB_FINAL {
 public:
    DummyStream() : m_Str() {}
    void swap(std::string& rhs) { m_Str.swap(rhs); }  // NOLINT
    const std::string& str() const { return m_Str; }
    std::string& str() { return m_Str; }

 public:
    DummyStream& operator<<(int32_t value);
    DummyStream& operator<<(uint32_t value);
    DummyStream& operator<<(int64_t value);
    DummyStream& operator<<(uint64_t value);
    DummyStream& operator<<(float value);
    DummyStream& operator<<(double value);
    DummyStream& operator<<(bool value);
    DummyStream& operator<<(const char* str);
    DummyStream& operator<<(const wchar_t* str);
    DummyStream& operator<<(const std::string& str);
    DummyStream& operator<<(const std::wstring& str);
    DummyStream& operator<<(char c);
    DummyStream& operator<<(wchar_t c);
    DummyStream& operator<<(unsigned char c);
    DummyStream& operator<<(signed char c);
    template <class T>
    DummyStream& operator<<(const T& rhs) {
        return Op(*this, rhs);
    }

 private:
    std::string m_Str;
};

// PrintTo
NLIB_VIS_PUBLIC void PrintTo(int32_t value, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(uint32_t x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(int64_t x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(uint64_t x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(float x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(double x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(bool x, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(const char* str, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(const wchar_t* str, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(char c, DummyStream* ostr);
NLIB_VIS_PUBLIC void PrintTo(wchar_t c, DummyStream* ostr);

inline void PrintTo(const std::string& str, DummyStream* ostr) { PrintTo(str.c_str(), ostr); }

inline void PrintTo(const std::wstring& str, DummyStream* ostr) { PrintTo(str.c_str(), ostr); }

inline void PrintTo(signed char c, DummyStream* ostr) { PrintTo(static_cast<char>(c), ostr); }

inline void PrintTo(unsigned char c, DummyStream* ostr) { PrintTo(static_cast<char>(c), ostr); }

template <class T>
void PrintToArray(const T* a, size_t n, DummyStream* ostr) {
    if (n == 0) {
        PrintTo("{}", ostr);
    } else {
        PrintTo("{", ostr);
        PrintTo(a[0], ostr);
        size_t nn = (n > 8) ? 8 : n;
        for (size_t i = 1; i < nn; ++i) {
            PrintTo(", ", ostr);
            PrintTo(a[i], ostr);
        }
        PrintTo((nn == n) ? "}" : ", ...}", ostr);
    }
}

template <class T, size_t N>
inline void PrintTo(const T (&a)[N], DummyStream* ostr) {
    PrintToArray(&a[0], N, ostr);
}

template <class T, class Alloc>
inline void PrintTo(const std::vector<T, Alloc>& value, DummyStream* ostr) {
    PrintToArray(&value[0], value.size(), ostr);
}

template <class T1, class T2>
void PrintTo(const std::pair<T1, T2>& value, DummyStream* ostr) {
    PrintTo("(", ostr);
    PrintTo(value.first, ostr);
    PrintTo(", ", ostr);
    PrintTo(value.second, ostr);
    PrintTo(")", ostr);
}

// NOTE: Add overloads of PrintTo if needed.

// DummyStream::operator<<
#define NLIB_DEF_DSOP_(tp)                   \
inline DummyStream& DummyStream::operator<<(tp x) { \
        PrintTo(x, this);                    \
        return *this;                        \
}

NLIB_DEF_DSOP_(int32_t)
NLIB_DEF_DSOP_(uint32_t)
NLIB_DEF_DSOP_(int64_t)
NLIB_DEF_DSOP_(uint64_t)
NLIB_DEF_DSOP_(float)   // NOLINT
NLIB_DEF_DSOP_(double)  // NOLINT
NLIB_DEF_DSOP_(bool)    // NOLINT
NLIB_DEF_DSOP_(const char*)
NLIB_DEF_DSOP_(const wchar_t*)
NLIB_DEF_DSOP_(const std::string&)
NLIB_DEF_DSOP_(const std::wstring&)
NLIB_DEF_DSOP_(char)  // NOLINT
NLIB_DEF_DSOP_(wchar_t)
NLIB_DEF_DSOP_(signed char)
NLIB_DEF_DSOP_(unsigned char)

#undef NLIB_DEF_DSOP_

#define NLIB_DEF_FUNCOP_(tp, coerce_tp)          \
inline DummyStream& Op(DummyStream& os, tp x) {  \
        PrintTo(static_cast<coerce_tp>(x), &os); \
        return os;                               \
    }  // NOLINT
NLIB_DEF_FUNCOP_(int, int32_t)
NLIB_DEF_FUNCOP_(unsigned int, uint32_t)
NLIB_DEF_FUNCOP_(short, int32_t)                // NOLINT
NLIB_DEF_FUNCOP_(unsigned short, uint32_t)      // NOLINT
NLIB_DEF_FUNCOP_(long, int32_t)                 // NOLINT
NLIB_DEF_FUNCOP_(unsigned long, uint32_t)       // NOLINT
NLIB_DEF_FUNCOP_(long long, int32_t)            // NOLINT
NLIB_DEF_FUNCOP_(unsigned long long, uint32_t)  // NOLINT
#undef NLIB_DEF_FUNCOP_

template <class T>
inline DummyStream& Op(DummyStream& ostr, const T& x) {  // NOLINT
    // This is the last(default) function.....
    NLIB_UNUSED(x);
    // TODO(nishida_kenji): Print byte sequence of x?
    ostr.str().append("?????");
    return ostr;
}

template <class T, class DUMMY>
inline void PrintTo(const T& value, DUMMY* ostr) {
    // NOTE:
    // DUMMY must be a template, not DummyStream.
    // It is to use user-defined PrintTo(UserType, STREAM* ostr) prior to this function.
    //
    // Though you can write as follows using gtest:
    //   void PrintTo(const Bar& bar, std::ostream* os) { ... }
    // using nlib::testing, we have to write like this:
    //   template<class OSTREAM> void PrintTo(const Bar& bar, OSTREAM* os) { ... }
    *ostr << value;
}

}  // namespace detail_func

template <class T>
inline NLIB_VIS_HIDDEN std::string PrintToString(const T& value) {
    // NOTE:
    // we have to make nlib::testing work if we cannot use std::ostream.
    ::nlib_ns::testing::detail_func::DummyStream str;

    // NOTE:
    // ADL is performed, if PrintTo for type T is defined.
    // otherwise, ::nlib_ns::testing::detail_func::PrintTo is used.
    using ::nlib_ns::testing::detail_func::PrintTo;
    PrintTo(value, &str);
    return str.str();
}

}  // namespace testing
NLIB_NAMESPACE_END

#if defined(_MSC_VER) && defined(nx_testing_EXPORTS)
#undef NLIB_VIS_PUBLIC
#define NLIB_VIS_PUBLIC NLIB_WINIMPORT
#endif

#endif  // INCLUDE_NN_NLIB_TESTING_PRINTER_H_
