﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_
#define INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_

#include "nn/nlib/UniquePtr.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

class AsyncFileIoService;
template <class R>
class Future;

class NLIB_VIS_PUBLIC AsyncFileIo NLIB_FINAL {
 public:
    enum Mode {
        ASYNCFILEIO_READ = NLIB_FD_O_RDONLY,
        ASYNCFILEIO_WRITE = NLIB_FD_O_WRONLY | NLIB_FD_O_CREAT | NLIB_FD_O_TRUNC
    };
    static bool HasNativeAsyncFileIo() NLIB_NOEXCEPT;
    AsyncFileIo() NLIB_NOEXCEPT;
    ~AsyncFileIo() NLIB_NOEXCEPT;
    errno_t Init() NLIB_NOEXCEPT;
    errno_t Open(const char* path, unsigned int flags) NLIB_NOEXCEPT;
    errno_t FdOpen(nlib_fd fd) NLIB_NOEXCEPT;
    NLIB_ALWAYS_INLINE nlib_fd GetFd() const NLIB_NOEXCEPT { return m_Fd; }
    errno_t Close() NLIB_NOEXCEPT;
    errno_t Read(Future<size_t>* future, void* buf, size_t nbytes, nlib_offset ofs,
                 AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Read(size_t* readBytes, void* buf, size_t nbytes, nlib_offset ofs,
                 AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Write(Future<size_t>* future, const void* buf, size_t nbytes, nlib_offset ofs,
                  AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Write(size_t* writeBytes, const void* buf, size_t nbytes, nlib_offset ofs,
                  AsyncFileIoService* ioservice) NLIB_NOEXCEPT;
    errno_t Cancel(AsyncFileIoService* ioservice) NLIB_NOEXCEPT;

 private:
    nlib_fd m_Fd;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AsyncFileIo);
};

struct AsyncFileIoServiceData;
class AsyncFileIoService {
 public:
    NLIB_VIS_PUBLIC AsyncFileIoService() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC ~AsyncFileIoService() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC errno_t Init() NLIB_NOEXCEPT;

 private:
    UniquePtr<AsyncFileIoServiceData> m_Data;
    NLIB_DISALLOW_COPY_AND_ASSIGN(AsyncFileIoService);
    friend class AsyncFileIo;
};

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_ASYNCFILEIO_H_
