﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_CONDVARFALLBACK_H_
#define INCLUDE_NN_NLIB_THREADING_CONDVARFALLBACK_H_

#include "nn/nlib/UniquePtr.h"
#include "nn/nlib/DateTime.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

class CondVarFallback NLIB_FINAL {
 public:
    NLIB_DEPRECATED NLIB_VIS_PUBLIC CondVarFallback() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC ~CondVarFallback() NLIB_NOEXCEPT;
    NLIB_VIS_PUBLIC errno_t Init() NLIB_NOEXCEPT;
    void Notify() NLIB_NOEXCEPT { this->signal_(false); }
    void NotifyAll() NLIB_NOEXCEPT { this->signal_(true); }
    template <class lock_type>
    errno_t Wait(lock_type& lock) NLIB_NOEXCEPT {  // NOLINT
        return this->wait_(lock, NULL);
    }
    template <class lock_type>
    errno_t WaitFor(lock_type& lock,  // NOLINT
                                       const nlib_ns::TimeSpan& timeout) NLIB_NOEXCEPT {
        return this->wait_(lock, &timeout);
    }
    template <class lock_type>
    errno_t WaitUntil(lock_type& lock,  // NOLINT
                                         const nlib_ns::DateTime& datetime) NLIB_NOEXCEPT {
        DateTime now;
        errno_t e = DateTime::GetNow(&now);
        if (e != 0) return e;
        return this->WaitFor(lock, datetime - now);
    }

 private:
    template <class lock_type>
    struct ScopedUnlock {
        explicit ScopedUnlock(lock_type& lock) : m_Lock(lock) {  // NOLINT
            m_Lock.unlock();
        }  // NOLINT
        ~ScopedUnlock() { m_Lock.lock(); }

     private:
        lock_type& m_Lock;
        NLIB_DISALLOW_COPY_AND_ASSIGN(ScopedUnlock);
    };
    errno_t signal_(bool bAll);
    template <class lock_type>
    errno_t wait_(lock_type& lock, const nlib_ns::TimeSpan* timeout);  // NOLINT
    errno_t wait_1();
    errno_t wait_2(const nlib_ns::TimeSpan* timeout);

 private:
    struct Data_;
    UniquePtr<Data_> m_Obj;
};

template <class lock_type>
inline errno_t CondVarFallback::wait_(lock_type& lock,  // NOLINT
                                      const nlib_ns::TimeSpan* timeout) {
    errno_t e;
    if (!m_Obj) return EINVAL;
    e = this->wait_1();
    if (e != 0) return e;
    ScopedUnlock<lock_type> ulk(lock);
    return this->wait_2(timeout);
}

#ifdef NLIB_CONDVAR_USE_FALLBACK
typedef CondVarFallback CondVar;
#endif

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_CONDVARFALLBACK_H_
