﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_SEMAPHORE_H_
#define INCLUDE_NN_NLIB_THREADING_SEMAPHORE_H_

#include "nn/nlib/Config.h"
#include "nn/nlib/DateTime.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

class Semaphore {
 public:
    typedef nlib_semaphore* NativeHandle;

    Semaphore() NLIB_NOEXCEPT : m_Obj() {}
    ~Semaphore() NLIB_NOEXCEPT { nlib_semaphore_destroy(&m_Obj); }
    errno_t Init(int initialCount) NLIB_NOEXCEPT {
        return nlib_semaphore_init(&m_Obj, initialCount);
    }
    errno_t Release(int* previousCount) NLIB_NOEXCEPT {
        return nlib_semaphore_post(&m_Obj, previousCount);
    }
    errno_t Release(int releaseCount, int* previousCount) NLIB_NOEXCEPT {
        return nlib_semaphore_post_ex(&m_Obj, releaseCount, previousCount);
    }
    errno_t Acquire() NLIB_NOEXCEPT { return nlib_semaphore_wait(&m_Obj); }
    errno_t TryAcquire() NLIB_NOEXCEPT {
        return nlib_semaphore_trywait(&m_Obj);
    }
    errno_t TryAcquire(const nlib_ns::TimeSpan& timeout) NLIB_NOEXCEPT {
        return nlib_semaphore_trywait_for(&m_Obj, timeout.ToTimeValue().tick);
    }
    NativeHandle GetNativeHandle() NLIB_NOEXCEPT { return &m_Obj; }

 private:
    nlib_semaphore m_Obj;
};

}  // namespace threading
NLIB_NAMESPACE_END

NLIB_NAMESPACE_BEGIN
namespace threading {

class ScopedSemaphore {
 public:
    explicit NLIB_ALWAYS_INLINE ScopedSemaphore(Semaphore& rhs) NLIB_NOEXCEPT  // NOLINT
        : m_Semaphore(rhs) {
        m_Semaphore.Acquire();
    }
    NLIB_ALWAYS_INLINE ~ScopedSemaphore() NLIB_NOEXCEPT {
        m_Semaphore.Release(NULL);
    }

 private:
    Semaphore& m_Semaphore;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ScopedSemaphore);
};

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_SEMAPHORE_H_
