﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_THREADING_TLS_H_
#define INCLUDE_NN_NLIB_THREADING_TLS_H_

#include "nn/nlib/Config.h"

NLIB_NAMESPACE_BEGIN
namespace threading {

template <class T>
class Tls NLIB_FINAL {
 public:
    NLIB_CEXPR Tls() NLIB_NOEXCEPT : m_IdxValid(false) {}
    errno_t Init() NLIB_NOEXCEPT {
        if (m_IdxValid) return EALREADY;
        errno_t e = nlib_tls_alloc(&m_Idx, destr_func);
        if (e == 0) m_IdxValid = true;
        return e;
    }
    ~Tls() NLIB_NOEXCEPT {
        if (m_IdxValid) {
            nlib_tls_free(m_Idx);
            m_IdxValid = false;
        }
    }
    T* Get() const NLIB_NOEXCEPT {
        void* p;
        errno_t e = nlib_tls_getvalue(m_Idx, &p);
        if (e != 0) return NULL;
        return reinterpret_cast<T*>(p);
    }
    T* operator->() const NLIB_NOEXCEPT { return this->Get(); }
    T& operator*() const NLIB_NOEXCEPT { return *this->Get(); }
    T* Release() NLIB_NOEXCEPT {
        T* obj = this->Get();
        nlib_tls_setvalue(m_Idx, NULL);
        return obj;
    }
    errno_t Reset(T* p = 0) NLIB_NOEXCEPT {
        T* obj = this->Get();
        if (obj) {
            delete obj;
        }
        return nlib_tls_setvalue(m_Idx, p);
    }

 private:
    static void destr_func(void* p) NLIB_NOEXCEPT {
        delete reinterpret_cast<T*>(p);
    }
    nlib_tls m_Idx;
    bool m_IdxValid;
};

}  // namespace threading
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_THREADING_TLS_H_
