﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_UNICODE_CONVERT_H_
#define INCLUDE_NN_NLIB_UNICODE_CONVERT_H_

#include "nn/nlib/Config.h"

// functions below may be deprecated in the future
NLIB_NAMESPACE_BEGIN
namespace unicode {

class NLIB_VIS_PUBLIC ToUtf8Obj NLIB_FINAL {
 public:
    explicit ToUtf8Obj(const wchar_t* wstr) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const nlib_utf16_t* utf16str) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const nlib_utf32_t* utf32str) NLIB_NOEXCEPT;
    explicit ToUtf8Obj(const char* str) NLIB_NOEXCEPT {
        // does not work if the encoding is not UTF-8.
        m_Buf = str;
        m_Alloc = false;
    }
    ~ToUtf8Obj() NLIB_NOEXCEPT {
        if (m_Alloc) delete[] m_Buf;
    }
    operator const char*() NLIB_NOEXCEPT { return m_Buf ? m_Buf : ""; }
    const char* c_str() NLIB_NOEXCEPT { return m_Buf ? m_Buf : ""; }
    bool IsOk() const NLIB_NOEXCEPT { return m_Buf != NULL; }

 private:
    const char* m_Buf;
    bool m_Alloc;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ToUtf8Obj);
};

class NLIB_VIS_PUBLIC ToWideObj NLIB_FINAL {
 public:
    explicit ToWideObj(const char* str) NLIB_NOEXCEPT;
    explicit ToWideObj(const wchar_t* wstr) NLIB_NOEXCEPT {
        m_Buf = wstr;
        m_Alloc = false;
    }
    ~ToWideObj() NLIB_NOEXCEPT {
        if (m_Alloc) delete[] m_Buf;
    }
    operator const wchar_t*() NLIB_NOEXCEPT { return m_Buf ? m_Buf : L""; }
    const wchar_t* c_str() NLIB_NOEXCEPT { return m_Buf ? m_Buf : L""; }
    bool IsOk() const NLIB_NOEXCEPT { return m_Buf != NULL; }

 private:
    const wchar_t* m_Buf;
    bool m_Alloc;
    NLIB_DISALLOW_COPY_AND_ASSIGN(ToWideObj);
};

}  // namespace unicode
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_UNICODE_CONVERT_H_
