﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/
#include "def_serializer.h"
#include "nn/nlib/Cstring.h"
#include "nn/nlib/StrTo.h"

using nlib_ns::OutputStream;
using nlib_ns::InputStream;
using nlib_ns::exi::XmlStreamReader;
using nlib_ns::exi::XmlStreamWriter;
using nlib_ns::exi::ExiChar;
using nlib_ns::SnPrintf;
using nlib_ns::StrTo;

SimpleSerializer::SimpleSerializer(OutputStream* os) {
    m_Writer.reset(XmlStreamWriter::Create(os));
}

template <>
void SimpleSerializer::serialize(const StdString& x) {
    m_Writer->WriteStartElement(N("s"));
    m_Writer->WriteCharacters(x.c_str());
    m_Writer->WriteEndElement();
}

template <>
void SimpleSerializer::serialize(const int& x) {
    char b[32];
    SnPrintf(b, "%d", x);
    m_Writer->WriteStartElement(N("i"));
    m_Writer->WriteCharacters(NLIB_EXI_STRING(b));
    m_Writer->WriteEndElement();
}

template <>
void SimpleSerializer::serialize(const float& x) {
    char b[32];
    SnPrintf(b, "%f", x);
    m_Writer->WriteStartElement(N("f"));
    m_Writer->WriteCharacters(NLIB_EXI_STRING(b));
    m_Writer->WriteEndElement();
}

SimpleDeserializer::SimpleDeserializer(InputStream* is) {
    m_Reader.reset(XmlStreamReader::Create(is));
    m_Failed = false;
    m_Cache = XmlStreamReader::NONE;
}

void SimpleDeserializer::Next(XmlStreamReader::XmlStreamConstants expect) {
    if (m_Failed) return;
    if (m_Cache == XmlStreamReader::NONE) m_Cache = m_Reader->Next();
    m_Failed |= (m_Cache != expect);
    m_Cache = XmlStreamReader::NONE;
}

bool SimpleDeserializer::Peek(XmlStreamReader::XmlStreamConstants expect) {
    if (m_Failed) return false;
    if (m_Cache == XmlStreamReader::NONE) m_Cache = m_Reader->Next();
    return (m_Cache == expect);
}

template <>
void SimpleDeserializer::serialize(StdString& x) {  // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);     // s
    this->Next(XmlStreamReader::CHARACTERS);
    x = m_Reader->GetText();
    this->Next(XmlStreamReader::END_ELEMENT);  // s
}

template <>
void SimpleDeserializer::serialize(int& x) {     // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);  // i
    this->Next(XmlStreamReader::CHARACTERS);
    errno_t e = StrTo(M(m_Reader->GetText()), 10, &x);
    if (e != 0) x = 0;
    this->Next(XmlStreamReader::END_ELEMENT);  // i
}

template <>
void SimpleDeserializer::serialize(float& x) {   // NOLINT
    this->Next(XmlStreamReader::START_ELEMENT);  // f
    this->Next(XmlStreamReader::CHARACTERS);
    errno_t e = StrTo(M(m_Reader->GetText()), &x);
    if (e != 0) x = 0.f;
    this->Next(XmlStreamReader::END_ELEMENT);  // f
}
