﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#include "nn/nlib/Config.h"

template<class TAG, class ERRORTYPE = bool>
class SafeInit {
 public:
    typedef ERRORTYPE ErrorType;
    typedef TAG TagType;
    static const ErrorType& Init() {
        // You can implement timeout by using nlib_tryonce().
        nlib_once(&m_Flag, OnceFunc);
        return m_Error;
    }

 private:
    static nlib_onceflag m_Flag;
    static ErrorType m_Error;
    static void OnceFunc();
};

struct a_init_tag {};
struct b_init_tag {};
struct c_init_tag {};
struct d_init_tag {};

template<> bool SafeInit<a_init_tag>::m_Error = false;
template<> nlib_onceflag SafeInit<a_init_tag>::m_Flag = NLIB_ONCE_INIT;
template<> void SafeInit<a_init_tag>::OnceFunc() {
    nlib_printf("SafeInit<a_init_tag>::OnceFunc()\n");
}

template<> bool SafeInit<b_init_tag>::m_Error = false;
template<> nlib_onceflag SafeInit<b_init_tag>::m_Flag = NLIB_ONCE_INIT;
template<> void SafeInit<b_init_tag>::OnceFunc() {
    bool err;
    err = SafeInit<a_init_tag>::Init();  // 'b' depends 'a'.
    if (err) {
        m_Error = true;
        return;
    }
    nlib_printf("SafeInit<b_init_tag>::OnceFunc()\n");
}

template<> bool SafeInit<c_init_tag>::m_Error = false;
template<> nlib_onceflag SafeInit<c_init_tag>::m_Flag = NLIB_ONCE_INIT;
template<> void SafeInit<c_init_tag>::OnceFunc() {
    bool err;
    err = SafeInit<a_init_tag>::Init();  // 'c' depends 'a'.
    if (err) {
        m_Error = true;
        return;
    }
    nlib_printf("SafeInit<c_init_tag>::OnceFunc()\n");
}

template<> bool SafeInit<d_init_tag>::m_Error = false;
template<> nlib_onceflag SafeInit<d_init_tag>::m_Flag = NLIB_ONCE_INIT;
template<> void SafeInit<d_init_tag>::OnceFunc() {
    bool err;
    err = SafeInit<b_init_tag>::Init();  // 'd' depends 'b'.
    if (err) {
        m_Error = true;
        return;
    }
    err = SafeInit<c_init_tag>::Init();  // 'd' depends 'c'.
    if (err) {
        m_Error = true;
        return;
    }
    nlib_printf("SafeInit<d_init_tag>::OnceFunc()\n");
}

bool SafeInitDemo() {
    bool err;

    err = SafeInit<c_init_tag>::Init();
    if (err) return false;
    err = SafeInit<d_init_tag>::Init();

    return !err;
}

static bool SampleMain(int, char**) { return SafeInitDemo(); }

NLIB_MAINFUNC
