﻿
/*---------------------------------------------------------------------------*

  Project: CrossRoad
  Copyright (C)2012-2015 Nintendo.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.

 *---------------------------------------------------------------------------*/

#pragma once
#ifndef INCLUDE_NN_NLIB_SIMD_SIMDINT_H_
#define INCLUDE_NN_NLIB_SIMD_SIMDINT_H_

#include "nn/nlib/Config.h"

#if defined(NLIB_SSE41)
typedef __m128i nlib_i64_t;
typedef __m128i nlib_i128_t;
#elif defined(NLIB_NEON)
typedef int8x8_t nlib_i64_t;
typedef int8x16_t nlib_i128_t;
#endif

#ifdef NLIB_SSE41
extern int nlib_sse42_supported;
#endif

NLIB_NAMESPACE_BEGIN
namespace simd {

// use each_int8 for the argument value
struct each_int8_tag {};
// use each_int16 for the argument value
struct each_int16_tag {};
// use each_int32 for the argument value
struct each_int32_tag {};
// use each_int64 for the argument value
struct each_int64_tag {};
// use each_uint8 for the argument value
struct each_uint8_tag {};
// use each_uint16 for the argument value
struct each_uint16_tag {};
// use each_uint32 for the argument value
struct each_uint32_tag {};
// use each_uint64 for the argument value
struct each_uint64_tag {};

NLIB_CEXPR const each_int8_tag each_int8 = {};
NLIB_CEXPR const each_int16_tag each_int16 = {};
NLIB_CEXPR const each_int32_tag each_int32 = {};
NLIB_CEXPR const each_int64_tag each_int64 = {};
NLIB_CEXPR const each_uint8_tag each_uint8 = {};
NLIB_CEXPR const each_uint16_tag each_uint16 = {};
NLIB_CEXPR const each_uint32_tag each_uint32 = {};
NLIB_CEXPR const each_uint64_tag each_uint64 = {};

// use each_select32 for the argument value
struct each_select32_tag {};
// use each_select16 for the argument value
struct each_select16_tag {};
// use each_select8 for the argument value
struct each_select8_tag {};

NLIB_CEXPR const each_select32_tag each_select32 = {};
NLIB_CEXPR const each_select16_tag each_select16 = {};
NLIB_CEXPR const each_select8_tag each_select8 = {};

#if !defined(_MSC_VER) || _MSC_VER < 1800
#ifndef __vectorcall
#define __vectorcall
#endif
#endif

#if defined(NLIB_SIMD)

// __m128i(SSE), int8x8_t(NEON)
typedef nlib_i64_t i64;
// __m128i(SSE), int8x16_t(NEON)
typedef nlib_i128_t i128;

#if _MSC_VER < 1800
typedef const i64& i64arg;
typedef const i128& i128arg;
#else
typedef const i64 i64arg;
typedef const i128 i128arg;
#endif

struct NLIB_VIS_HIDDEN I64 {
    static i64 __vectorcall SetValue(int8_t v, each_int8_tag) NLIB_NOEXCEPT;
    static i64 __vectorcall SetValue(int16_t v, each_int16_tag) NLIB_NOEXCEPT;
    static i64 __vectorcall SetValue(int32_t v, each_int32_tag) NLIB_NOEXCEPT;
    static i64 __vectorcall SetValue(uint8_t v, each_uint8_tag) NLIB_NOEXCEPT;
    static i64 __vectorcall SetValue(uint16_t v, each_uint16_tag) NLIB_NOEXCEPT;
    static i64 __vectorcall SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT;

    template <size_t N>
    static i64 __vectorcall SetValue(i64 value, each_select32_tag) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall SetValue(i64 value, each_select16_tag) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall SetValue(i64 value, each_select8_tag) NLIB_NOEXCEPT;

    static i64 __vectorcall SetZero() NLIB_NOEXCEPT;
    // set 1 to all bits
    static NLIB_ALWAYS_INLINE i64 __vectorcall SetFull(i64arg dummy) NLIB_NOEXCEPT {
        return I64::CmpEq8(dummy, dummy);
    }

    //
    // Load Operations
    //
    static i64 __vectorcall LoadA8(const void* p) NLIB_NOEXCEPT;
    static i64 __vectorcall LoadA4(const void* p) NLIB_NOEXCEPT;
    static i64 __vectorcall LoadA2(const void* p) NLIB_NOEXCEPT;
    static i64 __vectorcall LoadA1(const void* p) NLIB_NOEXCEPT;
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA8(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA8(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA4(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA4(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA2(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA2(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA1(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA1(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA8(intptr_t p) NLIB_NOEXCEPT {
        return LoadA8(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA4(intptr_t p) NLIB_NOEXCEPT {
        return LoadA4(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA2(intptr_t p) NLIB_NOEXCEPT {
        return LoadA2(reinterpret_cast<void*>(p));
    }  // NOLINT
    static NLIB_ALWAYS_INLINE i64 __vectorcall LoadA1(intptr_t p) NLIB_NOEXCEPT {
        return LoadA1(reinterpret_cast<void*>(p));
    }  // NOLINT

    //
    // Store Operations
    //
    static void __vectorcall StoreA8(void* p, i64arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA4(void* p, i64arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA2(void* p, i64arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA1(void* p, i64arg value) NLIB_NOEXCEPT;
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA8(uintptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA8(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA4(uintptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA4(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA2(uintptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA2(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA1(uintptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA1(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA8(intptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA8(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA4(intptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA4(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA2(intptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA2(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static NLIB_ALWAYS_INLINE void __vectorcall StoreA1(intptr_t p, i64arg value) NLIB_NOEXCEPT {
        StoreA1(reinterpret_cast<void*>(p), value);
    }  // NOLINT

    //
    // Get/Set
    //
    template <size_t N>
    static uint8_t __vectorcall GetUint8FromLane(i64arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint16_t __vectorcall GetUint16FromLane(i64arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint32_t __vectorcall GetUint32FromLane(i64arg value) NLIB_NOEXCEPT;

    template <size_t N>
    static i64 __vectorcall SetUint8ToLane(i64arg value, uint8_t v) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall SetUint16ToLane(i64arg value, uint16_t v) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall SetUint32ToLane(i64arg value, uint32_t v) NLIB_NOEXCEPT;

    //
    // Arithmetic Operations
    //
    static i64 __vectorcall Add8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Add16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Add32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Add64(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall AddInt8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall AddInt16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall AddUint8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall AddUint16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall Sub8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Sub16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Sub32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Sub64(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall SubInt8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall SubInt16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall SubUint8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall SubUint16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall PairwiseAdd8(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall PairwiseAdd16(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall PairwiseAdd32(i128arg value) NLIB_NOEXCEPT;

    static i64 __vectorcall Mult16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MultAdd16(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT;
    static i64 __vectorcall MultSub16(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT;
    static i64 __vectorcall Mult32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MultAdd32(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT;
    static i64 __vectorcall MultSub32(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT;

    static i64 __vectorcall MaxInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MaxInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MaxInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall MaxUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MaxUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MaxUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall MinUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall AbsInt8(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall AbsInt16(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall AbsInt32(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall AbsDiffInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall AbsDiffInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall AbsDiffInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall NegateInt8(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall NegateInt16(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall NegateInt32(i64arg value) NLIB_NOEXCEPT;

    //
    // Logical Operations
    //
    static i64 __vectorcall And(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Or(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Xor(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall Not(i64arg a) NLIB_NOEXCEPT;
    static i64 __vectorcall AndNot(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall OrNot(i64arg a, i64arg b) NLIB_NOEXCEPT;

    //
    // Comparison Operations
    //
    static i64 __vectorcall CmpEq8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpEq16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpEq32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall CmpLtInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLtInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLtInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall CmpLtUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLtUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLtUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGtUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall CmpLeInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLeInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLeInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeInt8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeInt16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeInt32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    static i64 __vectorcall CmpLeUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLeUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpLeUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeUint8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeUint16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i64 __vectorcall CmpGeUint32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    //
    // Bit Shift
    //
    static i64 __vectorcall
        ShiftLeftLogical8(i64arg value, int count) NLIB_NOEXCEPT;  // NOLINT not implemented
    static i64 __vectorcall
        ShiftRightLogical8(i64arg value, int count) NLIB_NOEXCEPT;  // NOLINT not implemented

    static i64 __vectorcall ShiftLeftLogical16(i64arg value, int count) NLIB_NOEXCEPT;
    static i64 __vectorcall ShiftRightLogical16(i64arg value, int count) NLIB_NOEXCEPT;
    static i64 __vectorcall ShiftRightArithmetic16(i64arg value, int count) NLIB_NOEXCEPT;

    static i64 __vectorcall ShiftLeftLogical32(i64arg value, int count) NLIB_NOEXCEPT;
    static i64 __vectorcall ShiftRightLogical32(i64arg value, int count) NLIB_NOEXCEPT;
    static i64 __vectorcall ShiftRightArithmetic32(i64arg value, int count) NLIB_NOEXCEPT;

    //
    // 64bit wide Byte Shift/Rotate
    //
    template <size_t N>
    static i64 __vectorcall ByteShiftLeft(i64arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall ByteShiftRight(i64arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i64 __vectorcall ByteRotateRight(i64arg value) NLIB_NOEXCEPT;

    //
    // Conversion
    //
    static i64 __vectorcall NarrowFrom16To8(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall NarrowFrom32To16(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall NarrowFrom64To32(i128arg value) NLIB_NOEXCEPT;

    static i64 __vectorcall ConvertFromUint16ToUint8Saturated(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall ConvertFromInt16ToInt8Saturated(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall ConvertFromUint32ToUint16Saturated(i128arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall ConvertFromInt32ToInt16Saturated(i128arg value) NLIB_NOEXCEPT;

    //
    // Swap endian
    //
    static i64 __vectorcall Reverse16(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall Reverse32(i64arg value) NLIB_NOEXCEPT;
    static i64 __vectorcall Reverse64(i64arg value) NLIB_NOEXCEPT;

    //
    // Misc
    //
    static int __vectorcall MoveMask8(i64arg value) NLIB_NOEXCEPT;  // NOLINT
    static bool __vectorcall IsZero(i64arg value) NLIB_NOEXCEPT;    // NOLINT
    static i64 __vectorcall Select(i64arg mask, i64arg a, i64arg b) NLIB_NOEXCEPT;

 private:
    I64();  // forbidden
};

#ifndef NLIB_DOXYGEN
#define NLIB_M(tp) NLIB_ALWAYS_INLINE tp __vectorcall

#ifdef NLIB_NEON
#define vreinterpret_s8_s8(a) (a)
#define NLIB_OP1(intrin, tp, a) \
    vreinterpret_s8_##tp(intrin##_##tp(vreinterpret_##tp##_s8(a)))
#define NLIB_OP2(intrin, tp, a, b) \
    vreinterpret_s8_##tp(intrin##_##tp(vreinterpret_##tp##_s8(a), \
                                       vreinterpret_##tp##_s8(b)))
#define NLIB_OP3(intrin, tp, a, b, c) \
    vreinterpret_s8_##tp(intrin##_##tp(vreinterpret_##tp##_s8(a), \
                                       vreinterpret_##tp##_s8(b), \
                                       vreinterpret_##tp##_s8(c)))
#define NLIB_CMP(intrin, tp, a, b, utp) \
    vreinterpret_s8_##utp(intrin##_##tp(vreinterpret_##tp##_s8(a), \
                                        vreinterpret_##tp##_s8(b)))
#define NLIB_SFT(intrin, tp, a, cnt, stp) \
    vreinterpret_s8_##tp(intrin##_##tp(vreinterpret_##tp##_s8(a), vdup_n_##stp(cnt)))
#define NLIB_CMB(tp, l, h) vreinterpretq_s8_##tp(vcombine_##tp(l, h))
#endif

// r.s8[i] = v for all i
NLIB_M(i64) I64::SetValue(int8_t v, each_int8_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // faster than _mm_set1_epi8(v)
    return _mm_shuffle_epi8(_mm_cvtsi32_si128(static_cast<uint8_t>(v)), _mm_setzero_si128());
#elif defined(NLIB_NEON)
    return vdup_n_s8(v);
#endif
}

// r.s16[i] = v for all i
NLIB_M(i64) I64::SetValue(int16_t v, each_int16_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi16(v);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_s16(vdup_n_s16(v));
#endif
}

// r.s32[i] = v for all i
NLIB_M(i64) I64::SetValue(int32_t v, each_int32_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi32(v);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_s32(vdup_n_s32(v));
#endif
}

// r.u8[i] = v for all i
NLIB_M(i64) I64::SetValue(uint8_t v, each_uint8_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // faster than _mm_set1_epi8(v)
    return _mm_shuffle_epi8(_mm_cvtsi32_si128(v), _mm_setzero_si128());
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u8(vdup_n_u8(v));
#endif
}

// r.u16[i] = v for all i
NLIB_M(i64) I64::SetValue(uint16_t v, each_uint16_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi16(static_cast<int16_t>(v));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u16(vdup_n_u16(v));
#endif
}

// r.u32[i] = v for all i
NLIB_M(i64) I64::SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi32(static_cast<int32_t>(v));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u32(vdup_n_u32(v));
#endif
}

template <size_t N>
// r.u32[i] = value.u32[N] for all i
NLIB_M(i64) I64::SetValue(i64 value, each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 2);
#if defined(NLIB_SSE41)
    return _mm_shuffle_epi32(value, _MM_SHUFFLE(N, N, N, N));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u32(vdup_lane_u32(vreinterpret_u32_s8(value), N));
#endif
}

template <size_t N>
// r.u16[i] = value.u16[N] for all i
NLIB_M(i64) I64::SetValue(i64 value, each_select16_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_SSE41)
    return _mm_shufflelo_epi16(value, _MM_SHUFFLE(N, N, N, N));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u16(vdup_lane_u16(vreinterpret_u16_s8(value), N));
#endif
}

template <size_t N>
// r.u8[i] = value.u8[N] for all i
NLIB_M(i64) I64::SetValue(i64 value, each_select8_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask[16] = {
        N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask[0]));
#elif defined(NLIB_NEON)
    return vdup_lane_s8(value, N);
#endif
}

// set 0 to all bits
NLIB_M(i64) I64::SetZero() NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_setzero_si128();
#elif defined(NLIB_NEON)
    return vdup_n_s8(0);
#endif
}

// load 8 bytes from p, p must be 8 bytes aligned
NLIB_M(i64) I64::LoadA8(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadl_epi64(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u64(vld1_u64(reinterpret_cast<const uint64_t*>(p)));
#endif
}

// load 8 bytes from p, p must be 4 bytes aligned
NLIB_M(i64) I64::LoadA4(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadl_epi64(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u32(vld1_u32(reinterpret_cast<const uint32_t*>(p)));
#endif
}

// load 8 bytes from p, p must be 2 bytes aligned
NLIB_M(i64) I64::LoadA2(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadl_epi64(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u16(vld1_u16(reinterpret_cast<const uint16_t*>(p)));
#endif
}

// load 8 bytes from p
NLIB_M(i64) I64::LoadA1(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadl_epi64(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    return vld1_s8(reinterpret_cast<const int8_t*>(p));
#endif
}

// store 8 bytes to p, p must be 8 bytes aligned
NLIB_M(void) I64::StoreA8(void* p, i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storel_epi64(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1_u64(reinterpret_cast<uint64_t*>(p), vreinterpret_u64_s8(value));
#endif
}

// store 8 bytes to p, p must be 4 bytes aligned
NLIB_M(void) I64::StoreA4(void* p, i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storel_epi64(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1_u32(reinterpret_cast<uint32_t*>(p), vreinterpret_u32_s8(value));
#endif
}

// store 8 bytes to p, p must be 2 bytes aligned
NLIB_M(void) I64::StoreA2(void* p, i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storel_epi64(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1_u16(reinterpret_cast<uint16_t*>(p), vreinterpret_u16_s8(value));
#endif
}

// store 8 bytes to p
NLIB_M(void) I64::StoreA1(void* p, i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storel_epi64(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1_s8(reinterpret_cast<int8_t*>(p), value);
#endif
}

template <size_t N>
// r = value.u8[N]
NLIB_M(uint8_t) I64::GetUint8FromLane(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return static_cast<uint8_t>(_mm_extract_epi8(value, N));
#elif defined(NLIB_NEON)
    return vget_lane_u8(vreinterpret_u8_s8(value), N);
#endif
}

template <size_t N>
// r = value.u16[N]
NLIB_M(uint16_t) I64::GetUint16FromLane(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_SSE41)
    return static_cast<uint16_t>(_mm_extract_epi16(value, N));
#elif defined(NLIB_NEON)
    return vget_lane_u16(vreinterpret_u16_s8(value), N);
#endif
}

template <size_t N>
// r = value.u32[N]
NLIB_M(uint32_t) I64::GetUint32FromLane(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 2);
#if defined(NLIB_SSE41)
    return static_cast<uint32_t>(_mm_extract_epi32(value, N));
#elif defined(NLIB_NEON)
    return vget_lane_u32(vreinterpret_u32_s8(value), N);
#endif
}

template <size_t N>
// r = value, r.u8[N] = v
NLIB_M(i64) I64::SetUint8ToLane(i64arg value, uint8_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return _mm_insert_epi8(value, static_cast<int8_t>(v), N);
#elif defined(NLIB_NEON)
    return vset_lane_u8(v, vreinterpret_u8_s8(value), N);
#endif
}

template <size_t N>
// r = value, r.u16[N] = v
NLIB_M(i64) I64::SetUint16ToLane(i64arg value, uint16_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_SSE41)
    return _mm_insert_epi16(value, static_cast<uint16_t>(v), N);
#elif defined(NLIB_NEON)
    return vset_lane_u16(v, vreinterpret_u16_s8(value), N);
#endif
}

template <size_t N>
// r = value, r.u32[N] = v
NLIB_M(i64) I64::SetUint32ToLane(i64arg value, uint32_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 2);
#if defined(NLIB_SSE41)
    return _mm_insert_epi32(value, static_cast<int32_t>(v), N);
#elif defined(NLIB_NEON)
    return vset_lane_u32(v, vreinterpret_u32_s8(value), N);
#endif
}

// r.i8[i] = a.i8[i] + b.i8[i]
NLIB_M(i64) I64::Add8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi8(a, b);
#elif defined(NLIB_NEON)
    return vadd_s8(a, b);
#endif
}

// r.i16[i] = a.i16[i] + b.i16[i]
NLIB_M(i64) I64::Add16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vadd, s16, a, b);
#endif
}

// r.i32[i] = a.i32[i] + b.i32[i]
NLIB_M(i64) I64::Add32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vadd, s32, a, b);
#endif
}

// r.i64[0] = a.i64[0] + b.i64[0]
NLIB_M(i64) I64::Add64(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi64(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vadd, s64, a, b);
#endif
}

// 127 if a[i] + b[i] > 127, -128 if a[i] + b[i] < -128
NLIB_M(i64) I64::AddInt8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epi8(a, b);
#elif defined(NLIB_NEON)
    return vqadd_s8(a, b);
#endif
}

// 32767 if a[i] + b[i] > 32767, -32768 if a[i] + b[i] < -32768
NLIB_M(i64) I64::AddInt16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqadd, s16, a, b);
#endif
}

// 255 if a[i] + b[i] > 255
NLIB_M(i64) I64::AddUint8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqadd, u8, a, b);
#endif
}

// 65535 if a[i] + b[i] > 65535
NLIB_M(i64) I64::AddUint16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqadd, u16, a, b);
#endif
}

// r.i8[i] = a.i8[i] - b.i8[i]
NLIB_M(i64) I64::Sub8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi8(a, b);
#elif defined(NLIB_NEON)
    return vsub_s8(a, b);
#endif
}

// r.i16[i] = a.i16[i] - b.i16[i]
NLIB_M(i64) I64::Sub16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsub, s16, a, b);
#endif
}

// r.i32[i] = a.i32[i] - b.i32[i]
NLIB_M(i64) I64::Sub32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsub, s32, a, b);
#endif
}

// r.i64[0] = a.i64[0] - b.i64[0]
NLIB_M(i64) I64::Sub64(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi64(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsub, s64, a, b);
#endif
}

// 127 if a[i] - b[i] > 127, -128 if a[i] - b[i] < -128
NLIB_M(i64) I64::SubInt8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epi8(a, b);
#elif defined(NLIB_NEON)
    return vqsub_s8(a, b);
#endif
}

// 32767 if a[i] - b[i] > 32767, -32768 if a[i] - b[i] < -32768
NLIB_M(i64) I64::SubInt16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsub, s16, a, b);
#endif
}

// 0 if a.u8[i] - b.u8[i] < 0
NLIB_M(i64) I64::SubUint8Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsub, u8, a, b);
#endif
}

// 0 if a.u16[i] - b.u16[i] < 0
NLIB_M(i64) I64::SubUint16Saturated(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsub, u16, a, b);
#endif
}

// r.i8[0] = value[0] + value[1], ..., r.i8[7] = value[14] + value[15]
NLIB_M(i64) I64::PairwiseAdd8(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ax = _mm_add_epi8(value, _mm_srli_epi16(value, 8));
    return I64::NarrowFrom16To8(ax);
#elif defined(NLIB_NEON)
    return vpadd_s8(vget_low_s8(value), vget_high_s8(value));
#endif
}

// r.i16[0] = value[0] + value[1], ..., r.i16[3] = value[6] + value[7]
NLIB_M(i64) I64::PairwiseAdd16(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_hadd_epi16(value, value);
#elif defined(NLIB_NEON)
    int8x8_t lo = vget_low_s8(value);
    int8x8_t hi = vget_high_s8(value);
    return NLIB_OP2(vpadd, s16, lo, hi);
#endif
}

// r.i32[0] = value[0] + value[1], r.i32[1] = value[2] + value[3]
NLIB_M(i64) I64::PairwiseAdd32(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_hadd_epi32(value, value);
#elif defined(NLIB_NEON)
    int8x8_t lo = vget_low_s8(value);
    int8x8_t hi = vget_high_s8(value);
    return NLIB_OP2(vpadd, s32, lo, hi);
#endif
}

// r.i16[i] = a.i16[i] * b.i16[i]
NLIB_M(i64) I64::Mult16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_mullo_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmul, s16, a, b);
#endif
}

// r = c + a * b
NLIB_M(i64) I64::MultAdd16(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi16(c, _mm_mullo_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmla, s16, c, a, b);
#endif
}

// r = c - a * b
NLIB_M(i64) I64::MultSub16(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(c, _mm_mullo_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmls, s16, c, a, b);
#endif
}

// r.i32[i] = a.i32[i] * b.i32[i]
NLIB_M(i64) I64::Mult32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_mullo_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmul, s32, a, b);
#endif
}

// r = c + a * b
NLIB_M(i64) I64::MultAdd32(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi32(c, _mm_mullo_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmla, s32, c, a, b);
#endif
}

// r = c - a * b
NLIB_M(i64) I64::MultSub32(i64arg a, i64arg b, i64arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(c, _mm_mullo_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmls, s32, c, a, b);
#endif
}

// r.s8[i] = max(a.s8[i], b.s8[i])
NLIB_M(i64) I64::MaxInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi8(a, b);
#elif defined(NLIB_NEON)
    return vmax_s8(a, b);
#endif
}

// r.s16[i] = max(a.s16[i], b.s16[i])
NLIB_M(i64) I64::MaxInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmax, s16, a, b);
#endif
}

// r.s32[i] = max(a.s32[i], b.s32[i])
NLIB_M(i64) I64::MaxInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmax, s32, a, b);
#endif
}

// r.u8[i] = max(a.u8[i], b.u8[i])
NLIB_M(i64) I64::MaxUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmax, u8, a, b);
#endif
}

// r.u16[i] = max(a.u16[i], b.u16[i])
NLIB_M(i64) I64::MaxUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmax, u16, a, b);
#endif
}

// r.u32[i] = max(a.u32[i], b.u32[i])
NLIB_M(i64) I64::MaxUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmax, u32, a, b);
#endif
}

// r.s8[i] = min(a.s8[i], b.s8[i])
NLIB_M(i64) I64::MinInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi8(a, b);
#elif defined(NLIB_NEON)
    return vmin_s8(a, b);
#endif
}

// r.s16[i] = min(a.s16[i], b.s16[i])
NLIB_M(i64) I64::MinInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmin, s16, a, b);
#endif
}

// r.s32[i] = min(a.s32[i], b.s32[i])
NLIB_M(i64) I64::MinInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmin, s32, a, b);
#endif
}

// r.u8[i] = min(a.u8[i], b.u8[i])
NLIB_M(i64) I64::MinUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmin, u8, a, b);
#endif
}

// r.u16[i] = min(a.u16[i], b.u16[i])
NLIB_M(i64) I64::MinUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmin, u16, a, b);
#endif
}

// r.u32[i] = min(a.u32[i], b.u32[i])
NLIB_M(i64) I64::MinUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmin, u32, a, b);
#endif
}

// r.s8[i] = abs(value.s8[i])
NLIB_M(i64) I64::AbsInt8(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi8(value);
#elif defined(NLIB_NEON)
    return vabs_s8(value);
#endif
}

// r.s16[i] = abs(value.s16[i])
NLIB_M(i64) I64::AbsInt16(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi16(value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vabs, s16, value);
#endif
}

// r.s32[i] = abs(value.s32[i])
NLIB_M(i64) I64::AbsInt32(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi32(value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vabs, s32, value);
#endif
}

// r.s8[i] = abs(a.s8[i] - b.s8[i])
NLIB_M(i64) I64::AbsDiffInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi8(_mm_sub_epi8(a, b));
#elif defined(NLIB_NEON)
    return vabd_s8(a, b);
#endif
}

// r.s16[i] = abs(a.s16[i] - b.s16[i])
NLIB_M(i64) I64::AbsDiffInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi16(_mm_sub_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP2(vabd, s16, a, b);
#endif
}

// r.s32[i] = abs(a.s32[i] - b.s32[i])
NLIB_M(i64) I64::AbsDiffInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi32(_mm_sub_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP2(vabd, s32, a, b);
#endif
}

// r.s8[i] = -value.s8[i]
NLIB_M(i64) I64::NegateInt8(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi8(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return vneg_s8(value);
#endif
}

// r.s16[i] = -value.s16[i]
NLIB_M(i64) I64::NegateInt16(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vneg, s16, value);
#endif
}

// r.s32[i] = -value.s32[i]
NLIB_M(i64) I64::NegateInt32(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vneg, s32, value);
#endif
}

// r = a & b
NLIB_M(i64) I64::And(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_and_si128(a, b);
#elif defined(NLIB_NEON)
    return vand_s8(a, b);
#endif
}

// r = a | b
NLIB_M(i64) I64::Or(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(a, b);
#elif defined(NLIB_NEON)
    return vorr_s8(a, b);
#endif
}

// r = a ^ b
NLIB_M(i64) I64::Xor(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_xor_si128(a, b);
#elif defined(NLIB_NEON)
    return veor_s8(a, b);
#endif
}

// r = ~a
NLIB_M(i64) I64::Not(i64arg a) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_andnot_si128(a, _mm_cmpeq_epi8(a, a));
#elif defined(NLIB_NEON)
    return vmvn_s8(a);
#endif
}

// r = ~a & b
NLIB_M(i64) I64::AndNot(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_andnot_si128(a, b);
#elif defined(NLIB_NEON)
    return vbic_s8(b, a);
#endif
}

// r = ~a | b
NLIB_M(i64) I64::OrNot(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i full = _mm_cmpeq_epi8(a, a);
    return _mm_or_si128(_mm_andnot_si128(a, full), b);
#elif defined(NLIB_NEON)
    return vorn_s8(b, a);
#endif
}

// r.i8[i] = a.i8[i] == a.i8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpEq8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(a, b);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u8(vceq_s8(a, b));
#endif
}

// r.i16[i] = a.i16[i] == a.i16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpEq16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vceq, s16, a, b, u16);
#endif
}

// r.i32[i] = a.i32[i] == a.i32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpEq32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vceq, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] < a.s8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpLtInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] < a.s16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpLtInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] < a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpLtInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] > a.s8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpGtInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] > a.s16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpGtInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] > a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpGtInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, s32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] < a.u8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpLtUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_shuffle_epi8(_mm_cvtsi32_si128(0x80), _mm_setzero_si128());
    return _mm_cmplt_epi8(_mm_add_epi8(a, ofs), _mm_add_epi8(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, u8, a, b, u8);
#endif
}

// r.u8[i] = a.u8[i] > a.u8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpGtUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_shuffle_epi8(_mm_cvtsi32_si128(0x80), _mm_setzero_si128());
    return _mm_cmpgt_epi8(_mm_add_epi8(a, ofs), _mm_add_epi8(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] < a.u16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpLtUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_set1_epi16(INT16_MIN);
    return _mm_cmplt_epi16(_mm_add_epi16(a, ofs), _mm_add_epi16(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, u16, a, b, u16);
#endif
}

// r.u16[i] = a.u16[i] > a.u16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpGtUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_set1_epi16(INT16_MIN);
    return _mm_cmpgt_epi16(_mm_add_epi16(a, ofs), _mm_add_epi16(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] < a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpLtUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_set1_epi32(INT32_MIN);
    return _mm_cmplt_epi32(_mm_add_epi32(a, ofs), _mm_add_epi32(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vclt, u32, a, b, u32);
#endif
}

// r.u32[i] = a.u32[i] > a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpGtUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ofs = _mm_set1_epi32(INT32_MIN);
    return _mm_cmpgt_epi32(_mm_add_epi32(a, ofs), _mm_add_epi32(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgt, u32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] <= a.s8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpLeInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi8(a, b), _mm_cmpeq_epi8(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] <= a.s16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpLeInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi16(a, b), _mm_cmpeq_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] <= a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpLeInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi32(a, b), _mm_cmpeq_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] >= a.s8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpGeInt8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi8(a, b), _mm_cmpeq_epi8(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] >= a.s16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpGeInt16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi16(a, b), _mm_cmpeq_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] >= a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpGeInt32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi32(a, b), _mm_cmpeq_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, s32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] <= a.u8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpLeUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(_mm_min_epu8(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] <= a.u16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpLeUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(_mm_min_epu16(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] <= a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpLeUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(_mm_min_epu32(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcle, u32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] >= a.u8[i] ? 0xFF : 0
NLIB_M(i64) I64::CmpGeUint8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(_mm_max_epu8(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] >= a.u16[i] ? 0xFFFF : 0
NLIB_M(i64) I64::CmpGeUint16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(_mm_max_epu16(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] >= a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i64) I64::CmpGeUint32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(_mm_max_epu32(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcge, u32, a, b, u32);
#endif
}

// r.u16[i] = value.u16[i] << count
NLIB_M(i64) I64::ShiftLeftLogical16(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_slli_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, u16, value, count, s16);
#endif
}

// r.u16[i] = value.u16[i] >> count
NLIB_M(i64) I64::ShiftRightLogical16(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srli_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, u16, value, -count, s16);
#endif
}

// r.s16[i] = value.s16[i] >> count
NLIB_M(i64) I64::ShiftRightArithmetic16(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srai_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, s16, value, -count, s16);
#endif
}

// r.u32[i] = value.u32[i] << count
NLIB_M(i64) I64::ShiftLeftLogical32(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_slli_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, u32, value, count, s32);
#endif
}

// r.u32[i] = value.u32[i] >> count
NLIB_M(i64) I64::ShiftRightLogical32(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srli_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, u32, value, -count, s32);
#endif
}

// r.s32[i] = value.s32[i] >> count
NLIB_M(i64) I64::ShiftRightArithmetic32(i64arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srai_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshl, s32, value, -count, s32);
#endif
}

template <size_t N>
// r.i8[i + N] = r.i8[i], fill zero
NLIB_M(i64) I64::ByteShiftLeft(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return _mm_slli_epi64(value, N * 8);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u64(vshl_n_u64(vreinterpret_u64_s8(value), N * 8));
#endif
}

template <size_t N>
// r.i8[i - N] = r.i8[i], fill zero
NLIB_M(i64) I64::ByteShiftRight(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return _mm_srli_epi64(value, N * 8);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u64(vshr_n_u64(vreinterpret_u64_s8(value), N * 8));
#endif
}

template <size_t N>
// left <- 76543210 -> right
NLIB_M(i64) I64::ByteRotateRight(i64arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    i64 tmp = _mm_shuffle_epi32(value, _MM_SHUFFLE(1, 0, 1, 0));
    return _mm_alignr_epi8(tmp, tmp, N);
#elif defined(NLIB_NEON)
    return vext_s8(value, value, N);
#endif
}

// r.u8[i] = value.u8[i * 2]
NLIB_M(i64) I64::NarrowFrom16To8(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask[16] = {
        0, 2, 4, 6, 8, 10, 12, 14, -1, -1, -1, -1, -1, -1, -1, -1
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask[0]));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u8(vmovn_u16(vreinterpretq_u16_s8(value)));
#endif
}

// r.u16[i] = value.u16[i * 2]
NLIB_M(i64) I64::NarrowFrom32To16(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask[16] = {
        0, 1, 4, 5, 8, 9, 12, 13, -1, -1, -1, -1, -1, -1, -1, -1
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask[0]));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u16(vmovn_u32(vreinterpretq_u32_s8(value)));
#endif
}

// r.u32[i] = value.u32[i * 2]
NLIB_M(i64) I64::NarrowFrom64To32(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_shuffle_epi32(value, _MM_SHUFFLE(3, 1, 2, 0));
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u32(vmovn_u64(vreinterpretq_u64_s8(value)));
#endif
}

// r.u8[i] = 255 if value.u16[i] > 255
NLIB_M(i64) I64::ConvertFromUint16ToUint8Saturated(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i64 masked = _mm_and_si128(value, _mm_set1_epi16(0x7FFFU));
    return _mm_packus_epi16(masked, masked);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u8(vqmovn_u16(vreinterpretq_u16_s8(value)));
#endif
}

// r.s8[i] = 127 if value.s16[i] > 127, -128 if value.s16[i] < -128
NLIB_M(i64) I64::ConvertFromInt16ToInt8Saturated(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_packs_epi16(value, value);
#elif defined(NLIB_NEON)
    return vqmovn_s16(vreinterpretq_s16_s8(value));
#endif
}

// r.u16[i] = 65535 if value.u32[i] > 65535
NLIB_M(i64) I64::ConvertFromUint32ToUint16Saturated(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i64 masked = _mm_and_si128(value, _mm_set1_epi32(0x7FFFFFFFU));
    return _mm_packus_epi32(masked, masked);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_u16(vqmovn_u32(vreinterpretq_u32_s8(value)));
#endif
}

// r.s16[i] = 32767 if value.s32[i] > 32767, -32768 if value.s32[i] < -32768
NLIB_M(i64) I64::ConvertFromInt32ToInt16Saturated(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_packs_epi32(value, value);
#elif defined(NLIB_NEON)
    return vreinterpret_s8_s16(vqmovn_s32(vreinterpretq_s32_s8(value)));
#endif
}

// ab cd ef gh -> ba dc fe hg
NLIB_M(i64) I64::Reverse16(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(8) const int8_t mask_[8] = {
        1, 0, 3, 2, 5, 4, 7, 6
    };
    i128 mask = I64::LoadA8(&mask_[0]);
    return _mm_shuffle_epi8(value, mask);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev16, u8, value);
#endif
}

// abcd efgh -> dcba hgfe
NLIB_M(i64) I64::Reverse32(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(8) const int8_t mask_[8] = {
        3, 2, 1, 0, 7, 6, 5, 4
    };
    i128 mask = I64::LoadA8(&mask_[0]);
    return _mm_shuffle_epi8(value, mask);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev32, u8, value);
#endif
}

// abcdefgh -> hgfedcba
NLIB_M(i64) I64::Reverse64(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(8) const int8_t mask_[8] = {
        7, 6, 5, 4, 3, 2, 1, 0
    };
    i128 mask = I64::LoadA8(&mask_[0]);
    return _mm_shuffle_epi8(value, mask);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev64, u8, value);
#endif
}

// r = Or(ismasked(value.u8[i]) ? (1 << i) : 0)
NLIB_M(int) I64::MoveMask8(i64arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    return _mm_movemask_epi8(value) & 0xFF;
#elif defined(NLIB_NEON)
    NLIB_ALIGNAS(16) static const uint64_t powers_[1] = {0x8040201008040201ULL};
    uint8x8_t powers = vreinterpret_u8_u64(vld1_u64(powers_));
    uint8x8_t a = vand_u8(vreinterpret_u8_s8(value), powers);
    uint8x8_t tmp = vpadd_u8(a, a);
    tmp = vpadd_u8(tmp, tmp);
    tmp = vpadd_u8(tmp, tmp);
    return vget_lane_u8(tmp, 0);
#endif
}

// true if value == 0
NLIB_M(bool) I64::IsZero(i64arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    i64 tmp = _mm_shuffle_epi32(value, _MM_SHUFFLE(1, 0, 1, 0));
    return _mm_testz_si128(tmp, tmp) != 0;
#elif defined(NLIB_NEON)
    return vget_lane_u64(vreinterpret_u64_s8(value), 0) == 0;
#endif
}

// r.i8[i] = ismasked(mask.i8[i]) ? a.i8[i] : b.i8[i]
NLIB_M(i64) I64::Select(i64arg mask, i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_blendv_epi8(b, a, mask);
#elif defined(NLIB_NEON)
    return vbsl_s8(vreinterpret_u8_s8(mask), a, b);
#endif
}

#undef NLIB_M
#endif  // NLIB_DOXYGEN

struct NLIB_VIS_HIDDEN I128 {
    static i128 __vectorcall SetValue(int8_t v, each_int8_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(int16_t v, each_int16_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(int32_t v, each_int32_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(int64_t v, each_int64_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(uint8_t v, each_uint8_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(uint16_t v, each_uint16_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT;
    static i128 __vectorcall SetValue(uint64_t v, each_uint64_tag) NLIB_NOEXCEPT;

    template <size_t N>
    static i128 __vectorcall SetValue(i128 value, each_select32_tag) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetValue(i128 value, each_select16_tag) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetValue(i128 value, each_select8_tag) NLIB_NOEXCEPT;

    static i128 __vectorcall SetZero() NLIB_NOEXCEPT;
    static i128 __vectorcall SetFull(i128arg dummy) NLIB_NOEXCEPT;

    static i64 __vectorcall GetLo(i128 value) NLIB_NOEXCEPT;
    static i64 __vectorcall GetHi(i128 value) NLIB_NOEXCEPT;

    static i128 __vectorcall LoadA16(const void* p) NLIB_NOEXCEPT;
    static i128 __vectorcall LoadA8(const void* p) NLIB_NOEXCEPT;
    static i128 __vectorcall LoadA4(const void* p) NLIB_NOEXCEPT;
    static i128 __vectorcall LoadA2(const void* p) NLIB_NOEXCEPT;
    static i128 __vectorcall LoadA1(const void* p) NLIB_NOEXCEPT;
    static i128 __vectorcall LoadA16(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA16(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA8(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA8(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA4(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA4(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA2(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA2(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA1(uintptr_t p) NLIB_NOEXCEPT {
        return LoadA1(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA16(intptr_t p) NLIB_NOEXCEPT {
        return LoadA16(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA8(intptr_t p) NLIB_NOEXCEPT {
        return LoadA8(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA4(intptr_t p) NLIB_NOEXCEPT {
        return LoadA4(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA2(intptr_t p) NLIB_NOEXCEPT {
        return LoadA2(reinterpret_cast<void*>(p));
    }  // NOLINT
    static i128 __vectorcall LoadA1(intptr_t p) NLIB_NOEXCEPT {
        return LoadA1(reinterpret_cast<void*>(p));
    }  // NOLINT

    static void __vectorcall StoreA16(void* p, i128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA8(void* p, i128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA4(void* p, i128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA2(void* p, i128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA1(void* p, i128arg value) NLIB_NOEXCEPT;
    static void __vectorcall StoreA16(uintptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA16(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA8(uintptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA8(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA4(uintptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA4(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA2(uintptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA2(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA1(uintptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA1(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA16(intptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA16(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA8(intptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA8(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA4(intptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA4(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA2(intptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA2(reinterpret_cast<void*>(p), value);
    }  // NOLINT
    static void __vectorcall StoreA1(intptr_t p, i128arg value) NLIB_NOEXCEPT {
        StoreA1(reinterpret_cast<void*>(p), value);
    }  // NOLINT

    //
    // Get/Set
    //
    template <size_t N>
    static uint8_t __vectorcall GetUint8FromLane(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint16_t __vectorcall GetUint16FromLane(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint32_t __vectorcall GetUint32FromLane(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static uint64_t __vectorcall GetUint64FromLane(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetUint8ToLane(i128arg value, uint8_t v) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetUint16ToLane(i128arg value, uint16_t v) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetUint32ToLane(i128arg value, uint32_t v) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall SetUint64ToLane(i128arg value, uint64_t v) NLIB_NOEXCEPT;

    //
    // Arithmetic Operations
    //
    static i128 __vectorcall Add8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Add16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Add32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Add64(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall AddInt8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall AddInt16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall AddUint8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall AddUint16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall Sub8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Sub16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Sub32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Sub64(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall SubInt8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall SubInt16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall SubUint8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall SubUint16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall PairwiseAdd8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall PairwiseAdd16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall PairwiseAdd32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    // PairwiseMaxInt8, PairwiseMaxInt16, PairwiseMaxInt32
    // PairwiseMaxUint8, PairwiseMaxUint16, PairwiseMaxUint32
    // PairwiseMinInt8, PairwiseMinInt16, PairwiseMinInt32
    // PairwiseMinUint8, PairwiseMinUint16, PairwiseMinUint32

    static i128 __vectorcall Mult16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MultAdd16(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT;
    static i128 __vectorcall MultSub16(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT;
    static i128 __vectorcall Mult32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MultAdd32(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT;
    static i128 __vectorcall MultSub32(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT;

    static i128 __vectorcall NegateInt8(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall NegateInt16(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall NegateInt32(i128arg value) NLIB_NOEXCEPT;

    static i128 __vectorcall MaxInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MaxInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MaxInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MaxUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MaxUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MaxUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall MinUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall AbsInt8(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall AbsInt16(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall AbsInt32(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall AbsDiffInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall AbsDiffInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall AbsDiffInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    //
    // Logical Operations
    //
    static i128 __vectorcall And(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Or(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Xor(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Not(i128arg a) NLIB_NOEXCEPT;
    static i128 __vectorcall AndNot(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall OrNot(i128arg a, i128arg b) NLIB_NOEXCEPT;

    //
    // Comparison Operations
    //
    static i128 __vectorcall CmpEq8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpEq16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpEq32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpLtInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLtInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLtInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpGtInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGtInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGtInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpLtUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLtUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLtUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpGtUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGtUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGtUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpLeInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLeInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLeInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpGeInt8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGeInt16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGeInt32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpLeUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLeUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpLeUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    static i128 __vectorcall CmpGeUint8(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGeUint16(i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall CmpGeUint32(i128arg a, i128arg b) NLIB_NOEXCEPT;

    //
    // Bit Shift
    //
    static i128 __vectorcall ShiftLeftLogical8(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightLogical8(i128arg value, int count) NLIB_NOEXCEPT;

    static i128 __vectorcall ShiftLeftLogical16(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightLogical16(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightArithmetic16(i128arg value, int count) NLIB_NOEXCEPT;

    static i128 __vectorcall ShiftLeftLogical32(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightLogical32(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightArithmetic32(i128arg value, int count) NLIB_NOEXCEPT;

    static i128 __vectorcall ShiftLeftLogical64(i128arg value, int count) NLIB_NOEXCEPT;
    static i128 __vectorcall ShiftRightLogical64(i128arg value, int count) NLIB_NOEXCEPT;

    //
    // 128bit wide Byte Shift/Rotate
    //
    template <size_t N>
    static i128 __vectorcall ByteShiftLeft(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall ByteShiftRight(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall ByteRotateRight(i128arg value) NLIB_NOEXCEPT;
    template <size_t N>
    static i128 __vectorcall AlignR(i128arg a, i128arg b) NLIB_NOEXCEPT;

    //
    // Conversion
    //
    static i128 __vectorcall NarrowFrom16To8(i128arg lo, i128arg hi) NLIB_NOEXCEPT;
    static i128 __vectorcall NarrowFrom32To16(i128arg lo, i128arg hi) NLIB_NOEXCEPT;
    static i128 __vectorcall NarrowFrom64To32(i128arg lo, i128arg hi) NLIB_NOEXCEPT;

    static i128 __vectorcall
        ConvertFromUint16ToUint8Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT;  // NOLINT
    static i128 __vectorcall ConvertFromInt16ToInt8Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT;
    static i128 __vectorcall
        ConvertFromUint32ToUint16Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT;  // NOLINT
    static i128 __vectorcall
        ConvertFromInt32ToInt16Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT;  // NOLINT

    static i128 __vectorcall ConvertFromInt8ToInt16(i64arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertFromInt16ToInt32(i64arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertFromInt32ToInt64(i64arg value) NLIB_NOEXCEPT;

    static i128 __vectorcall ConvertFromUint8ToUint16(i64arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertFromUint16ToUint32(i64arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall ConvertFromUint32ToUint64(i64arg value) NLIB_NOEXCEPT;

    static i128 __vectorcall Zip8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Unzip8(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Zip16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Unzip16(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Zip32(i64arg a, i64arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Unzip32(i64arg a, i64arg b) NLIB_NOEXCEPT;

    //
    // Swap endian
    //
    static i128 __vectorcall Reverse16(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall Reverse32(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall Reverse64(i128arg value) NLIB_NOEXCEPT;

    //
    // Misc
    //
    static int __vectorcall MoveMask8(i128arg value) NLIB_NOEXCEPT;
    static int __vectorcall MoveMask16(i128arg value) NLIB_NOEXCEPT;
    static int __vectorcall MoveMask32(i128arg value) NLIB_NOEXCEPT;
    static bool __vectorcall IsZero(i128arg value) NLIB_NOEXCEPT;
    static bool __vectorcall IsFull(i128arg value) NLIB_NOEXCEPT;
    static i128 __vectorcall Select(i128arg mask, i128arg a, i128arg b) NLIB_NOEXCEPT;
    static i128 __vectorcall Shuffle8(i128arg value, i128arg shuffle) NLIB_NOEXCEPT;

 private:
    I128();  // forbidden
};

#ifndef NLIB_DOXYGEN

#define NLIB_M(tp) NLIB_ALWAYS_INLINE tp __vectorcall
#define NLIB_M2(tp) inline tp __vectorcall

#ifdef NLIB_NEON
# undef vreinterpret_s8_s8
# undef NLIB_OP1
# undef NLIB_OP2
# undef NLIB_OP3
# undef NLIB_CMP
# undef NLIB_SFT
# undef NLIB_CMB

#define vreinterpretq_s8_s8(a) (a)
#define NLIB_OP1(intrin, tp, a) \
    vreinterpretq_s8_##tp(intrin##_##tp(vreinterpretq_##tp##_s8(a)))
#define NLIB_OP2(intrin, tp, a, b) \
    vreinterpretq_s8_##tp(intrin##_##tp(vreinterpretq_##tp##_s8(a), \
                                        vreinterpretq_##tp##_s8(b)))
#define NLIB_OP3(intrin, tp, a, b, c) \
    vreinterpretq_s8_##tp(intrin##_##tp(vreinterpretq_##tp##_s8(a), \
                                        vreinterpretq_##tp##_s8(b), \
                                        vreinterpretq_##tp##_s8(c)))
#define NLIB_CMP(intrin, tp, a, b, utp) \
    vreinterpretq_s8_##utp(intrin##_##tp(vreinterpretq_##tp##_s8(a), \
                                         vreinterpretq_##tp##_s8(b)))
#define NLIB_SFT(intrin, tp, a, cnt, stp) \
    vreinterpretq_s8_##tp(intrin##_##tp(vreinterpretq_##tp##_s8(a), vdupq_n_##stp(cnt)))
#define NLIB_CMB(tp, l, h) vreinterpretq_s8_##tp(vcombine_##tp(l, h))
#endif

// r.s8[i] = v for all i_mm_cvtsi32_si128
NLIB_M(i128) I128::SetValue(int8_t v, each_int8_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // faster than _mm_set1_epi8(v)
    return _mm_shuffle_epi8(_mm_cvtsi32_si128(static_cast<uint8_t>(v)), _mm_setzero_si128());
#elif defined(NLIB_NEON)
    return vdupq_n_s8(v);
#endif
}

// r.s16[i] = v for all i
NLIB_M(i128) I128::SetValue(int16_t v, each_int16_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi16(v);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s16(vdupq_n_s16(v));
#endif
}

// r.s32[i] = v for all i
NLIB_M(i128) I128::SetValue(int32_t v, each_int32_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi32(v);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s32(vdupq_n_s32(v));
#endif
}

// r.s64[i] = v for all i
NLIB_M(i128) I128::SetValue(int64_t v, each_int64_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
#ifdef _MSC_VER
    // conflicts with ffloor()?
    // Do not use MMX anyway
    // return _mm_set1_epi64(*reinterpret_cast<__m64*>(&x));
    NLIB_ALIGNAS(16) int64_t tmp[2] = {v, v};
    return I128::LoadA16(tmp);
#else
    return _mm_set1_epi64x(v);
#endif
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s64(vdupq_n_s64(v));
#endif
}

// r.u8[i] = v for all i
NLIB_M(i128) I128::SetValue(uint8_t v, each_uint8_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // faster than _mm_set1_epi8(v)
    return _mm_shuffle_epi8(_mm_cvtsi32_si128(v), _mm_setzero_si128());
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u8(vdupq_n_u8(v));
#endif
}

// r.u16[i] = v for all i
NLIB_M(i128) I128::SetValue(uint16_t v, each_uint16_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi16(static_cast<int16_t>(v));
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u16(vdupq_n_u16(v));
#endif
}

// r.u32[i] = v for all i
NLIB_M(i128) I128::SetValue(uint32_t v, each_uint32_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_set1_epi32(static_cast<int32_t>(v));
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u32(vdupq_n_u32(v));
#endif
}

// r.u64[i] = v for all i
NLIB_M(i128) I128::SetValue(uint64_t v, each_uint64_tag) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
#ifdef _MSC_VER
    NLIB_ALIGNAS(16) uint64_t tmp[2] = {v, v};
    return I128::LoadA16(tmp);
#else
    return _mm_set1_epi64x(static_cast<int64_t>(v));
#endif
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u64(vdupq_n_u64(v));
#endif
}

#if defined(NLIB_SSE41)
template <size_t N>
// r.u32[i] = value.u32[N] for all i
NLIB_M(i128) I128::SetValue(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
    return _mm_shuffle_epi32(value, _MM_SHUFFLE(N, N, N, N));
}
#elif defined(NLIB_NEON)
# ifdef __aarch64__
template <size_t N>
// r.u32[i] = value.u32[N] for all i
NLIB_M(i128) I128::SetValue(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
    uint32x4_t v = vreinterpretq_u32_s8(value);
    return vreinterpretq_s8_u32(vdupq_laneq_u32(v, N));
}
# else
template <>
NLIB_M(i128) I128::SetValue<0>(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    uint32x2_t v = vget_low_u32(vreinterpretq_u32_s8(value));
    return vreinterpretq_s8_u32(vdupq_lane_u32(v, 0));
}
template <>
NLIB_M(i128) I128::SetValue<1>(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    uint32x2_t v = vget_low_u32(vreinterpretq_u32_s8(value));
    return vreinterpretq_s8_u32(vdupq_lane_u32(v, 1));
}
template <>
NLIB_M(i128) I128::SetValue<2>(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    uint32x2_t v = vget_high_u32(vreinterpretq_u32_s8(value));
    return vreinterpretq_s8_u32(vdupq_lane_u32(v, 0));
}
template <>
NLIB_M(i128) I128::SetValue<3>(i128 value, each_select32_tag) NLIB_NOEXCEPT {
    uint32x2_t v = vget_high_u32(vreinterpretq_u32_s8(value));
    return vreinterpretq_s8_u32(vdupq_lane_u32(v, 1));
}
# endif
#endif

#if defined(NLIB_SSE41)
template <size_t N>
// r.u16[i] = value.u16[N] for all i
NLIB_M2(i128) I128::SetValue(i128 value, each_select16_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
    NLIB_ALIGNAS(16) const int8_t mask[16] = {
        2 * N, 2 * N + 1, 2 * N, 2 * N + 1, 2 * N, 2 * N + 1, 2 * N, 2 * N + 1,
        2 * N, 2 * N + 1, 2 * N, 2 * N + 1, 2 * N, 2 * N + 1, 2 * N, 2 * N + 1
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(mask));
}
#elif defined(NLIB_NEON)
template <>
NLIB_M(i128) I128::SetValue<0>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 0));
# else
    uint16x4_t v = vget_low_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 0));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<1>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 1));
# else
    uint16x4_t v = vget_low_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 1));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<2>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 2));
# else
    uint16x4_t v = vget_low_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 2));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<3>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 3));
# else
    uint16x4_t v = vget_low_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 3));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<4>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 4));
# else
    uint16x4_t v = vget_high_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 0));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<5>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 5));
# else
    uint16x4_t v = vget_high_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 1));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<6>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 6));
# else
    uint16x4_t v = vget_high_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 2));
# endif
}

template <>
NLIB_M(i128) I128::SetValue<7>(i128 value, each_select16_tag) NLIB_NOEXCEPT {
# ifdef __aarch64__
    uint16x8_t v = vreinterpretq_u16_s8(value);
    return vreinterpretq_s8_u16(vdupq_laneq_u16(v, 7));
# else
    uint16x4_t v = vget_high_u16(vreinterpretq_u16_s8(value));
    return vreinterpretq_s8_u16(vdupq_lane_u16(v, 3));
# endif
}
#endif

#if defined(NLIB_SSE41)
template <size_t N>
// r.u8[i] = value.u8[N] for all i
NLIB_M2(i128) I128::SetValue(i128 value, each_select8_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
    NLIB_ALIGNAS(16) const int8_t mask[16] = {
        N, N, N, N, N, N, N, N, N, N, N, N, N, N, N, N
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask[0]));
}
#elif defined(NLIB_NEON)
namespace detail {
template <size_t N, bool IsLower>
struct SetValue8Helper {
    NLIB_M(i128) operator()(i128 value) NLIB_NOEXCEPT {
        return vdupq_lane_s8(vget_low_s8(value), N);
    }
};

template <size_t N>
struct SetValue8Helper<N, false> {
    NLIB_M(i128) operator()(i128 value) NLIB_NOEXCEPT {
        return vdupq_lane_s8(vget_high_s8(value), N - 8);
    }
};

}  // namespace detail

template <size_t N>
NLIB_M(i128) I128::SetValue(i128 value, each_select8_tag) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
# ifdef __aarch64__
    return vdupq_laneq_s8(value, N);
# else
    return detail::SetValue8Helper<N, (N < 8)>()(value);
# endif
}
#endif

// set 0 to all bits
NLIB_M(i128) I128::SetZero() NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_setzero_si128();
#elif defined(NLIB_NEON)
    return vdupq_n_s8(0);
#endif
}

// set 1 to all bits
NLIB_M(i128) I128::SetFull(i128arg dummy) NLIB_NOEXCEPT { return I128::CmpEq8(dummy, dummy); }

// r = value.u8[0-7]
NLIB_M(i64) I128::GetLo(i128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return value;
#elif defined(NLIB_NEON)
    return vget_low_s8(value);
#endif
}

// r = value.u8[8-15]
NLIB_M(i64) I128::GetHi(i128 value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_shuffle_epi32(value, _MM_SHUFFLE(1, 0, 3, 2));
#elif defined(NLIB_NEON)
    return vget_high_s8(value);
#endif
}

// r[i] = p[i], p is 16 bytes aligned
NLIB_M(i128) I128::LoadA16(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_load_si128(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    uint64x2_t tmp = vld1q_u64(reinterpret_cast<const uint64_t*>(p));
    return vreinterpretq_s8_u64(tmp);
#endif
}

// r[i] = p[i], p is 8 bytes aligned
NLIB_M(i128) I128::LoadA8(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadu_si128(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    uint64x2_t tmp = vld1q_u64(reinterpret_cast<const uint64_t*>(p));
    return vreinterpretq_s8_u64(tmp);
#endif
}

// r[i] = p[i], p is 4 bytes aligned
NLIB_M(i128) I128::LoadA4(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadu_si128(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    uint32x4_t tmp = vld1q_u32(reinterpret_cast<const uint32_t*>(p));
    return vreinterpretq_s8_u32(tmp);
#endif
}

// r[i] = p[i], p is 2 bytes aligned
NLIB_M(i128) I128::LoadA2(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadu_si128(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    uint16x8_t tmp = vld1q_u16(reinterpret_cast<const uint16_t*>(p));
    return vreinterpretq_s8_u16(tmp);
#endif
}

// r[i] = p[i]
NLIB_M(i128) I128::LoadA1(const void* p) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_loadu_si128(reinterpret_cast<const __m128i*>(p));
#elif defined(NLIB_NEON)
    return vld1q_s8(reinterpret_cast<const int8_t*>(p));
#endif
}

// p[i] = value[i], p is 16 bytes aligned
NLIB_M(void) I128::StoreA16(void* p, i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_store_si128(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1q_u64(reinterpret_cast<uint64_t*>(p), vreinterpretq_u64_s8(value));
#endif
}

// p[i] = value[i], p is 8 bytes aligned
NLIB_M(void) I128::StoreA8(void* p, i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storeu_si128(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1q_u64(reinterpret_cast<uint64_t*>(p), vreinterpretq_u64_s8(value));
#endif
}

// p[i] = value[i], p is 4 bytes aligned
NLIB_M(void) I128::StoreA4(void* p, i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storeu_si128(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1q_u32(reinterpret_cast<uint32_t*>(p), vreinterpretq_u32_s8(value));
#endif
}

// p[i] = value[i], p is 2 bytes aligned
NLIB_M(void) I128::StoreA2(void* p, i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storeu_si128(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1q_u16(reinterpret_cast<uint16_t*>(p), vreinterpretq_u16_s8(value));
#endif
}

// p[i] = value[i]
NLIB_M(void) I128::StoreA1(void* p, i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    _mm_storeu_si128(reinterpret_cast<i128*>(p), value);
#elif defined(NLIB_NEON)
    vst1q_s8(reinterpret_cast<int8_t*>(p), value);
#endif
}

template <size_t N>
// r = value.u8[N]
NLIB_M(uint8_t) I128::GetUint8FromLane(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return static_cast<uint8_t>(_mm_extract_epi8(value, N));
#elif defined(NLIB_NEON)
    return vgetq_lane_u8(vreinterpretq_u8_s8(value), N);
#endif
}

template <size_t N>
// r = value.u16[N]
NLIB_M(uint16_t) I128::GetUint16FromLane(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return static_cast<uint16_t>(_mm_extract_epi16(value, N));
#elif defined(NLIB_NEON)
    return vgetq_lane_u16(vreinterpretq_u16_s8(value), N);
#endif
}

template <size_t N>
// r = value.u32[N]
NLIB_M(uint32_t) I128::GetUint32FromLane(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_SSE41)
    return static_cast<uint32_t>(_mm_extract_epi32(value, N));
#elif defined(NLIB_NEON)
    return vgetq_lane_u32(vreinterpretq_u32_s8(value), N);
#endif
}

template <size_t N>
// r = value.u64[N]
NLIB_M(uint64_t) I128::GetUint64FromLane(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 2);
#if defined(NLIB_SSE41)
#ifdef NLIB_64BIT
    return static_cast<uint64_t>(_mm_extract_epi64(value, N));
#else
    NLIB_UNUSED(value);
    return 0;  // dummy
#endif
#elif defined(NLIB_NEON)
    return vgetq_lane_u64(vreinterpretq_u64_s8(value), N);
#endif
}

#if defined(NLIB_SSE41) && !defined(NLIB_64BIT)
template <>
NLIB_M(uint64_t) I128::GetUint64FromLane<0>(i128arg value) NLIB_NOEXCEPT {
    uint64_t rval;
    _mm_storel_epi64(reinterpret_cast<i128*>(&rval), value);
    return rval;
}
template <>
NLIB_M(uint64_t) I128::GetUint64FromLane<1>(i128arg value) NLIB_NOEXCEPT {
    uint64_t rval;
    i128 tmp = I128::GetHi(value);
    _mm_storel_epi64(reinterpret_cast<i128*>(&rval), tmp);
    return rval;
}
#endif

template <size_t N>
// r = value, r.u8[N] = v
NLIB_M(i128) I128::SetUint8ToLane(i128arg value, uint8_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return _mm_insert_epi8(value, static_cast<int8_t>(v), N);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u8(vsetq_lane_u8(v, vreinterpretq_u8_s8(value), N));
#endif
}

template <size_t N>
// r = value, r.u16[N] = v
NLIB_M(i128) I128::SetUint16ToLane(i128arg value, uint16_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 8);
#if defined(NLIB_SSE41)
    return _mm_insert_epi16(value, static_cast<int16_t>(v), N);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u16(vsetq_lane_u16(v, vreinterpretq_u16_s8(value), N));
#endif
}

template <size_t N>
// r = value, r.u32[N] = v
NLIB_M(i128) I128::SetUint32ToLane(i128arg value, uint32_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 4);
#if defined(NLIB_SSE41)
    return _mm_insert_epi32(value, static_cast<uint32_t>(v), N);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u32(vsetq_lane_u32(v, vreinterpretq_u32_s8(value), N));
#endif
}

template <size_t N>
// r = value, r.u64[N] = v
NLIB_M(i128) I128::SetUint64ToLane(i128arg value, uint64_t v) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 2);
#if defined(NLIB_SSE41)
#ifdef NLIB_64BIT
    return _mm_insert_epi64(value, static_cast<int64_t>(v), N);
#else
    union {
        int32_t i32[2];
        int64_t i64;
    } tmp;
    tmp.i64 = static_cast<int64_t>(v);
    __m128i rval;
    rval = _mm_insert_epi32(value, tmp.i32[0], N * 2 + 0);
    return _mm_insert_epi32(rval, tmp.i32[1], N * 2 + 1);
#endif
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u64(vsetq_lane_u64(v, vreinterpretq_u64_s8(value), N));
#endif
}

// r.i8[i] = a.i8[i] + b.i8[i]
NLIB_M(i128) I128::Add8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi8(a, b);
#elif defined(NLIB_NEON)
    return vaddq_s8(a, b);
#endif
}

// r.i16[i] = a.i16[i] + b.i16[i]
NLIB_M(i128) I128::Add16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vaddq, s16, a, b);
#endif
}

// r.i32[i] = a.i32[i] + b.i32[i]
NLIB_M(i128) I128::Add32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vaddq, s32, a, b);
#endif
}

// r.i64[i] = a.i64[i] + b.i64[i]
NLIB_M(i128) I128::Add64(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi64(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vaddq, s64, a, b);
#endif
}

// 127 if a[i] + b[i] > 127, -128 if a[i] + b[i] < -128
NLIB_M(i128) I128::AddInt8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epi8(a, b);
#elif defined(NLIB_NEON)
    return vqaddq_s8(a, b);
#endif
}

// 32767 if a[i] + b[i] > 32767, -32768 if a[i] + b[i] < -32768
NLIB_M(i128) I128::AddInt16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqaddq, s16, a, b);
#endif
}

// 255 if a[i] + b[i] > 255
NLIB_M(i128) I128::AddUint8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqaddq, u8, a, b);
#endif
}

// 65535 if a[i] + b[i] > 65535
NLIB_M(i128) I128::AddUint16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_adds_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqaddq, u16, a, b);
#endif
}

// r.i8[i] = a.i8[i] - b.i8[i]
NLIB_M(i128) I128::Sub8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi8(a, b);
#elif defined(NLIB_NEON)
    return vsubq_s8(a, b);
#endif
}

// r.i16[i] = a.i16[i] - b.i16[i]
NLIB_M(i128) I128::Sub16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsubq, s16, a, b);
#endif
}

// r.i32[i] = a.i32[i] - b.i32[i]
NLIB_M(i128) I128::Sub32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsubq, s32, a, b);
#endif
}

// r.i64[0] = a.i64[0] - b.i64[0]
NLIB_M(i128) I128::Sub64(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi64(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vsubq, s64, a, b);
#endif
}

// 127 if a[i] - b[i] > 127, -128 if a[i] - b[i] < -128
NLIB_M(i128) I128::SubInt8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsubq, s8, a, b);
#endif
}

// 32767 if a[i] - b[i] > 32767, -32768 if a[i] - b[i] < -32768
NLIB_M(i128) I128::SubInt16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsubq, s16, a, b);
#endif
}

// 0 if a.u8[i] - b.u8[i] < 0
NLIB_M(i128) I128::SubUint8Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsubq, u8, a, b);
#endif
}

// 0 if a.u16[i] - b.u16[i] < 0
NLIB_M(i128) I128::SubUint16Saturated(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_subs_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vqsubq, u16, a, b);
#endif
}

// r.i8[0] = value[0] + value[1], ..., r.i8[7] = value[14] + value[15]
NLIB_M(i128) I128::PairwiseAdd8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i ax = _mm_add_epi8(a, _mm_srli_epi16(a, 8));
    __m128i bx = _mm_add_epi8(b, _mm_srli_epi16(b, 8));
    return I128::NarrowFrom16To8(ax, bx);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vpaddq_s8(a, b);
# else
    int8x8_t al = vget_low_s8(a);
    int8x8_t ah = vget_high_s8(a);
    int8x8_t rl = vpadd_s8(al, ah);
    int8x8_t bl = vget_low_s8(b);
    int8x8_t bh = vget_high_s8(b);
    int8x8_t rh = vpadd_s8(bl, bh);
    return vcombine_s8(rl, rh);
# endif
#endif
}

// r.i16[0] = value[0] + value[1], ..., r.i16[3] = value[6] + value[7]
NLIB_M(i128) I128::PairwiseAdd16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_hadd_epi16(a, b);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vreinterpretq_s8_s16(vpaddq_s16(vreinterpretq_s16_s8(a), vreinterpretq_s16_s8(b)));
# else
    int16x4_t al = vget_low_s16(vreinterpretq_s16_s8(a));
    int16x4_t ah = vget_high_s16(vreinterpretq_s16_s8(a));
    int16x4_t rl = vpadd_s16(al, ah);
    int16x4_t bl = vget_low_s16(vreinterpretq_s16_s8(b));
    int16x4_t bh = vget_high_s16(vreinterpretq_s16_s8(b));
    int16x4_t rh = vpadd_s16(bl, bh);
    return NLIB_CMB(s16, rl, rh);
# endif
#endif
}

// r.i32[0] = value[0] + value[1], r.i32[1] = value[2] + value[3]
NLIB_M(i128) I128::PairwiseAdd32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_hadd_epi32(a, b);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vreinterpretq_s8_s32(vpaddq_s32(vreinterpretq_s32_s8(a), vreinterpretq_s32_s8(b)));
# else
    int32x2_t al = vget_low_s32(vreinterpretq_s32_s8(a));
    int32x2_t ah = vget_high_s32(vreinterpretq_s32_s8(a));
    int32x2_t rl = vpadd_s32(al, ah);
    int32x2_t bl = vget_low_s32(vreinterpretq_s32_s8(b));
    int32x2_t bh = vget_high_s32(vreinterpretq_s32_s8(b));
    int32x2_t rh = vpadd_s32(bl, bh);
    return NLIB_CMB(s32, rl, rh);
# endif
#endif
}

// r.i16[i] = a.i16[i] * b.i16[i]
NLIB_M(i128) I128::Mult16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_mullo_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmulq, s16, a, b);
#endif
}

// r = c + a * b
NLIB_M(i128) I128::MultAdd16(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi16(c, _mm_mullo_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmlaq, s16, c, a, b);
#endif
}

// r = c - a * b
NLIB_M(i128) I128::MultSub16(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(c, _mm_mullo_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmlsq, s16, c, a, b);
#endif
}

// r.i32[i] = a.i32[i] * b.i32[i]
NLIB_M(i128) I128::Mult32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_mullo_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmulq, s32, a, b);
#endif
}

// r = c + a * b
NLIB_M(i128) I128::MultAdd32(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_add_epi32(c, _mm_mullo_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmlaq, s32, c, a, b);
#endif
}

// r = c - a * b
NLIB_M(i128) I128::MultSub32(i128arg a, i128arg b, i128arg c) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(c, _mm_mullo_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP3(vmlsq, s32, c, a, b);
#endif
}

// r.s8[i] = max(a.s8[i], b.s8[i])
NLIB_M(i128) I128::MaxInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, s8, a, b);
#endif
}

// r.s16[i] = max(a.s16[i], b.s16[i])
NLIB_M(i128) I128::MaxInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, s16, a, b);
#endif
}

// r.s32[i] = max(a.s32[i], b.s32[i])
NLIB_M(i128) I128::MaxInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, s32, a, b);
#endif
}

// r.u8[i] = max(a.u8[i], b.u8[i])
NLIB_M(i128) I128::MaxUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, u8, a, b);
#endif
}

// r.u16[i] = max(a.u16[i], b.u16[i])
NLIB_M(i128) I128::MaxUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, u16, a, b);
#endif
}

// r.u32[i] = max(a.u32[i], b.u32[i])
NLIB_M(i128) I128::MaxUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_max_epu32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vmaxq, u32, a, b);
#endif
}

// r.s8[i] = min(a.s8[i], b.s8[i])
NLIB_M(i128) I128::MinInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, s8, a, b);
#endif
}

// r.s16[i] = min(a.s16[i], b.s16[i])
NLIB_M(i128) I128::MinInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, s16, a, b);
#endif
}

// r.s32[i] = min(a.s32[i], b.s32[i])
NLIB_M(i128) I128::MinInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, s32, a, b);
#endif
}

// r.u8[i] = min(a.u8[i], b.u8[i])
NLIB_M(i128) I128::MinUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, u8, a, b);
#endif
}

// r.u16[i] = min(a.u16[i], b.u16[i])
NLIB_M(i128) I128::MinUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, u16, a, b);
#endif
}

// r.u32[i] = min(a.u32[i], b.u32[i])
NLIB_M(i128) I128::MinUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_min_epu32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vminq, u32, a, b);
#endif
}

// r.s8[i] = abs(value.s8[i])
NLIB_M(i128) I128::AbsInt8(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi8(value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vabsq, s8, value);
#endif
}

// r.s16[i] = abs(value.s16[i])
NLIB_M(i128) I128::AbsInt16(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi16(value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vabsq, s16, value);
#endif
}

// r.s32[i] = abs(value.s32[i])
NLIB_M(i128) I128::AbsInt32(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi32(value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vabsq, s32, value);
#endif
}

// r.s8[i] = abs(a.s8[i] - b.s8[i])
NLIB_M(i128) I128::AbsDiffInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi8(_mm_sub_epi8(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP2(vabdq, s8, a, b);
#endif
}

// r.s16[i] = abs(a.s16[i] - b.s16[i])
NLIB_M(i128) I128::AbsDiffInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi16(_mm_sub_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP2(vabdq, s16, a, b);
#endif
}

// r.s32[i] = abs(a.s32[i] - b.s32[i])
NLIB_M(i128) I128::AbsDiffInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_abs_epi32(_mm_sub_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_OP2(vabdq, s32, a, b);
#endif
}

// r.s8[i] = -value.s8[i]
NLIB_M(i128) I128::NegateInt8(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi8(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vnegq, s8, value);
#endif
}

// r.s16[i] = -value.s16[i]
NLIB_M(i128) I128::NegateInt16(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi16(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vnegq, s16, value);
#endif
}

// r.s32[i] = -value.s32[i]
NLIB_M(i128) I128::NegateInt32(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_sub_epi32(_mm_setzero_si128(), value);
#elif defined(NLIB_NEON)
    return NLIB_OP1(vnegq, s32, value);
#endif
}

// r = a & b
NLIB_M(i128) I128::And(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_and_si128(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vandq, s8, a, b);
#endif
}

// r = a | b
NLIB_M(i128) I128::Or(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vorrq, s8, a, b);
#endif
}

// r = a ^ b
NLIB_M(i128) I128::Xor(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_xor_si128(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(veorq, s8, a, b);
#endif
}

// r = ~a
NLIB_M(i128) I128::Not(i128arg a) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_andnot_si128(a, _mm_cmpeq_epi8(a, a));
#elif defined(NLIB_NEON)
    return NLIB_OP1(vmvnq, s8, a);
#endif
}

// r = ~a & b
NLIB_M(i128) I128::AndNot(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_andnot_si128(a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vbicq, s8, b, a);
#endif
}

// r = ~a | b
NLIB_M(i128) I128::OrNot(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i not_a = _mm_andnot_si128(a, _mm_cmpeq_epi8(a, a));
    return _mm_or_si128(not_a, b);
#elif defined(NLIB_NEON)
    return NLIB_OP2(vornq, s8, b, a);
#endif
}

// r.i8[i] = a.i8[i] == a.i8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpEq8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vceqq, s8, a, b, u8);
#endif
}

// r.i16[i] = a.i16[i] == a.i16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpEq16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vceqq, s16, a, b, u16);
#endif
}

// r.i32[i] = a.i32[i] == a.i32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpEq32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vceqq, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] < a.s8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpLtInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] < a.s16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpLtInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] < a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpLtInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmplt_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] > a.s8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpGtInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi8(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] > a.s16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpGtInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi16(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] > a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpGtInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpgt_epi32(a, b);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, s32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] < a.u8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpLtUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x80, each_uint8);
    return _mm_cmplt_epi8(_mm_add_epi8(a, ofs), _mm_add_epi8(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, u8, a, b, u8);
#endif
}

// r.u8[i] = a.u8[i] > a.u8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpGtUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x80, each_uint8);
    return _mm_cmpgt_epi8(_mm_add_epi8(a, ofs), _mm_add_epi8(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] < a.u16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpLtUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x8000U, each_uint16);
    return _mm_cmplt_epi16(_mm_add_epi16(a, ofs), _mm_add_epi16(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, u16, a, b, u16);
#endif
}

// r.u16[i] = a.u16[i] > a.u16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpGtUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x8000U, each_uint16);
    return _mm_cmpgt_epi16(_mm_add_epi16(a, ofs), _mm_add_epi16(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] < a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpLtUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x80000000U, each_uint32);
    return _mm_cmplt_epi32(_mm_add_epi32(a, ofs), _mm_add_epi32(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcltq, u32, a, b, u32);
#endif
}

// r.u32[i] = a.u32[i] > a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpGtUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 ofs = I128::SetValue(0x80000000U, each_uint32);
    return _mm_cmpgt_epi32(_mm_add_epi32(a, ofs), _mm_add_epi32(b, ofs));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgtq, u32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] <= a.s8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpLeInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi8(a, b), _mm_cmpeq_epi8(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] <= a.s16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpLeInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi16(a, b), _mm_cmpeq_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] <= a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpLeInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmplt_epi32(a, b), _mm_cmpeq_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, s32, a, b, u32);
#endif
}

// r.s8[i] = a.s8[i] >= a.s8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpGeInt8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi8(a, b), _mm_cmpeq_epi8(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, s8, a, b, u8);
#endif
}

// r.s16[i] = a.s16[i] >= a.s16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpGeInt16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi16(a, b), _mm_cmpeq_epi16(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, s16, a, b, u16);
#endif
}

// r.s32[i] = a.s32[i] >= a.s32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpGeInt32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_or_si128(_mm_cmpgt_epi32(a, b), _mm_cmpeq_epi32(a, b));
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, s32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] <= a.u8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpLeUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(_mm_min_epu8(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] <= a.u16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpLeUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(_mm_min_epu16(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] <= a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpLeUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(_mm_min_epu32(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcleq, u32, a, b, u32);
#endif
}

// r.u8[i] = a.u8[i] >= a.u8[i] ? 0xFF : 0
NLIB_M(i128) I128::CmpGeUint8(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi8(_mm_max_epu8(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, u8, a, b, u8);
#endif
}

// r.u16[i] = a.u16[i] >= a.u16[i] ? 0xFFFF : 0
NLIB_M(i128) I128::CmpGeUint16(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi16(_mm_max_epu16(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, u16, a, b, u16);
#endif
}

// r.u32[i] = a.u32[i] >= a.u32[i] ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::CmpGeUint32(i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cmpeq_epi32(_mm_max_epu32(a, b), a);
#elif defined(NLIB_NEON)
    return NLIB_CMP(vcgeq, u32, a, b, u32);
#endif
}

// r.u8[i] = value.u8[i] << count
NLIB_M(i128) I128::ShiftLeftLogical8(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i hi = I128::GetHi(value);
    __m128i xh = _mm_slli_epi16(_mm_cvtepu8_epi16(hi), count);
    __m128i xl = _mm_slli_epi16(_mm_cvtepu8_epi16(value), count);
    return I128::NarrowFrom16To8(xl, xh);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u8, value, count, s8);
#endif
}

// r.u8[i] = value.u8[i] >> count
NLIB_M(i128) I128::ShiftRightLogical8(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i hi = I128::GetHi(value);
    __m128i xh = _mm_srli_epi16(_mm_cvtepu8_epi16(hi), count);
    __m128i xl = _mm_srli_epi16(_mm_cvtepu8_epi16(value), count);
    return _mm_packus_epi16(xl, xh);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u8, value, -count, s8);
#endif
}

// r.u16[i] = value.u16[i] << count
NLIB_M(i128) I128::ShiftLeftLogical16(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_slli_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u16, value, count, s16);
#endif
}

// r.u16[i] = value.u16[i] >> count
NLIB_M(i128) I128::ShiftRightLogical16(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srli_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u16, value, -count, s16);
#endif
}

// r.s16[i] = value.s16[i] >> count
NLIB_M(i128) I128::ShiftRightArithmetic16(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srai_epi16(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, s16, value, -count, s16);
#endif
}

// r.u32[i] = value.u32[i] << count
NLIB_M(i128) I128::ShiftLeftLogical32(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_slli_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u32, value, count, s32);
#endif
}

// r.u32[i] = value.u32[i] >> count
NLIB_M(i128) I128::ShiftRightLogical32(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srli_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u32, value, -count, s32);
#endif
}

// r.s32[i] = value.s32[i] >> count
NLIB_M(i128) I128::ShiftRightArithmetic32(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srai_epi32(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, s32, value, -count, s32);
#endif
}

// r.u32[i] = value.u64[i] << count
NLIB_M(i128) I128::ShiftLeftLogical64(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_slli_epi64(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u64, value, count, s64);
#endif
}

// r.u32[i] = value.u64[i] >> count
NLIB_M(i128) I128::ShiftRightLogical64(i128arg value, int count) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_srli_epi64(value, count);
#elif defined(NLIB_NEON)
    return NLIB_SFT(vshlq, u64, value, -count, s64);
#endif
}

template <size_t N>
// r.i8[i + N] = r.i8[i], fill zero
NLIB_M(i128) I128::ByteShiftLeft(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return _mm_slli_si128(value, N);
#elif defined(NLIB_NEON)
    return vextq_s8(vdupq_n_s8(0), value, 16 - N);
#endif
}

template <size_t N>
// r.i8[i - N] = r.i8[i], fill zero
NLIB_M(i128) I128::ByteShiftRight(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return _mm_srli_si128(value, N);
#elif defined(NLIB_NEON)
    return vextq_s8(value, vdupq_n_s8(0), N);
#endif
}

template <size_t N>
// left <- fedcba9876543210 -> right
NLIB_M(i128) I128::ByteRotateRight(i128arg value) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return _mm_alignr_epi8(value, value, N);
#elif defined(NLIB_NEON)
    return vextq_s8(value, value, N);
#endif
}

template <size_t N>
// _mm_alignr_epi8(a, b, N), which returns ((a << 128) | b) >> (N * 8)
NLIB_M(i128) I128::AlignR(i128arg a, i128arg b) NLIB_NOEXCEPT {
    NLIB_STATIC_ASSERT(N < 16);
#if defined(NLIB_SSE41)
    return _mm_alignr_epi8(a, b, N);
#elif defined(NLIB_NEON)
    return vextq_s8(b, a, N);
#endif
}

// r.u8[i] = value.u8[i * 2]
NLIB_M(i128) I128::NarrowFrom16To8(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 mask = I128::SetValue(0x00FFU, each_uint16);
    __m128i lo_mask = _mm_and_si128(lo, mask);
    __m128i hi_mask = _mm_and_si128(hi, mask);
    return _mm_packus_epi16(lo_mask, hi_mask);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint8x8_t l = vmovn_u16(vreinterpretq_u16_s8(lo));
    return vreinterpretq_s8_u8(vmovn_high_u16(l, vreinterpretq_u16_s8(hi)));
# else
    uint8x8_t l = vmovn_u16(vreinterpretq_u16_s8(lo));
    uint8x8_t h = vmovn_u16(vreinterpretq_u16_s8(hi));
    return NLIB_CMB(u8, l, h);
# endif
#endif
}

// r.u16[i] = value.u16[i * 2]
NLIB_M(i128) I128::NarrowFrom32To16(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 mask = I128::SetValue(0xFFFFU, each_uint32);
    __m128i lo_mask = _mm_and_si128(lo, mask);
    __m128i hi_mask = _mm_and_si128(hi, mask);
    return _mm_packus_epi32(lo_mask, hi_mask);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint16x4_t l = vmovn_u32(vreinterpretq_u32_s8(lo));
    return vreinterpretq_s8_u16(vmovn_high_u32(l, vreinterpretq_u32_s8(hi)));
# else
    uint16x4_t l = vmovn_u32(vreinterpretq_u32_s8(lo));
    uint16x4_t h = vmovn_u32(vreinterpretq_u32_s8(hi));
    return NLIB_CMB(u16, l, h);
# endif
#endif
}

// r.u32[i] = value.u32[i * 2]
NLIB_M(i128) I128::NarrowFrom64To32(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    __m128i lo_ = _mm_shuffle_epi32(lo, _MM_SHUFFLE(3, 1, 2, 0));
    __m128i hi_ = _mm_shuffle_epi32(hi, _MM_SHUFFLE(3, 1, 2, 0));
    return _mm_unpacklo_epi64(lo_, hi_);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint32x2_t l = vmovn_u64(vreinterpretq_u64_s8(lo));
    return vreinterpretq_s8_u32(vmovn_high_u64(l, vreinterpretq_u64_s8(hi)));
# else
    uint32x2_t l = vmovn_u64(vreinterpretq_u64_s8(lo));
    uint32x2_t h = vmovn_u64(vreinterpretq_u64_s8(hi));
    return NLIB_CMB(u32, l, h);
# endif
#endif
}

// r.u8[i] = 255 if value.u16[i] > 255
NLIB_M(i128) I128::ConvertFromUint16ToUint8Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 b7FFF = I128::SetValue(0x7FFFU, each_uint16);
    __m128i lotmp = _mm_and_si128(lo, b7FFF);
    __m128i hitmp = _mm_and_si128(hi, b7FFF);
    return _mm_packus_epi16(lotmp, hitmp);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint8x8_t l = vqmovn_u16(vreinterpretq_u16_s8(lo));
    return vreinterpretq_s8_u8(vqmovn_high_u16(l, vreinterpretq_u16_s8(hi)));
# else
    uint8x8_t l = vqmovn_u16(vreinterpretq_u16_s8(lo));
    uint8x8_t h = vqmovn_u16(vreinterpretq_u16_s8(hi));
    return NLIB_CMB(u8, l, h);
# endif
#endif
}

// r.s8[i] = 127 if value.s16[i] > 127, -128 if value.s16[i] < -128
NLIB_M(i128) I128::ConvertFromInt16ToInt8Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_packs_epi16(lo, hi);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    int8x8_t l = vqmovn_s16(vreinterpretq_s16_s8(lo));
    return vqmovn_high_s16(l, vreinterpretq_s16_s8(hi));
# else
    int8x8_t l = vqmovn_s16(vreinterpretq_s16_s8(lo));
    int8x8_t h = vqmovn_s16(vreinterpretq_s16_s8(hi));
    return NLIB_CMB(s8, l, h);
# endif
#endif
}

// r.u16[i] = 65535 if value.u32[i] > 65535
NLIB_M(i128) I128::ConvertFromUint32ToUint16Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    i128 b7FFFFFFF = I128::SetValue(0x7FFFFFFFU, each_uint32);
    __m128i lotmp = _mm_and_si128(lo, b7FFFFFFF);
    __m128i hitmp = _mm_and_si128(hi, b7FFFFFFF);
    return _mm_packus_epi32(lotmp, hitmp);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    uint16x4_t l = vqmovn_u32(vreinterpretq_u32_s8(lo));
    return vreinterpretq_s8_u16(vqmovn_high_u32(l, vreinterpretq_u32_s8(hi)));
# else
    uint16x4_t l = vqmovn_u32(vreinterpretq_u32_s8(lo));
    uint16x4_t h = vqmovn_u32(vreinterpretq_u32_s8(hi));
    return NLIB_CMB(u16, l, h);
# endif
#endif
}

// r.s16[i] = 32767 if value.s32[i] > 32767, -32768 if value.s32[i] < -32768
NLIB_M(i128) I128::ConvertFromInt32ToInt16Saturated(i128arg lo, i128arg hi) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_packs_epi32(lo, hi);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    int16x4_t l = vqmovn_s32(vreinterpretq_s32_s8(lo));
    return vreinterpretq_s8_s16(vqmovn_high_s32(l, vreinterpretq_s32_s8(hi)));
# else
    int16x4_t l = vqmovn_s32(vreinterpretq_s32_s8(lo));
    int16x4_t h = vqmovn_s32(vreinterpretq_s32_s8(hi));
    return NLIB_CMB(s16, l, h);
# endif
#endif
}

// r.s8[2 * i] = value.s8[i], r.u8[2 * i + 1] = value.s8[i] < 0 ? 0xFF : 0
NLIB_M(i128) I128::ConvertFromInt8ToInt16(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepi8_epi16(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s16(vmovl_s8(value));
#endif
}

// r.s16[2 * i] = value.s16[i], r.u16[2 * i + 1] = value.s16[i] < 0 ? 0xFFFF : 0
NLIB_M(i128) I128::ConvertFromInt16ToInt32(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepi16_epi32(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s32(vmovl_s16(vreinterpret_s16_s8(value)));
#endif
}

// r.s32[2 * i] = value.s32[i], r.u32[2 * i + 1] = value.s32[i] < 0 ? 0xFFFFFFFF : 0
NLIB_M(i128) I128::ConvertFromInt32ToInt64(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepi32_epi64(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_s64(vmovl_s32(vreinterpret_s32_s8(value)));
#endif
}

// r.u16[i] = value.u8[i]
NLIB_M(i128) I128::ConvertFromUint8ToUint16(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepu8_epi16(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u16(vmovl_u8(vreinterpret_u8_s8(value)));
#endif
}

// r.u32[i] = value.u16[i]
NLIB_M(i128) I128::ConvertFromUint16ToUint32(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepu16_epi32(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u32(vmovl_u16(vreinterpret_u16_s8(value)));
#endif
}

// r.u64[i] = value.u32[i]
NLIB_M(i128) I128::ConvertFromUint32ToUint64(i64arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_cvtepu32_epi64(value);
#elif defined(NLIB_NEON)
    return vreinterpretq_s8_u64(vmovl_u32(vreinterpret_u32_s8(value)));
#endif
}

// 01234567 abcdefgh -> 0a1b2c3d4e5f6g7h
NLIB_M(i128) I128::Zip8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_unpacklo_epi8(a, b);
#elif defined(NLIB_NEON)
    int8x8x2_t tmp = vzip_s8(a, b);
    return vcombine_s8(tmp.val[0], tmp.val[1]);
#endif
}

// 01234567 abcdefgh -> 0246aceg1357bdfh
NLIB_M(i128) I128::Unzip8(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // -> a0,b0,a1,b1,a2,b2,a3,b3, ..., a7,b7
    __m128i tmp = _mm_unpacklo_epi16(a, b);
    // a0,a2,a4,a6,b0,b2,b4,b6,a1,a3,a5,a7,b1,b3,b5,b7
    NLIB_ALIGNAS(16) static const int8_t mask[16] = {0, 4, 8, 12, 2, 6, 10, 14,
                                                     1, 5, 9, 13, 3, 7, 11, 15};
    return _mm_shuffle_epi8(tmp, *reinterpret_cast<const __m128i*>(&mask[0]));
#elif defined(NLIB_NEON)
    int8x8x2_t tmp = vuzp_s8(a, b);
    return vcombine_s8(tmp.val[0], tmp.val[1]);
#endif
}

// 0123 abcd -> 0a1b2c3d
NLIB_M(i128) I128::Zip16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_unpacklo_epi16(a, b);
#elif defined(NLIB_NEON)
    uint16x4x2_t tmp = vzip_u16(vreinterpret_u16_s8(a), vreinterpret_u16_s8(b));
    return NLIB_CMB(u16, tmp.val[0], tmp.val[1]);
#endif
}

// 0123 abcd -> 02ac13bd
NLIB_M(i128) I128::Unzip16(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    // a0,a1,a2,a3,b0,b1,b2,b3 -> a0,b0,a1,b1,a2,b2,a3,b3
    __m128i tmp = _mm_unpacklo_epi16(a, b);
    // a0,a2,b0,b2,a1,b3,b1,b3
    NLIB_ALIGNAS(16) static const int8_t mask[16] = {0, 1, 8,  9,  2, 3, 10, 11,
                                                     4, 5, 12, 13, 6, 7, 14, 15};
    return _mm_shuffle_epi8(tmp, *reinterpret_cast<const __m128i*>(&mask[0]));
#elif defined(NLIB_NEON)
    uint16x4x2_t tmp = vuzp_u16(vreinterpret_u16_s8(a), vreinterpret_u16_s8(b));
    return NLIB_CMB(u16, tmp.val[0], tmp.val[1]);
#endif
}

// 01 ab -> 0a1b
NLIB_M(i128) I128::Zip32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_unpacklo_epi32(a, b);
#elif defined(NLIB_NEON)
    uint32x2x2_t tmp = vuzp_u32(vreinterpret_u32_s8(a), vreinterpret_u32_s8(b));
    return NLIB_CMB(u32, tmp.val[0], tmp.val[1]);
#endif
}

// 01 ab -> 0a1b
NLIB_M(i128) I128::Unzip32(i64arg a, i64arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_unpacklo_epi32(a, b);
#elif defined(NLIB_NEON)
    uint32x2x2_t tmp = vuzp_u32(vreinterpret_u32_s8(a), vreinterpret_u32_s8(b));
    return NLIB_CMB(u32, tmp.val[0], tmp.val[1]);
#endif
}

// 0123456789abcdef -> 1032547698badcfe
NLIB_M(i128) I128::Reverse16(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask_[16] = {
        1, 0, 3, 2, 5, 4, 7, 6, 9, 8, 11, 10, 13, 12, 15, 14
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask_[0]));
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev16q, u8, value);
#endif
}

// 0123456789abcdef -> 32107654ba98fedc
NLIB_M(i128) I128::Reverse32(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask_[16] = {
        3, 2, 1, 0, 7, 6, 5, 4, 11, 10, 9, 8, 15, 14, 13, 12
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask_[0]));
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev32q, u8, value);
#endif
}

// 0123456789abcdef -> 76543210fedcba98
NLIB_M(i128) I128::Reverse64(i128arg value) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    NLIB_ALIGNAS(16) const int8_t mask_[16] = {
        7, 6, 5, 4, 3, 2, 1, 0, 15, 14, 13, 12, 11, 10, 9, 8
    };
    return _mm_shuffle_epi8(value, *reinterpret_cast<const __m128i*>(&mask_[0]));
#elif defined(NLIB_NEON)
    return NLIB_OP1(vrev64q, u8, value);
#endif
}

// r = Or(ismasked(value.u8[i]) ? (1 << i) : 0)
NLIB_M(int) I128::MoveMask8(i128arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    return _mm_movemask_epi8(value);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    NLIB_ALIGNAS(16) static const uint64_t powers_[2] = {0x8040201008040201ULL,
                                                         0x8040201008040201ULL};
    uint64x2_t pwrtmp = vld1q_u64(powers_);
    uint8x16_t powers = vreinterpretq_u8_u64(pwrtmp);
    uint8x16_t a = vandq_u8(vreinterpretq_u8_s8(value), powers);
    uint8x16_t tmp = vpaddq_u8(a, a);
    tmp = vpaddq_u8(tmp, tmp);
    tmp = vpaddq_u8(tmp, tmp);
    return vgetq_lane_u16(vreinterpretq_u16_u8(tmp), 0);
# else
    NLIB_ALIGNAS(16) static const uint64_t powers_[2] = {0x8040201008040201ULL,
                                                         0x8040201008040201ULL};
    uint64x2_t pwrtmp = vld1q_u64(powers_);  // for NEONvsSSE
    uint8x16_t powers = vreinterpretq_u8_u64(pwrtmp);
    uint8x16_t a = vandq_u8(vreinterpretq_u8_s8(value), powers);
    uint8x8_t al = vget_low_u8(a);
    uint8x8_t ah = vget_high_u8(a);
    uint8x8_t tmp = vpadd_u8(al, ah);
    tmp = vpadd_u8(tmp, tmp);
    tmp = vpadd_u8(tmp, tmp);
    return vget_lane_u16(vreinterpret_u16_u8(tmp), 0);
# endif
#endif
}

// r = Or(ismasked(value.u16[i]) ? (1 << i) : 0)
NLIB_M(int) I128::MoveMask16(i128arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    __m128i tmp = _mm_packs_epi16(value, value);
    return _mm_movemask_epi8(tmp) & 255;
#elif defined(NLIB_NEON)
    NLIB_ALIGNAS(16) static const uint64_t powers_[2] = {0x0008000400020001ULL,
                                                         0x0080004000200010ULL};
    uint64x2_t pwrtmp = vld1q_u64(powers_);  // for NEONvsSSE
    uint16x8_t powers = vreinterpretq_u16_u64(pwrtmp);
    uint16x8_t a = vandq_u16(vreinterpretq_u16_s8(value), powers);
    uint8x8_t tmp = vmovn_u16(a);
    tmp = vpadd_u8(tmp, tmp);
    tmp = vpadd_u8(tmp, tmp);
    tmp = vpadd_u8(tmp, tmp);
    return vget_lane_u8(tmp, 0);
#endif
}

// r = Or(ismasked(value.u32[i]) ? (1 << i) : 0)
NLIB_M(int) I128::MoveMask32(i128arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    __m128i tmp = _mm_packs_epi16(value, value);
    tmp = _mm_packs_epi16(tmp, tmp);
    return _mm_movemask_epi8(tmp) & 15;
#elif defined(NLIB_NEON)
    NLIB_ALIGNAS(16) static const uint64_t powers_[2] = {0x0000000200000001ULL,
                                                         0x0000000800000004ULL};
    uint64x2_t pwrtmp = vld1q_u64(powers_);  // for NEONvsSSE
    uint32x4_t powers = vreinterpretq_u32_u64(pwrtmp);
    uint32x4_t a = vandq_u32(vreinterpretq_u32_s8(value), powers);
    uint16x4_t tmp = vmovn_u32(a);
    tmp = vpadd_u16(tmp, tmp);
    tmp = vpadd_u16(tmp, tmp);
    return vget_lane_u8(vreinterpret_u8_u16(tmp), 0);
#endif
}

// true if value == 0
NLIB_M(bool) I128::IsZero(i128arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    return _mm_testz_si128(value, value) != 0;
#elif defined(NLIB_NEON)
    int8x8_t tmp = vorr_s8(vget_low_s8(value), vget_high_s8(value));
    return vget_lane_u64(vreinterpret_u64_s8(tmp), 0) == 0;
#endif
}

// true if all bits on
NLIB_M(bool) I128::IsFull(i128arg value) NLIB_NOEXCEPT {  // NOLINT
#if defined(NLIB_SSE41)
    return _mm_testc_si128(value, _mm_cmpeq_epi8(value, value)) != 0;
#elif defined(NLIB_NEON)
    int8x8_t tmp = vand_s8(vget_low_s8(value), vget_high_s8(value));
    return vget_lane_s64(vreinterpret_s64_s8(tmp), 0) == -1;
#endif
}

// r.i8[i] = ismasked(mask.i8[i]) ? a.i8[i] : b.i8[i]
NLIB_M(i128) I128::Select(i128arg mask, i128arg a, i128arg b) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_blendv_epi8(b, a, mask);
#elif defined(NLIB_NEON)
    return NLIB_OP3(vbslq, u32, mask, a, b);
#endif
}

// r[i] = 0 if shuffle.u8[i] == 0x80, r[i] = value[shuffle.u8[i]] if 0 <= shuffle.u8[i] < 16
NLIB_M(i128) I128::Shuffle8(i128arg value, i128arg shuffle) NLIB_NOEXCEPT {
#if defined(NLIB_SSE41)
    return _mm_shuffle_epi8(value, shuffle);
#elif defined(NLIB_NEON)
# ifdef __aarch64__
    return vqtbl1q_s8(value, vreinterpretq_u8_s8(shuffle));
# else
    int8x8x2_t x;
    x.val[0] = vget_low_s8(value);
    x.val[1] = vget_high_s8(value);
    int8x8_t lo = vtbl2_s8(x, vget_low_s8(shuffle));
    int8x8_t hi = vtbl2_s8(x, vget_high_s8(shuffle));
    return vcombine_s8(lo, hi);
# endif
#endif
}

#ifdef NLIB_NEON
# undef vreinterpretq_s8_s8
# undef NLIB_OP1
# undef NLIB_OP2
# undef NLIB_OP3
# undef NLIB_CMP
# undef NLIB_SFT
# undef NLIB_CMB
#endif

#endif  // NLIB_DOXYGEN

#undef NLIB_M
#undef NLIB_M2

#if defined(NLIB_SSE41)
#define NLIB_I128_TRANSPOSE32(row0, row1, row2, row3)                \
    {                                                                \
        row0 = _mm_shuffle_epi32(row0, _MM_SHUFFLE(3, 1, 2, 0));     \
        row1 = _mm_shuffle_epi32(row1, _MM_SHUFFLE(3, 1, 2, 0));     \
        row2 = _mm_shuffle_epi32(row2, _MM_SHUFFLE(3, 1, 2, 0));     \
        row3 = _mm_shuffle_epi32(row3, _MM_SHUFFLE(3, 1, 2, 0));     \
        __m128i t0_transpose32_ = _mm_unpacklo_epi32(row0, row1);    \
        __m128i t1_transpose32_ = _mm_unpackhi_epi32(row0, row1);    \
        __m128i t2_transpose32_ = _mm_unpacklo_epi32(row2, row3);    \
        __m128i t3_transpose32_ = _mm_unpackhi_epi32(row2, row3);    \
        row0 = _mm_unpacklo_epi64(t0_transpose32_, t2_transpose32_); \
        row1 = _mm_unpacklo_epi64(t1_transpose32_, t3_transpose32_); \
        row2 = _mm_unpackhi_epi64(t0_transpose32_, t2_transpose32_); \
        row3 = _mm_unpackhi_epi64(t1_transpose32_, t3_transpose32_); \
}
#elif defined(NLIB_NEON)
#define NLIB_I128_TRANSPOSE32(row0, row1, row2, row3)                           \
    {                                                                           \
        uint32x4x2_t trn_f0_ = vtrnq_u32(vreinterpretq_u32_s8(row0),            \
                                         vreinterpretq_u32_s8(row1));           \
        uint32x4x2_t trn_f1_ = vtrnq_u32(vreinterpretq_u32_s8(row2),            \
                                         vreinterpretq_u32_s8(row3));           \
        uint32x4_t row0_, row1_, row2_, row3_;                                  \
        uint32x2_t lo, hi;                                                      \
        lo = vget_low_u32(trn_f0_.val[0]); hi = vget_low_u32(trn_f1_.val[0]);   \
        row0_ = vcombine_u32(lo, hi);                                           \
        row0 = vreinterpretq_s8_u32(row0_);                                     \
        lo = vget_low_u32(trn_f0_.val[1]); hi = vget_low_u32(trn_f1_.val[1]);   \
        row1_ = vcombine_u32(lo, hi);                                           \
        row1 = vreinterpretq_s8_u32(row1_);                                     \
        lo = vget_high_u32(trn_f0_.val[0]); hi = vget_high_u32(trn_f1_.val[0]); \
        row2_ = vcombine_u32(lo, hi);                                           \
        row2 = vreinterpretq_s8_u32(row2_);                                     \
        lo = vget_high_u32(trn_f0_.val[1]); hi = vget_high_u32(trn_f1_.val[1]); \
        row3_ = vcombine_u32(lo, hi);                                           \
        row3 = vreinterpretq_s8_u32(row3_);                                     \
    }
#endif

#endif  // NLIB_SIMD

}  // namespace simd
NLIB_NAMESPACE_END

#endif  // INCLUDE_NN_NLIB_SIMD_SIMDINT_H_
